@@ -16,6 +16,8 @@
  *
  */
 
+#include "src/compiler/python_generator.h"
+
 #include <algorithm>
 #include <cassert>
 #include <cctype>
@@ -33,7 +35,6 @@
 #include "src/compiler/config.h"
 #include "src/compiler/generator_helpers.h"
 #include "src/compiler/protobuf_plugin.h"
-#include "src/compiler/python_generator.h"
 #include "src/compiler/python_generator_helpers.h"
 #include "src/compiler/python_private_generator.h"
 
@@ -51,13 +52,13 @@ using std::vector;
 
 namespace grpc_python_generator {
 
-grpc::string generator_file_name;
+std::string generator_file_name;
 
 namespace {
 
-typedef map<grpc::string, grpc::string> StringMap;
-typedef vector<grpc::string> StringVector;
-typedef tuple<grpc::string, grpc::string> StringPair;
+typedef map<std::string, std::string> StringMap;
+typedef vector<std::string> StringVector;
+typedef tuple<std::string, std::string> StringPair;
 typedef set<StringPair> StringPairSet;
 
 // Provides RAII indentation handling. Use as:
@@ -107,7 +108,7 @@ void PrivateGenerator::PrintAllComments(StringVector comments,
   for (StringVector::iterator it = comments.begin(); it != comments.end();
        ++it) {
     size_t start_pos = it->find_first_not_of(' ');
-    if (start_pos != grpc::string::npos) {
+    if (start_pos != std::string::npos) {
       out->PrintRaw(it->c_str() + start_pos);
     }
     out->Print("\n");
@@ -134,7 +135,7 @@ bool PrivateGenerator::PrintBetaServicer(const grpc_generator::Service* service,
     PrintAllComments(service_comments, out);
     for (int i = 0; i < service->method_count(); ++i) {
       auto method = service->method(i);
-      grpc::string arg_name =
+      std::string arg_name =
           method->ClientStreaming() ? "request_iterator" : "request";
       StringMap method_dict;
       method_dict["Method"] = method->name();
@@ -170,7 +171,7 @@ bool PrivateGenerator::PrintBetaStub(const grpc_generator::Service* service,
     PrintAllComments(service_comments, out);
     for (int i = 0; i < service->method_count(); ++i) {
       auto method = service->method(i);
-      grpc::string arg_name =
+      std::string arg_name =
           method->ClientStreaming() ? "request_iterator" : "request";
       StringMap method_dict;
       method_dict["Method"] = method->name();
@@ -193,7 +194,7 @@ bool PrivateGenerator::PrintBetaStub(const grpc_generator::Service* service,
 }
 
 bool PrivateGenerator::PrintBetaServerFactory(
-    const grpc::string& package_qualified_service_name,
+    const std::string& package_qualified_service_name,
     const grpc_generator::Service* service, grpc_generator::Printer* out) {
   StringMap service_dict;
   service_dict["Service"] = service->name();
@@ -214,18 +215,18 @@ bool PrivateGenerator::PrintBetaServerFactory(
     StringMap output_message_modules_and_classes;
     for (int i = 0; i < service->method_count(); ++i) {
       auto method = service->method(i);
-      const grpc::string method_implementation_constructor =
-          grpc::string(method->ClientStreaming() ? "stream_" : "unary_") +
-          grpc::string(method->ServerStreaming() ? "stream_" : "unary_") +
+      const std::string method_implementation_constructor =
+          std::string(method->ClientStreaming() ? "stream_" : "unary_") +
+          std::string(method->ServerStreaming() ? "stream_" : "unary_") +
           "inline";
-      grpc::string input_message_module_and_class;
+      std::string input_message_module_and_class;
       if (!method->get_module_and_message_path_input(
               &input_message_module_and_class, generator_file_name,
               generate_in_pb2_grpc, config.import_prefix,
               config.prefixes_to_filter)) {
         return false;
       }
-      grpc::string output_message_module_and_class;
+      std::string output_message_module_and_class;
       if (!method->get_module_and_message_path_output(
               &output_message_module_and_class, generator_file_name,
               generate_in_pb2_grpc, config.import_prefix,
@@ -280,7 +281,7 @@ bool PrivateGenerator::PrintBetaServerFactory(
       method_dict["Method"] = name_and_implementation_constructor->first;
       method_dict["Constructor"] = name_and_implementation_constructor->second;
       IndentScope raii_descriptions_indent(out);
-      const grpc::string method_name =
+      const std::string method_name =
           name_and_implementation_constructor->first;
       out->Print(method_dict,
                  "(\'$PackageQualifiedServiceName$\', \'$Method$\'): "
@@ -302,7 +303,7 @@ bool PrivateGenerator::PrintBetaServerFactory(
 }
 
 bool PrivateGenerator::PrintBetaStubFactory(
-    const grpc::string& package_qualified_service_name,
+    const std::string& package_qualified_service_name,
     const grpc_generator::Service* service, grpc_generator::Printer* out) {
   StringMap dict;
   dict["Service"] = service->name();
@@ -323,17 +324,17 @@ bool PrivateGenerator::PrintBetaStubFactory(
     StringMap output_message_modules_and_classes;
     for (int i = 0; i < service->method_count(); ++i) {
       auto method = service->method(i);
-      const grpc::string method_cardinality =
-          grpc::string(method->ClientStreaming() ? "STREAM" : "UNARY") + "_" +
-          grpc::string(method->ServerStreaming() ? "STREAM" : "UNARY");
-      grpc::string input_message_module_and_class;
+      const std::string method_cardinality =
+          std::string(method->ClientStreaming() ? "STREAM" : "UNARY") + "_" +
+          std::string(method->ServerStreaming() ? "STREAM" : "UNARY");
+      std::string input_message_module_and_class;
       if (!method->get_module_and_message_path_input(
               &input_message_module_and_class, generator_file_name,
               generate_in_pb2_grpc, config.import_prefix,
               config.prefixes_to_filter)) {
         return false;
       }
-      grpc::string output_message_module_and_class;
+      std::string output_message_module_and_class;
       if (!method->get_module_and_message_path_output(
               &output_message_module_and_class, generator_file_name,
               generate_in_pb2_grpc, config.import_prefix,
@@ -406,7 +407,7 @@ bool PrivateGenerator::PrintBetaStubFactory(
 }
 
 bool PrivateGenerator::PrintStub(
-    const grpc::string& package_qualified_service_name,
+    const std::string& package_qualified_service_name,
     const grpc_generator::Service* service, grpc_generator::Printer* out) {
   StringMap dict;
   dict["Service"] = service->name();
@@ -430,17 +431,17 @@ bool PrivateGenerator::PrintStub(
       out->Print("\"\"\"\n");
       for (int i = 0; i < service->method_count(); ++i) {
         auto method = service->method(i);
-        grpc::string multi_callable_constructor =
-            grpc::string(method->ClientStreaming() ? "stream" : "unary") + "_" +
-            grpc::string(method->ServerStreaming() ? "stream" : "unary");
-        grpc::string request_module_and_class;
+        std::string multi_callable_constructor =
+            std::string(method->ClientStreaming() ? "stream" : "unary") + "_" +
+            std::string(method->ServerStreaming() ? "stream" : "unary");
+        std::string request_module_and_class;
         if (!method->get_module_and_message_path_input(
                 &request_module_and_class, generator_file_name,
                 generate_in_pb2_grpc, config.import_prefix,
                 config.prefixes_to_filter)) {
           return false;
         }
-        grpc::string response_module_and_class;
+        std::string response_module_and_class;
         if (!method->get_module_and_message_path_output(
                 &response_module_and_class, generator_file_name,
                 generate_in_pb2_grpc, config.import_prefix,
@@ -466,7 +467,7 @@ bool PrivateGenerator::PrintStub(
           out->Print(
               method_dict,
               "response_deserializer=$ResponseModuleAndClass$.FromString,\n");
-          out->Print(")\n");
+          out->Print("_registered_method=True)\n");
         }
       }
     }
@@ -486,7 +487,7 @@ bool PrivateGenerator::PrintServicer(const grpc_generator::Service* service,
     PrintAllComments(service_comments, out);
     for (int i = 0; i < service->method_count(); ++i) {
       auto method = service->method(i);
-      grpc::string arg_name =
+      std::string arg_name =
           method->ClientStreaming() ? "request_iterator" : "request";
       StringMap method_dict;
       method_dict["Method"] = method->name();
@@ -507,7 +508,7 @@ bool PrivateGenerator::PrintServicer(const grpc_generator::Service* service,
 }
 
 bool PrivateGenerator::PrintAddServicerToServer(
-    const grpc::string& package_qualified_service_name,
+    const std::string& package_qualified_service_name,
     const grpc_generator::Service* service, grpc_generator::Printer* out) {
   StringMap service_dict;
   service_dict["Service"] = service->name();
@@ -522,18 +523,18 @@ bool PrivateGenerator::PrintAddServicerToServer(
       IndentScope raii_dict_second_indent(out);
       for (int i = 0; i < service->method_count(); ++i) {
         auto method = service->method(i);
-        grpc::string method_handler_constructor =
-            grpc::string(method->ClientStreaming() ? "stream" : "unary") + "_" +
-            grpc::string(method->ServerStreaming() ? "stream" : "unary") +
+        std::string method_handler_constructor =
+            std::string(method->ClientStreaming() ? "stream" : "unary") + "_" +
+            std::string(method->ServerStreaming() ? "stream" : "unary") +
             "_rpc_method_handler";
-        grpc::string request_module_and_class;
+        std::string request_module_and_class;
         if (!method->get_module_and_message_path_input(
                 &request_module_and_class, generator_file_name,
                 generate_in_pb2_grpc, config.import_prefix,
                 config.prefixes_to_filter)) {
           return false;
         }
-        grpc::string response_module_and_class;
+        std::string response_module_and_class;
         if (!method->get_module_and_message_path_output(
                 &response_module_and_class, generator_file_name,
                 generate_in_pb2_grpc, config.import_prefix,
@@ -573,6 +574,9 @@ bool PrivateGenerator::PrintAddServicerToServer(
                  "'$PackageQualifiedServiceName$', rpc_method_handlers)\n");
     }
     out->Print("server.add_generic_rpc_handlers((generic_handler,))\n");
+    out->Print(method_dict,
+               "server.add_registered_method_handlers('$"
+               "PackageQualifiedServiceName$', rpc_method_handlers)\n");
   }
   return true;
 }
@@ -583,7 +587,7 @@ bool PrivateGenerator::PrintAddServicerToServer(
  * should never be instantiated.
  */
 bool PrivateGenerator::PrintServiceClass(
-    const grpc::string& package_qualified_service_name,
+    const std::string& package_qualified_service_name,
     const grpc_generator::Service* service, grpc_generator::Printer* out) {
   StringMap dict;
   dict["Service"] = service->name();
@@ -596,14 +600,14 @@ bool PrivateGenerator::PrintServiceClass(
     PrintAllComments(service_comments, out);
     for (int i = 0; i < service->method_count(); ++i) {
       const auto& method = service->method(i);
-      grpc::string request_module_and_class;
+      std::string request_module_and_class;
       if (!method->get_module_and_message_path_input(
               &request_module_and_class, generator_file_name,
               generate_in_pb2_grpc, config.import_prefix,
               config.prefixes_to_filter)) {
         return false;
       }
-      grpc::string response_module_and_class;
+      std::string response_module_and_class;
       if (!method->get_module_and_message_path_output(
               &response_module_and_class, generator_file_name,
               generate_in_pb2_grpc, config.import_prefix,
@@ -615,7 +619,7 @@ bool PrivateGenerator::PrintServiceClass(
       method_dict["Method"] = method->name();
       out->Print("@staticmethod\n");
       out->Print(method_dict, "def $Method$(");
-      grpc::string request_parameter(
+      std::string request_parameter(
           method->ClientStreaming() ? "request_iterator" : "request");
       StringMap args_dict;
       args_dict["RequestParameter"] = request_parameter;
@@ -627,6 +631,7 @@ bool PrivateGenerator::PrintServiceClass(
         out->Print("options=(),\n");
         out->Print("channel_credentials=None,\n");
         out->Print("call_credentials=None,\n");
+        out->Print("insecure=False,\n");
         out->Print("compression=None,\n");
         out->Print("wait_for_ready=None,\n");
         out->Print("timeout=None,\n");
@@ -634,28 +639,33 @@ bool PrivateGenerator::PrintServiceClass(
       }
       {
         IndentScope method_indent(out);
-        grpc::string arity_method_name =
-            grpc::string(method->ClientStreaming() ? "stream" : "unary") + "_" +
-            grpc::string(method->ServerStreaming() ? "stream" : "unary");
+        std::string arity_method_name =
+            std::string(method->ClientStreaming() ? "stream" : "unary") + "_" +
+            std::string(method->ServerStreaming() ? "stream" : "unary");
         args_dict["ArityMethodName"] = arity_method_name;
         args_dict["PackageQualifiedService"] = package_qualified_service_name;
         args_dict["Method"] = method->name();
-        out->Print(args_dict,
-                   "return "
-                   "grpc.experimental.$ArityMethodName$($RequestParameter$, "
-                   "target, '/$PackageQualifiedService$/$Method$',\n");
+        out->Print(args_dict, "return grpc.experimental.$ArityMethodName$(\n");
         {
           IndentScope continuation_indent(out);
           StringMap serializer_dict;
+          out->Print(args_dict, "$RequestParameter$,\n");
+          out->Print("target,\n");
+          out->Print(args_dict, "'/$PackageQualifiedService$/$Method$',\n");
           serializer_dict["RequestModuleAndClass"] = request_module_and_class;
           serializer_dict["ResponseModuleAndClass"] = response_module_and_class;
           out->Print(serializer_dict,
                      "$RequestModuleAndClass$.SerializeToString,\n");
           out->Print(serializer_dict, "$ResponseModuleAndClass$.FromString,\n");
-          out->Print("options, channel_credentials,\n");
-          out->Print(
-              "call_credentials, compression, wait_for_ready, timeout, "
-              "metadata)\n");
+          out->Print("options,\n");
+          out->Print("channel_credentials,\n");
+          out->Print("insecure,\n");
+          out->Print("call_credentials,\n");
+          out->Print("compression,\n");
+          out->Print("wait_for_ready,\n");
+          out->Print("timeout,\n");
+          out->Print("metadata,\n");
+          out->Print("_registered_method=True)\n");
         }
       }
     }
@@ -681,6 +691,9 @@ bool PrivateGenerator::PrintPreamble(grpc_generator::Printer* out) {
   StringMap var;
   var["Package"] = config.grpc_package_root;
   out->Print(var, "import $Package$\n");
+  if (config.grpc_tools_version.size() > 0) {
+    out->Print(var, "import warnings\n");
+  }
   if (generate_in_pb2_grpc) {
     out->Print("\n");
     StringPairSet imports_set;
@@ -689,21 +702,21 @@ bool PrivateGenerator::PrintPreamble(grpc_generator::Printer* out) {
       for (int j = 0; j < service->method_count(); ++j) {
         auto method = service.get()->method(j);
 
-        grpc::string input_type_file_name = method->get_input_type_name();
-        grpc::string input_module_name =
+        std::string input_type_file_name = method->get_input_type_name();
+        std::string input_module_name =
             ModuleName(input_type_file_name, config.import_prefix,
                        config.prefixes_to_filter);
-        grpc::string input_module_alias =
+        std::string input_module_alias =
             ModuleAlias(input_type_file_name, config.import_prefix,
                         config.prefixes_to_filter);
         imports_set.insert(
             std::make_tuple(input_module_name, input_module_alias));
 
-        grpc::string output_type_file_name = method->get_output_type_name();
-        grpc::string output_module_name =
+        std::string output_type_file_name = method->get_output_type_name();
+        std::string output_module_name =
             ModuleName(output_type_file_name, config.import_prefix,
                        config.prefixes_to_filter);
-        grpc::string output_module_alias =
+        std::string output_module_alias =
             ModuleAlias(output_type_file_name, config.import_prefix,
                         config.prefixes_to_filter);
         imports_set.insert(
@@ -716,7 +729,7 @@ bool PrivateGenerator::PrintPreamble(grpc_generator::Printer* out) {
       auto module_name = std::get<0>(*it);
       var["ModuleAlias"] = std::get<1>(*it);
       const size_t last_dot_pos = module_name.rfind('.');
-      if (last_dot_pos == grpc::string::npos) {
+      if (last_dot_pos == std::string::npos) {
         var["ImportStatement"] = "import " + module_name;
       } else {
         var["ImportStatement"] = "from " + module_name.substr(0, last_dot_pos) +
@@ -725,18 +738,68 @@ bool PrivateGenerator::PrintPreamble(grpc_generator::Printer* out) {
       }
       out->Print(var, "$ImportStatement$ as $ModuleAlias$\n");
     }
+
+    // Checks if generate code is used with a supported grpcio version.
+    if (config.grpc_tools_version.size() > 0) {
+      var["ToolsVersion"] = config.grpc_tools_version;
+      out->Print(var, "\nGRPC_GENERATED_VERSION = '$ToolsVersion$'\n");
+      out->Print("GRPC_VERSION = grpc.__version__\n");
+      out->Print("EXPECTED_ERROR_RELEASE = '1.66.0'\n");
+      out->Print("SCHEDULED_RELEASE_DATE = 'August 6, 2024'\n");
+      out->Print("_version_not_supported = False\n\n");
+      out->Print("try:\n");
+      {
+        IndentScope raii_import_indent(out);
+        out->Print(
+            "from grpc._utilities import first_version_is_lower\n"
+            "_version_not_supported = first_version_is_lower(GRPC_VERSION, "
+            "GRPC_GENERATED_VERSION)\n");
+      }
+      out->Print("except ImportError:\n");
+      {
+        IndentScope raii_import_error_indent(out);
+        out->Print("_version_not_supported = True\n");
+      }
+      out->Print("\nif _version_not_supported:\n");
+      {
+        IndentScope raii_warning_indent(out);
+        out->Print("warnings.warn(\n");
+        {
+          IndentScope raii_warning_string_indent(out);
+          std::string filename_without_ext = file->filename_without_ext();
+          std::replace(filename_without_ext.begin(), filename_without_ext.end(),
+                       '-', '_');
+          var["Pb2GrpcFileName"] = filename_without_ext;
+          out->Print(
+              var,
+              "f'The grpc package installed is at version {GRPC_VERSION},'\n"
+              "+ f' but the generated code in $Pb2GrpcFileName$_pb2_grpc.py "
+              "depends on'\n"
+              "+ f' grpcio>={GRPC_GENERATED_VERSION}.'\n"
+              "+ f' Please upgrade your grpc module to "
+              "grpcio>={GRPC_GENERATED_VERSION}'\n"
+              "+ f' or downgrade your generated code using "
+              "grpcio-tools<={GRPC_VERSION}.'\n"
+              "+ f' This warning will become an error in "
+              "{EXPECTED_ERROR_RELEASE},'\n"
+              "+ f' scheduled for release on {SCHEDULED_RELEASE_DATE}.',\n"
+              "RuntimeWarning\n");
+        }
+        out->Print(")\n");
+      }
+    }
   }
   return true;
 }
 
 bool PrivateGenerator::PrintGAServices(grpc_generator::Printer* out) {
-  grpc::string package = file->package();
+  std::string package = file->package();
   if (!package.empty()) {
     package = package.append(".");
   }
   for (int i = 0; i < file->service_count(); ++i) {
     auto service = file->service(i);
-    grpc::string package_qualified_service_name = package + service->name();
+    std::string package_qualified_service_name = package + service->name();
     if (!(PrintStub(package_qualified_service_name, service.get(), out) &&
           PrintServicer(service.get(), out) &&
           PrintAddServicerToServer(package_qualified_service_name,
@@ -750,13 +813,13 @@ bool PrivateGenerator::PrintGAServices(grpc_generator::Printer* out) {
 }
 
 bool PrivateGenerator::PrintBetaServices(grpc_generator::Printer* out) {
-  grpc::string package = file->package();
+  std::string package = file->package();
   if (!package.empty()) {
     package = package.append(".");
   }
   for (int i = 0; i < file->service_count(); ++i) {
     auto service = file->service(i);
-    grpc::string package_qualified_service_name = package + service->name();
+    std::string package_qualified_service_name = package + service->name();
     if (!(PrintBetaServicer(service.get(), out) &&
           PrintBetaStub(service.get(), out) &&
           PrintBetaServerFactory(package_qualified_service_name, service.get(),
@@ -769,8 +832,8 @@ bool PrivateGenerator::PrintBetaServices(grpc_generator::Printer* out) {
   return true;
 }
 
-pair<bool, grpc::string> PrivateGenerator::GetGrpcServices() {
-  grpc::string output;
+pair<bool, std::string> PrivateGenerator::GetGrpcServices() {
+  std::string output;
   {
     // Scope the output stream so it closes and finalizes output to the string.
     auto out = file->CreatePrinter(&output);
@@ -821,7 +884,14 @@ pair<bool, grpc::string> PrivateGenerator::GetGrpcServices() {
 GeneratorConfiguration::GeneratorConfiguration()
     : grpc_package_root("grpc"),
       beta_package_root("grpc.beta"),
-      import_prefix("") {}
+      import_prefix(""),
+      grpc_tools_version("") {}
+
+GeneratorConfiguration::GeneratorConfiguration(std::string version)
+    : grpc_package_root("grpc"),
+      beta_package_root("grpc.beta"),
+      import_prefix(""),
+      grpc_tools_version(version) {}
 
 PythonGrpcGenerator::PythonGrpcGenerator(const GeneratorConfiguration& config)
     : config_(config) {}
@@ -829,11 +899,11 @@ PythonGrpcGenerator::PythonGrpcGenerator(const GeneratorConfiguration& config)
 PythonGrpcGenerator::~PythonGrpcGenerator() {}
 
 static bool GenerateGrpc(GeneratorContext* context, PrivateGenerator& generator,
-                         grpc::string file_name, bool generate_in_pb2_grpc) {
+                         std::string file_name, bool generate_in_pb2_grpc) {
   bool success;
   std::unique_ptr<ZeroCopyOutputStream> output;
   std::unique_ptr<CodedOutputStream> coded_output;
-  grpc::string grpc_code;
+  std::string grpc_code;
 
   if (generate_in_pb2_grpc) {
     output.reset(context->Open(file_name));
@@ -854,11 +924,11 @@ static bool GenerateGrpc(GeneratorContext* context, PrivateGenerator& generator,
   }
 }
 
-static bool ParseParameters(const grpc::string& parameter,
-                            grpc::string* grpc_version,
-                            std::vector<grpc::string>* strip_prefixes,
-                            grpc::string* error) {
-  std::vector<grpc::string> comma_delimited_parameters;
+static bool ParseParameters(const std::string& parameter,
+                            std::string* grpc_version,
+                            std::vector<std::string>* strip_prefixes,
+                            std::string* error) {
+  std::vector<std::string> comma_delimited_parameters;
   grpc_python_generator::Split(parameter, ',', &comma_delimited_parameters);
   if (comma_delimited_parameters.size() == 1 &&
       comma_delimited_parameters[0].empty()) {
@@ -877,21 +947,17 @@ static bool ParseParameters(const grpc::string& parameter,
   return true;
 }
 
-uint64_t PythonGrpcGenerator::GetSupportedFeatures() const {
-  return FEATURE_PROTO3_OPTIONAL;
-}
-
 bool PythonGrpcGenerator::Generate(const FileDescriptor* file,
-                                   const grpc::string& parameter,
+                                   const std::string& parameter,
                                    GeneratorContext* context,
-                                   grpc::string* error) const {
+                                   std::string* error) const {
   // Get output file name.
-  grpc::string pb2_file_name;
-  grpc::string pb2_grpc_file_name;
+  std::string pb2_file_name;
+  std::string pb2_grpc_file_name;
   static const int proto_suffix_length = strlen(".proto");
   if (file->name().size() > static_cast<size_t>(proto_suffix_length) &&
       file->name().find_last_of(".proto") == file->name().size() - 1) {
-    grpc::string base =
+    std::string base =
         file->name().substr(0, file->name().size() - proto_suffix_length);
     std::replace(base.begin(), base.end(), '-', '_');
     pb2_file_name = base + "_pb2.py";
@@ -903,7 +969,7 @@ bool PythonGrpcGenerator::Generate(const FileDescriptor* file,
   generator_file_name = file->name();
 
   ProtoBufFile pbfile(file);
-  grpc::string grpc_version;
+  std::string grpc_version;
   GeneratorConfiguration extended_config(config_);
   bool success = ParseParameters(parameter, &grpc_version,
                                  &(extended_config.prefixes_to_filter), error);
