@@ -16,8 +16,8 @@
 //
 //
 
-#ifndef GRPC_CORE_LIB_CHANNEL_CHANNEL_STACK_H
-#define GRPC_CORE_LIB_CHANNEL_CHANNEL_STACK_H
+#ifndef GRPC_SRC_CORE_LIB_CHANNEL_CHANNEL_STACK_H
+#define GRPC_SRC_CORE_LIB_CHANNEL_CHANNEL_STACK_H
 
 //////////////////////////////////////////////////////////////////////////////
 // IMPORTANT NOTE:
@@ -44,8 +44,6 @@
 // it can have an effect on the call status.
 //
 
-#include <grpc/support/port_platform.h>
-
 #include <stddef.h>
 
 #include <functional>
@@ -56,13 +54,12 @@
 #include <grpc/slice.h>
 #include <grpc/status.h>
 #include <grpc/support/log.h>
+#include <grpc/support/port_platform.h>
 #include <grpc/support/time.h>
 
 #include "src/core/lib/channel/channel_args.h"
 #include "src/core/lib/channel/channel_fwd.h"
-#include "src/core/lib/channel/context.h"
 #include "src/core/lib/debug/trace.h"
-#include "src/core/lib/gpr/time_precise.h"
 #include "src/core/lib/gprpp/manual_constructor.h"
 #include "src/core/lib/gprpp/ref_counted_ptr.h"
 #include "src/core/lib/gprpp/time.h"
@@ -72,34 +69,26 @@
 #include "src/core/lib/iomgr/polling_entity.h"
 #include "src/core/lib/promise/arena_promise.h"
 #include "src/core/lib/resource_quota/arena.h"
+#include "src/core/lib/transport/call_final_info.h"
 #include "src/core/lib/transport/transport.h"
+#include "src/core/telemetry/metrics.h"
+#include "src/core/util/time_precise.h"
 
 struct grpc_channel_element_args {
   grpc_channel_stack* channel_stack;
-  const grpc_channel_args* channel_args;
+  grpc_core::ChannelArgs channel_args;
   int is_first;
   int is_last;
 };
 struct grpc_call_element_args {
   grpc_call_stack* call_stack;
   const void* server_transport_data;
-  grpc_call_context_element* context;
   const grpc_slice& path;
   gpr_cycle_counter start_time;  // Note: not populated in subchannel stack.
   grpc_core::Timestamp deadline;
   grpc_core::Arena* arena;
   grpc_core::CallCombiner* call_combiner;
 };
-struct grpc_call_stats {
-  grpc_transport_stream_stats transport_stream_stats;
-  gpr_timespec latency;  // From call creating to enqueing of received status
-};
-/// Information about the call upon completion.
-struct grpc_call_final_info {
-  grpc_call_stats stats;
-  grpc_status_code final_status = GRPC_STATUS_OK;
-  const char* error_string = nullptr;
-};
 
 // Channel filters specify:
 // 1. the amount of memory needed in the channel & call (via the sizeof_XXX
@@ -116,18 +105,6 @@ struct grpc_channel_filter {
   // See grpc_call_next_op on how to call the next element in the stack
   void (*start_transport_stream_op_batch)(grpc_call_element* elem,
                                           grpc_transport_stream_op_batch* op);
-  // Create a promise to execute one call.
-  // If this is non-null, it may be used in preference to
-  // start_transport_stream_op_batch.
-  // If this is used in preference to start_transport_stream_op_batch, the
-  // following can be omitted also:
-  //   - calling init_call_elem, destroy_call_elem, set_pollset_or_pollset_set
-  //   - allocation of memory for call data
-  // There is an on-going migration to move all filters to providing this, and
-  // then to drop start_transport_stream_op_batch.
-  grpc_core::ArenaPromise<grpc_core::ServerMetadataHandle> (*make_call_promise)(
-      grpc_channel_element* elem, grpc_core::CallArgs call_args,
-      grpc_core::NextPromiseFactory next_promise_factory);
   // Called to handle channel level operations - e.g. new calls, or transport
   // closure.
   // See grpc_channel_next_op on how to call the next element in the stack
@@ -222,6 +199,10 @@ struct grpc_channel_stack {
     return event_engine->get();
   }
 
+  grpc_core::ManualConstructor<
+      grpc_core::GlobalStatsPluginRegistry::StatsPluginGroup>
+      stats_plugin_group;
+
   // Minimal infrastructure to act like a RefCounted thing without converting
   // everything.
   // It's likely that we'll want to replace grpc_channel_stack with something
@@ -229,15 +210,11 @@ struct grpc_channel_stack {
   // full C++-ification for now.
   void IncrementRefCount();
   void Unref();
+  void Unref(const grpc_core::DebugLocation& location, const char* reason);
   grpc_core::RefCountedPtr<grpc_channel_stack> Ref() {
     IncrementRefCount();
     return grpc_core::RefCountedPtr<grpc_channel_stack>(this);
   }
-
-  grpc_core::ArenaPromise<grpc_core::ServerMetadataHandle>
-  MakeClientCallPromise(grpc_core::CallArgs call_args);
-  grpc_core::ArenaPromise<grpc_core::ServerMetadataHandle>
-  MakeServerCallPromise(grpc_core::CallArgs call_args);
 };
 
 // A call stack tracks a set of related filters for one call, and guarantees
@@ -342,6 +319,11 @@ inline void grpc_channel_stack::Unref() {
   GRPC_CHANNEL_STACK_UNREF(this, "smart_pointer");
 }
 
+inline void grpc_channel_stack::Unref(const grpc_core::DebugLocation&,
+                                      const char* reason) {
+  GRPC_CHANNEL_STACK_UNREF(this, reason);
+}
+
 inline void grpc_call_stack::IncrementRefCount() {
   GRPC_CALL_STACK_REF(this, "smart_pointer");
 }
@@ -382,13 +364,11 @@ void grpc_call_log_op(const char* file, int line, gpr_log_severity severity,
 void grpc_channel_stack_no_post_init(grpc_channel_stack* stk,
                                      grpc_channel_element* elem);
 
-extern grpc_core::TraceFlag grpc_trace_channel;
-
-#define GRPC_CALL_LOG_OP(sev, elem, op)                \
-  do {                                                 \
-    if (GRPC_TRACE_FLAG_ENABLED(grpc_trace_channel)) { \
-      grpc_call_log_op(sev, elem, op);                 \
-    }                                                  \
+#define GRPC_CALL_LOG_OP(sev, elem, op)     \
+  do {                                      \
+    if (GRPC_TRACE_FLAG_ENABLED(channel)) { \
+      grpc_call_log_op(sev, elem, op);      \
+    }                                       \
   } while (0)
 
-#endif  // GRPC_CORE_LIB_CHANNEL_CHANNEL_STACK_H
+#endif  // GRPC_SRC_CORE_LIB_CHANNEL_CHANNEL_STACK_H
