@@ -12,8 +12,6 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include <grpc/support/port_platform.h>
-
 #include "src/core/lib/gprpp/crash.h"
 
 #include <stdio.h>
@@ -21,15 +19,15 @@
 
 #include <string>
 
+#include "absl/log/log.h"
 #include "absl/strings/str_cat.h"
 
-#include <grpc/support/log.h>
+#include <grpc/support/port_platform.h>
 
 namespace grpc_core {
 
 void Crash(absl::string_view message, SourceLocation location) {
-  gpr_log(location.file(), location.line(), GPR_LOG_SEVERITY_ERROR, "%s",
-          std::string(message).c_str());
+  LOG(ERROR).AtLocation(location.file(), location.line()) << message;
   abort();
 }
 
