@@ -15,18 +15,19 @@
 // limitations under the License.
 //
 //
-#include <grpc/support/port_platform.h>
-
 #include "src/core/lib/iomgr/error.h"
 
 #include <inttypes.h>
 #include <string.h>
 
+#include "absl/log/check.h"
+#include "absl/strings/str_cat.h"
 #include "absl/strings/str_format.h"
 
 #include <grpc/status.h>
 #include <grpc/support/alloc.h>
 #include <grpc/support/log.h>
+#include <grpc/support/port_platform.h>
 #include <grpc/support/string_util.h>
 
 #include "src/core/lib/gprpp/crash.h"
@@ -36,13 +37,9 @@
 #endif
 
 #include "src/core/lib/debug/trace.h"
-#include "src/core/lib/gpr/useful.h"
 #include "src/core/lib/gprpp/strerror.h"
 #include "src/core/lib/slice/slice_internal.h"
-
-grpc_core::DebugOnlyTraceFlag grpc_trace_error_refcount(false,
-                                                        "error_refcount");
-grpc_core::DebugOnlyTraceFlag grpc_trace_closure(false, "closure");
+#include "src/core/util/useful.h"
 
 absl::Status grpc_status_create(absl::StatusCode code, absl::string_view msg,
                                 const grpc_core::DebugLocation& location,
@@ -58,31 +55,56 @@ absl::Status grpc_status_create(absl::StatusCode code, absl::string_view msg,
 
 absl::Status grpc_os_error(const grpc_core::DebugLocation& location, int err,
                            const char* call_name) {
-  auto err_string = grpc_core::StrError(err);
-  absl::Status s =
-      StatusCreate(absl::StatusCode::kUnknown, err_string, location, {});
-  grpc_core::StatusSetInt(&s, grpc_core::StatusIntProperty::kErrorNo, err);
-  grpc_core::StatusSetStr(&s, grpc_core::StatusStrProperty::kOsError,
-                          err_string);
-  grpc_core::StatusSetStr(&s, grpc_core::StatusStrProperty::kSyscall,
-                          call_name);
-  return s;
+  return StatusCreate(
+      absl::StatusCode::kUnknown,
+      absl::StrCat(call_name, ": ", grpc_core::StrError(err), " (", err, ")"),
+      location, {});
 }
 
 #ifdef GPR_WINDOWS
+std::string WSAErrorToShortDescription(int err) {
+  switch (err) {
+    case WSAEACCES:
+      return "Permission denied";
+    case WSAEFAULT:
+      return "Bad address";
+    case WSAEMFILE:
+      return "Too many open files";
+    case WSAEMSGSIZE:
+      return "Message too long";
+    case WSAENETDOWN:
+      return "Network is down";
+    case WSAENETUNREACH:
+      return "Network is unreachable";
+    case WSAENETRESET:
+      return "Network dropped connection on reset";
+    case WSAECONNABORTED:
+      return "Connection aborted";
+    case WSAECONNRESET:
+      return "Connection reset";
+    case WSAETIMEDOUT:
+      return "Connection timed out";
+    case WSAECONNREFUSED:
+      return "Connection refused";
+    case WSAEHOSTUNREACH:
+      return "No route to host";
+    default:
+      return "WSA Error";
+  };
+}
 // TODO(veblush): lift out of iomgr for use in the WindowsEventEngine
 absl::Status grpc_wsa_error(const grpc_core::DebugLocation& location, int err,
                             absl::string_view call_name) {
   char* utf8_message = gpr_format_message(err);
-  absl::Status s =
-      StatusCreate(absl::StatusCode::kUnavailable, "WSA Error", location, {});
-  StatusSetInt(&s, grpc_core::StatusIntProperty::kWsaError, err);
-  StatusSetInt(&s, grpc_core::StatusIntProperty::kRpcStatus,
+  absl::Status status = StatusCreate(
+      absl::StatusCode::kUnavailable,
+      absl::StrCat(call_name, ": ", WSAErrorToShortDescription(err), " (",
+                   utf8_message, " -- ", err, ")"),
+      location, {});
+  StatusSetInt(&status, grpc_core::StatusIntProperty::kRpcStatus,
                GRPC_STATUS_UNAVAILABLE);
-  StatusSetStr(&s, grpc_core::StatusStrProperty::kOsError, utf8_message);
-  StatusSetStr(&s, grpc_core::StatusStrProperty::kSyscall, call_name);
   gpr_free(utf8_message);
-  return s;
+  return status;
 }
 #endif
 
@@ -198,7 +220,7 @@ grpc_error_handle grpc_error_add_child(grpc_error_handle src,
 
 bool grpc_log_error(const char* what, grpc_error_handle error, const char* file,
                     int line) {
-  GPR_DEBUG_ASSERT(!error.ok());
+  DCHECK(!error.ok());
   gpr_log(file, line, GPR_LOG_SEVERITY_ERROR, "%s: %s", what,
           grpc_core::StatusToString(error).c_str());
   return false;
