@@ -43,16 +43,20 @@
 
 #include <string>
 
+#include "absl/log/check.h"
+#include "absl/log/log.h"
+#include "absl/strings/str_cat.h"
+
 #include <grpc/event_engine/endpoint_config.h>
 #include <grpc/support/alloc.h>
 #include <grpc/support/log.h>
 #include <grpc/support/sync.h>
 
 #include "src/core/lib/address_utils/sockaddr_utils.h"
-#include "src/core/lib/gpr/string.h"
 #include "src/core/lib/gprpp/crash.h"
 #include "src/core/lib/gprpp/strerror.h"
 #include "src/core/lib/iomgr/sockaddr.h"
+#include "src/core/util/string.h"
 
 // set a socket to use zerocopy
 grpc_error_handle grpc_set_socket_zerocopy(int fd) {
@@ -327,12 +331,24 @@ void config_default_tcp_user_timeout(bool enable, int timeout, bool is_client) {
 }
 
 // Set TCP_USER_TIMEOUT
+// As documented in
+// https://github.com/grpc/proposal/blob/master/A18-tcp-user-timeout.md, the
+// default values for TCP_USER_TIMEOUT are currently configured to be in line
+// with the default values of KEEPALIVE_TIMEOUT as proposed in
+// https://github.com/grpc/proposal/blob/master/A18-tcp-user-timeout.md. In
+// other words, by default, TCP_USER_TIMEOUT is disabled on clients (since
+// keepalive is disabled on clients by default), and enabled on servers. To
+// override the default settings of enabling/disabling TCP_USER_TIMEOUT, the
+// value of KEEPALIVE_TIME on channel args is used. If present, a value of
+// INT_MAX means that TCP_USER_TIMEOUT would be disabled, while any other value
+// would enable TCP_USER_TIMEOUT. If TCP_USER_TIMEOUT is enabled, the default
+// setting is 20 seconds (aligning with the default of KEEPALIVE_TIMEOUT). The
+// KEEPALIVE_TIMEOUT channel arg overrides the value used for TCP_USER_TIMEOUT.
 grpc_error_handle grpc_set_socket_tcp_user_timeout(
     int fd, const grpc_core::PosixTcpOptions& options, bool is_client) {
   // Use conditionally-important parameter to avoid warning
   (void)fd;
   (void)is_client;
-  extern grpc_core::TraceFlag grpc_tcp_trace;
   if (g_socket_supports_tcp_user_timeout.load() >= 0) {
     bool enable;
     int timeout;
@@ -358,22 +374,21 @@ grpc_error_handle grpc_set_socket_tcp_user_timeout(
       // if it is available.
       if (g_socket_supports_tcp_user_timeout.load() == 0) {
         if (0 != getsockopt(fd, IPPROTO_TCP, TCP_USER_TIMEOUT, &newval, &len)) {
-          gpr_log(GPR_INFO,
-                  "TCP_USER_TIMEOUT is not available. TCP_USER_TIMEOUT won't "
-                  "be used thereafter");
+          GRPC_TRACE_LOG(tcp, INFO)
+              << "TCP_USER_TIMEOUT is not available. TCP_USER_TIMEOUT won't be "
+                 "used thereafter";
           g_socket_supports_tcp_user_timeout.store(-1);
         } else {
-          gpr_log(GPR_INFO,
-                  "TCP_USER_TIMEOUT is available. TCP_USER_TIMEOUT will be "
-                  "used thereafter");
+          GRPC_TRACE_LOG(tcp, INFO)
+              << "TCP_USER_TIMEOUT is available. TCP_USER_TIMEOUT will be used "
+                 "thereafter";
           g_socket_supports_tcp_user_timeout.store(1);
         }
       }
       if (g_socket_supports_tcp_user_timeout.load() > 0) {
-        if (GRPC_TRACE_FLAG_ENABLED(grpc_tcp_trace)) {
-          gpr_log(GPR_INFO, "Enabling TCP_USER_TIMEOUT with a timeout of %d ms",
-                  timeout);
-        }
+        GRPC_TRACE_LOG(tcp, INFO)
+            << "Enabling TCP_USER_TIMEOUT with a timeout of " << timeout
+            << " ms";
         if (0 != setsockopt(fd, IPPROTO_TCP, TCP_USER_TIMEOUT, &timeout,
                             sizeof(timeout))) {
           gpr_log(GPR_ERROR, "setsockopt(TCP_USER_TIMEOUT) %s",
@@ -386,18 +401,16 @@ grpc_error_handle grpc_set_socket_tcp_user_timeout(
           return absl::OkStatus();
         }
         if (newval != timeout) {
-          gpr_log(GPR_INFO,
-                  "Setting TCP_USER_TIMEOUT to value %d ms. Actual "
-                  "TCP_USER_TIMEOUT value is %d ms",
-                  timeout, newval);
+          GRPC_TRACE_LOG(tcp, INFO)
+              << "Setting TCP_USER_TIMEOUT to value " << timeout
+              << " ms. Actual TCP_USER_TIMEOUT value is " << newval << " ms";
           return absl::OkStatus();
         }
       }
     }
   } else {
-    if (GRPC_TRACE_FLAG_ENABLED(grpc_tcp_trace)) {
-      gpr_log(GPR_INFO, "TCP_USER_TIMEOUT not supported for this platform");
-    }
+    GRPC_TRACE_LOG(tcp, INFO)
+        << "TCP_USER_TIMEOUT not supported for this platform";
   }
   return absl::OkStatus();
 }
@@ -405,7 +418,7 @@ grpc_error_handle grpc_set_socket_tcp_user_timeout(
 // set a socket using a grpc_socket_mutator
 grpc_error_handle grpc_set_socket_with_mutator(int fd, grpc_fd_usage usage,
                                                grpc_socket_mutator* mutator) {
-  GPR_ASSERT(mutator);
+  CHECK(mutator);
   if (!grpc_socket_mutator_mutate_fd(mutator, fd, usage)) {
     return GRPC_ERROR_CREATE("grpc_socket_mutator failed.");
   }
@@ -427,7 +440,8 @@ static void probe_ipv6_once(void) {
   int fd = socket(AF_INET6, SOCK_STREAM, 0);
   g_ipv6_loopback_available = 0;
   if (fd < 0) {
-    gpr_log(GPR_INFO, "Disabling AF_INET6 sockets because socket() failed.");
+    GRPC_TRACE_LOG(tcp, INFO)
+        << "Disabling AF_INET6 sockets because socket() failed.";
   } else {
     grpc_sockaddr_in6 addr;
     memset(&addr, 0, sizeof(addr));
@@ -436,8 +450,8 @@ static void probe_ipv6_once(void) {
     if (bind(fd, reinterpret_cast<grpc_sockaddr*>(&addr), sizeof(addr)) == 0) {
       g_ipv6_loopback_available = 1;
     } else {
-      gpr_log(GPR_INFO,
-              "Disabling AF_INET6 sockets because ::1 is not available.");
+      GRPC_TRACE_LOG(tcp, INFO)
+          << "Disabling AF_INET6 sockets because ::1 is not available.";
     }
     close(fd);
   }
@@ -448,15 +462,9 @@ int grpc_ipv6_loopback_available(void) {
   return g_ipv6_loopback_available;
 }
 
-static grpc_error_handle error_for_fd(int fd,
-                                      const grpc_resolved_address* addr) {
+static grpc_error_handle error_for_fd(int fd) {
   if (fd >= 0) return absl::OkStatus();
-  auto addr_str = grpc_sockaddr_to_string(addr, false);
-  grpc_error_handle err = grpc_error_set_str(
-      GRPC_OS_ERROR(errno, "socket"),
-      grpc_core::StatusStrProperty::kTargetAddress,
-      addr_str.ok() ? addr_str.value() : addr_str.status().ToString());
-  return err;
+  return GRPC_OS_ERROR(errno, "socket");
 }
 
 grpc_error_handle grpc_create_dualstack_socket(
@@ -507,7 +515,7 @@ grpc_error_handle grpc_create_dualstack_socket_using_factory(
     // If this isn't an IPv4 address, then return whatever we've got.
     if (!grpc_sockaddr_is_v4mapped(resolved_addr, nullptr)) {
       *dsmode = GRPC_DSMODE_IPV6;
-      return error_for_fd(*newfd, resolved_addr);
+      return error_for_fd(*newfd);
     }
     // Fall back to AF_INET.
     if (*newfd >= 0) {
@@ -517,7 +525,7 @@ grpc_error_handle grpc_create_dualstack_socket_using_factory(
   }
   *dsmode = family == AF_INET ? GRPC_DSMODE_IPV4 : GRPC_DSMODE_NONE;
   *newfd = create_socket(factory, family, type, protocol);
-  return error_for_fd(*newfd, resolved_addr);
+  return error_for_fd(*newfd);
 }
 
 #endif
