@@ -27,18 +27,29 @@
 
 #include <vector>
 
+#include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "absl/strings/str_cat.h"
 
 #include <grpc/event_engine/endpoint_config.h>
+#include <grpc/event_engine/event_engine.h>
+#include <grpc/event_engine/memory_allocator.h>
 #include <grpc/support/alloc.h>
-#include <grpc/support/log.h>
 #include <grpc/support/log_windows.h>
 #include <grpc/support/string_util.h>
 #include <grpc/support/sync.h>
 #include <grpc/support/time.h>
 
 #include "src/core/lib/address_utils/sockaddr_utils.h"
+#include "src/core/lib/event_engine/memory_allocator_factory.h"
+#include "src/core/lib/event_engine/resolved_address_internal.h"
+#include "src/core/lib/event_engine/tcp_socket_utils.h"
+#include "src/core/lib/event_engine/windows/windows_engine.h"
+#include "src/core/lib/event_engine/windows/windows_listener.h"
 #include "src/core/lib/gprpp/crash.h"
+#include "src/core/lib/iomgr/closure.h"
+#include "src/core/lib/iomgr/event_engine_shims/closure.h"
+#include "src/core/lib/iomgr/event_engine_shims/endpoint.h"
 #include "src/core/lib/iomgr/iocp_windows.h"
 #include "src/core/lib/iomgr/pollset_windows.h"
 #include "src/core/lib/iomgr/resolve_address.h"
@@ -47,22 +58,42 @@
 #include "src/core/lib/iomgr/tcp_server.h"
 #include "src/core/lib/iomgr/tcp_windows.h"
 #include "src/core/lib/resource_quota/api.h"
+#include "src/core/lib/resource_quota/resource_quota.h"
 #include "src/core/lib/slice/slice_internal.h"
 
 #define MIN_SAFE_ACCEPT_QUEUE_SIZE 100
 
+namespace {
+using ::grpc_event_engine::experimental::CreateResolvedAddress;
 using ::grpc_event_engine::experimental::EndpointConfig;
+using ::grpc_event_engine::experimental::EventEngine;
+using ::grpc_event_engine::experimental::grpc_event_engine_endpoint_create;
+using ::grpc_event_engine::experimental::MemoryAllocator;
+using ::grpc_event_engine::experimental::MemoryQuotaBasedMemoryAllocatorFactory;
+using ::grpc_event_engine::experimental::ResolvedAddressSetPort;
+using ::grpc_event_engine::experimental::RunEventEngineClosure;
+using ::grpc_event_engine::experimental::WindowsEventEngine;
+using ::grpc_event_engine::experimental::WindowsEventEngineListener;
+}  // namespace
 
 // one listening port
 typedef struct grpc_tcp_listener grpc_tcp_listener;
 struct grpc_tcp_listener {
+  // Buffer to hold the local and remote address.
   // This seemingly magic number comes from AcceptEx's documentation. each
   // address buffer needs to have at least 16 more bytes at their end.
+#ifdef GRPC_HAVE_UNIX_SOCKET
+  // unix addr is larger than ip addr.
+  uint8_t addresses[(sizeof(sockaddr_un) + 16) * 2] = {};
+#else
   uint8_t addresses[(sizeof(grpc_sockaddr_in6) + 16) * 2];
+#endif  // GRPC_HAVE_UNIX_SOCKET
   // This will hold the socket for the next accept.
   SOCKET new_socket;
   // The listener winsocket.
   grpc_winsocket* socket;
+  // address of listener
+  grpc_resolved_address resolved_addr;
   // The actual TCP port number.
   int port;
   unsigned port_index;
@@ -98,12 +129,44 @@ struct grpc_tcp_server {
 
   // shutdown callback
   grpc_closure* shutdown_complete;
+
+  // used for the EventEngine shim
+  WindowsEventEngineListener* ee_listener;
 };
 
+// TODO(hork): This may be refactored to share with posix engine and event
+// engine.
+void unlink_if_unix_domain_socket(const grpc_resolved_address* resolved_addr) {
+#ifdef GRPC_HAVE_UNIX_SOCKET
+  const grpc_sockaddr* addr =
+      reinterpret_cast<const grpc_sockaddr*>(resolved_addr->addr);
+  if (addr->sa_family != AF_UNIX) {
+    return;
+  }
+  struct sockaddr_un* un =
+      reinterpret_cast<struct sockaddr_un*>(const_cast<sockaddr*>(addr));
+  // There is nothing to unlink for an abstract unix socket.
+  if (un->sun_path[0] == '\0' && un->sun_path[1] != '\0') {
+    return;
+  }
+  // For windows we need to remove the file instead of unlink.
+  DWORD attr = ::GetFileAttributesA(un->sun_path);
+  if (attr == INVALID_FILE_ATTRIBUTES) {
+    return;
+  }
+  if (attr & FILE_ATTRIBUTE_DIRECTORY || attr & FILE_ATTRIBUTE_READONLY) {
+    return;
+  }
+  ::DeleteFileA(un->sun_path);
+#else
+  (void)resolved_addr;
+#endif
+}
+
 // Public function. Allocates the proper data structures to hold a
 // grpc_tcp_server.
 static grpc_error_handle tcp_server_create(grpc_closure* shutdown_complete,
-                                           const EndpointConfig& config,
+                                           const EndpointConfig& /* config */,
                                            grpc_tcp_server_cb on_accept_cb,
                                            void* on_accept_cb_arg,
                                            grpc_tcp_server** server) {
@@ -122,7 +185,7 @@ static grpc_error_handle tcp_server_create(grpc_closure* shutdown_complete,
   return absl::OkStatus();
 }
 
-static void destroy_server(void* arg, grpc_error_handle error) {
+static void destroy_server(void* arg, grpc_error_handle /* error */) {
   grpc_tcp_server* s = (grpc_tcp_server*)arg;
 
   // Now that the accepts have been aborted, we can destroy the sockets.
@@ -133,6 +196,7 @@ static void destroy_server(void* arg, grpc_error_handle error) {
     s->head = sp->next;
     sp->next = NULL;
     grpc_winsocket_destroy(sp->socket);
+    unlink_if_unix_domain_socket(&sp->resolved_addr);
     gpr_free(sp);
   }
   gpr_mu_destroy(&s->mu);
@@ -197,12 +261,15 @@ static grpc_error_handle prepare_socket(SOCKET sock,
   grpc_resolved_address sockname_temp;
   grpc_error_handle error;
   int sockname_temp_len;
-
-  error = grpc_tcp_prepare_socket(sock);
+  if (grpc_sockaddr_get_family(addr) == AF_UNIX) {
+    error = grpc_tcp_set_non_block(sock);
+  } else {
+    error = grpc_tcp_prepare_socket(sock);
+  }
   if (!error.ok()) {
     goto failure;
   }
-
+  unlink_if_unix_domain_socket(addr);
   if (bind(sock, (const grpc_sockaddr*)addr->addr, (int)addr->len) ==
       SOCKET_ERROR) {
     error = GRPC_WSA_ERROR(WSAGetLastError(), "bind");
@@ -226,22 +293,17 @@ static grpc_error_handle prepare_socket(SOCKET sock,
   return absl::OkStatus();
 
 failure:
-  GPR_ASSERT(!error.ok());
-  auto addr_uri = grpc_sockaddr_to_uri(addr);
-  grpc_error_set_int(
-      grpc_error_set_str(
-          GRPC_ERROR_CREATE_REFERENCING("Failed to prepare server socket",
-                                        &error, 1),
-          grpc_core::StatusStrProperty::kTargetAddress,
-          addr_uri.ok() ? *addr_uri : addr_uri.status().ToString()),
-      grpc_core::StatusIntProperty::kFd, (intptr_t)sock);
+  CHECK(!error.ok());
+  error = grpc_error_set_int(GRPC_ERROR_CREATE_REFERENCING(
+                                 "Failed to prepare server socket", &error, 1),
+                             grpc_core::StatusIntProperty::kFd, (intptr_t)sock);
   if (sock != INVALID_SOCKET) closesocket(sock);
   return error;
 }
 
 static void decrement_active_ports_and_notify_locked(grpc_tcp_listener* sp) {
   sp->shutting_down = 0;
-  GPR_ASSERT(sp->server->active_ports > 0);
+  CHECK_GT(sp->server->active_ports, 0u);
   if (0 == --sp->server->active_ports) {
     finish_shutdown_locked(sp->server);
   }
@@ -252,22 +314,28 @@ static void decrement_active_ports_and_notify_locked(grpc_tcp_listener* sp) {
 static grpc_error_handle start_accept_locked(grpc_tcp_listener* port) {
   SOCKET sock = INVALID_SOCKET;
   BOOL success;
-  DWORD addrlen = sizeof(grpc_sockaddr_in6) + 16;
+  const DWORD addrlen = sizeof(port->addresses) / 2;
   DWORD bytes_received = 0;
   grpc_error_handle error;
 
   if (port->shutting_down) {
     return absl::OkStatus();
   }
-
-  sock = WSASocket(AF_INET6, SOCK_STREAM, IPPROTO_TCP, NULL, 0,
+  const int addr_family =
+      grpc_sockaddr_get_family(&port->resolved_addr) == AF_UNIX ? AF_UNIX
+                                                                : AF_INET6;
+  const int protocol = addr_family == AF_UNIX ? 0 : IPPROTO_TCP;
+  sock = WSASocket(addr_family, SOCK_STREAM, protocol, NULL, 0,
                    grpc_get_default_wsa_socket_flags());
   if (sock == INVALID_SOCKET) {
     error = GRPC_WSA_ERROR(WSAGetLastError(), "WSASocket");
     goto failure;
   }
-
-  error = grpc_tcp_prepare_socket(sock);
+  if (addr_family == AF_UNIX) {
+    error = grpc_tcp_set_non_block(sock);
+  } else {
+    error = grpc_tcp_prepare_socket(sock);
+  }
   if (!error.ok()) goto failure;
 
   // Start the "accept" asynchronously.
@@ -293,7 +361,7 @@ static grpc_error_handle start_accept_locked(grpc_tcp_listener* port) {
   return error;
 
 failure:
-  GPR_ASSERT(!error.ok());
+  CHECK(!error.ok());
   if (sock != INVALID_SOCKET) closesocket(sock);
   return error;
 }
@@ -318,8 +386,8 @@ static void on_accept(void* arg, grpc_error_handle error) {
   // this is necessary in the read/write case, it's useless for the accept
   // case. We only need to adjust the pending callback count
   if (!error.ok()) {
-    gpr_log(GPR_INFO, "Skipping on_accept due to error: %s",
-            grpc_core::StatusToString(error).c_str());
+    LOG(INFO) << "Skipping on_accept due to error: "
+              << grpc_core::StatusToString(error);
 
     gpr_mu_unlock(&sp->server->mu);
     return;
@@ -332,7 +400,7 @@ static void on_accept(void* arg, grpc_error_handle error) {
   if (!wsa_success) {
     if (!sp->shutting_down) {
       char* utf8_message = gpr_format_message(WSAGetLastError());
-      gpr_log(GPR_ERROR, "on_accept error: %s", utf8_message);
+      LOG(ERROR) << "on_accept error: " << utf8_message;
       gpr_free(utf8_message);
     }
     closesocket(sock);
@@ -342,7 +410,7 @@ static void on_accept(void* arg, grpc_error_handle error) {
                        (char*)&sp->socket->socket, sizeof(sp->socket->socket));
       if (err) {
         char* utf8_message = gpr_format_message(WSAGetLastError());
-        gpr_log(GPR_ERROR, "setsockopt error: %s", utf8_message);
+        LOG(ERROR) << "setsockopt error: " << utf8_message;
         gpr_free(utf8_message);
       }
       int peer_name_len = (int)peer_name.len;
@@ -354,12 +422,11 @@ static void on_accept(void* arg, grpc_error_handle error) {
         if (addr_uri.ok()) {
           peer_name_string = addr_uri.value();
         } else {
-          gpr_log(GPR_ERROR, "invalid peer name: %s",
-                  addr_uri.status().ToString().c_str());
+          LOG(ERROR) << "invalid peer name: " << addr_uri.status();
         }
       } else {
         char* utf8_message = gpr_format_message(WSAGetLastError());
-        gpr_log(GPR_ERROR, "getpeername error: %s", utf8_message);
+        LOG(ERROR) << "getpeername error: " << utf8_message;
         gpr_free(utf8_message);
       }
       std::string fd_name = absl::StrCat("tcp_server:", peer_name_string);
@@ -386,7 +453,7 @@ static void on_accept(void* arg, grpc_error_handle error) {
   // the former socked we created has now either been destroy or assigned
   // to the new connection. We need to create a new one for the next
   // connection.
-  GPR_ASSERT(GRPC_LOG_IF_ERROR("start_accept", start_accept_locked(sp)));
+  CHECK(GRPC_LOG_IF_ERROR("start_accept", start_accept_locked(sp)));
   if (0 == --sp->outstanding_calls) {
     decrement_active_ports_and_notify_locked(sp);
   }
@@ -412,11 +479,9 @@ static grpc_error_handle add_socket_to_server(grpc_tcp_server* s, SOCKET sock,
                &AcceptEx, sizeof(AcceptEx), &ioctl_num_bytes, NULL, NULL);
 
   if (status != 0) {
-    char* utf8_message = gpr_format_message(WSAGetLastError());
-    gpr_log(GPR_ERROR, "on_connect error: %s", utf8_message);
-    gpr_free(utf8_message);
+    error = GRPC_WSA_ERROR(WSAGetLastError(), "AcceptEx pointer retrieval");
     closesocket(sock);
-    return absl::OkStatus();
+    return error;
   }
 
   error = prepare_socket(sock, addr, &port);
@@ -424,7 +489,7 @@ static grpc_error_handle add_socket_to_server(grpc_tcp_server* s, SOCKET sock,
     return error;
   }
 
-  GPR_ASSERT(port >= 0);
+  CHECK_GE(port, 0);
   gpr_mu_lock(&s->mu);
   sp = (grpc_tcp_listener*)gpr_malloc(sizeof(grpc_tcp_listener));
   sp->next = NULL;
@@ -440,10 +505,11 @@ static grpc_error_handle add_socket_to_server(grpc_tcp_server* s, SOCKET sock,
   sp->outstanding_calls = 0;
   sp->AcceptEx = AcceptEx;
   sp->new_socket = INVALID_SOCKET;
+  sp->resolved_addr = *addr;
   sp->port = port;
   sp->port_index = port_index;
   GRPC_CLOSURE_INIT(&sp->on_accept, on_accept, sp, grpc_schedule_on_exec_ctx);
-  GPR_ASSERT(sp->socket);
+  CHECK(sp->socket);
   gpr_mu_unlock(&s->mu);
   *listener = sp;
 
@@ -458,7 +524,6 @@ static grpc_error_handle tcp_server_add_port(grpc_tcp_server* s,
   grpc_resolved_address addr6_v4mapped;
   grpc_resolved_address wildcard;
   grpc_resolved_address* allocated_addr = NULL;
-  grpc_resolved_address sockname_temp;
   unsigned port_index = 0;
   grpc_error_handle error;
 
@@ -470,6 +535,7 @@ static grpc_error_handle tcp_server_add_port(grpc_tcp_server* s,
   // as some previously created listener.
   if (grpc_sockaddr_get_port(addr) == 0) {
     for (sp = s->head; sp; sp = sp->next) {
+      grpc_resolved_address sockname_temp;
       int sockname_temp_len = sizeof(struct sockaddr_storage);
       if (0 == getsockname(sp->socket->socket,
                            (grpc_sockaddr*)sockname_temp.addr,
@@ -499,7 +565,10 @@ static grpc_error_handle tcp_server_add_port(grpc_tcp_server* s,
     addr = &wildcard;
   }
 
-  sock = WSASocket(AF_INET6, SOCK_STREAM, IPPROTO_TCP, NULL, 0,
+  const int addr_family =
+      grpc_sockaddr_get_family(addr) == AF_UNIX ? AF_UNIX : AF_INET6;
+  const int protocol = addr_family == AF_UNIX ? 0 : IPPROTO_TCP;
+  sock = WSASocket(addr_family, SOCK_STREAM, protocol, NULL, 0,
                    grpc_get_default_wsa_socket_flags());
   if (sock == INVALID_SOCKET) {
     error = GRPC_WSA_ERROR(WSAGetLastError(), "WSASocket");
@@ -517,7 +586,7 @@ done:
     error = error_out;
     *port = -1;
   } else {
-    GPR_ASSERT(sp != NULL);
+    CHECK(sp != NULL);
     *port = sp->port;
   }
   return error;
@@ -527,35 +596,177 @@ static void tcp_server_start(grpc_tcp_server* s,
                              const std::vector<grpc_pollset*>* /*pollsets*/) {
   grpc_tcp_listener* sp;
   gpr_mu_lock(&s->mu);
-  GPR_ASSERT(s->active_ports == 0);
+  CHECK_EQ(s->active_ports, 0u);
   for (sp = s->head; sp; sp = sp->next) {
-    GPR_ASSERT(GRPC_LOG_IF_ERROR("start_accept", start_accept_locked(sp)));
+    CHECK(GRPC_LOG_IF_ERROR("start_accept", start_accept_locked(sp)));
     s->active_ports++;
   }
   gpr_mu_unlock(&s->mu);
 }
 
-static unsigned tcp_server_port_fd_count(grpc_tcp_server* s,
-                                         unsigned port_index) {
+static unsigned tcp_server_port_fd_count(grpc_tcp_server* /* s */,
+                                         unsigned /* port_index */) {
   return 0;
 }
 
-static int tcp_server_port_fd(grpc_tcp_server* s, unsigned port_index,
-                              unsigned fd_index) {
+static int tcp_server_port_fd(grpc_tcp_server* /* s */,
+                              unsigned /* port_index */,
+                              unsigned /* fd_index */) {
   return -1;
 }
 
 static grpc_core::TcpServerFdHandler* tcp_server_create_fd_handler(
-    grpc_tcp_server* s) {
+    grpc_tcp_server* /* s */) {
   return nullptr;
 }
 
-static void tcp_server_shutdown_listeners(grpc_tcp_server* s) {}
+static void tcp_server_shutdown_listeners(grpc_tcp_server* /* s */) {}
+
+static int tcp_pre_allocated_fd(grpc_tcp_server* /* s */) { return -1; }
+
+static void tcp_set_pre_allocated_fd(grpc_tcp_server* /* s */, int /* fd */) {}
 
 grpc_tcp_server_vtable grpc_windows_tcp_server_vtable = {
     tcp_server_create,        tcp_server_start,
     tcp_server_add_port,      tcp_server_create_fd_handler,
     tcp_server_port_fd_count, tcp_server_port_fd,
     tcp_server_ref,           tcp_server_shutdown_starting_add,
-    tcp_server_unref,         tcp_server_shutdown_listeners};
+    tcp_server_unref,         tcp_server_shutdown_listeners,
+    tcp_pre_allocated_fd,     tcp_set_pre_allocated_fd};
+
+// ---- EventEngine shim ------------------------------------------------------
+
+namespace {
+
+static grpc_error_handle event_engine_create(grpc_closure* shutdown_complete,
+                                             const EndpointConfig& config,
+                                             grpc_tcp_server_cb on_accept_cb,
+                                             void* on_accept_cb_arg,
+                                             grpc_tcp_server** server) {
+  // On Windows, the event_engine_listener experiment only supports the
+  // default engine
+  WindowsEventEngine* engine_ptr = reinterpret_cast<WindowsEventEngine*>(
+      config.GetVoidPointer(GRPC_INTERNAL_ARG_EVENT_ENGINE));
+  grpc_tcp_server* s = (grpc_tcp_server*)gpr_malloc(sizeof(grpc_tcp_server));
+  CHECK_NE(on_accept_cb, nullptr);
+  auto accept_cb = [s, on_accept_cb, on_accept_cb_arg](
+                       std::unique_ptr<EventEngine::Endpoint> endpoint,
+                       MemoryAllocator memory_allocator) {
+    grpc_core::ApplicationCallbackExecCtx app_ctx;
+    grpc_core::ExecCtx exec_ctx;
+    grpc_tcp_server_acceptor* acceptor =
+        static_cast<grpc_tcp_server_acceptor*>(gpr_malloc(sizeof(*acceptor)));
+    acceptor->from_server = s;
+    acceptor->port_index = -1;
+    acceptor->fd_index = -1;
+    acceptor->external_connection = false;
+    on_accept_cb(on_accept_cb_arg,
+                 grpc_event_engine_endpoint_create(std::move(endpoint)),
+                 nullptr, acceptor);
+  };
+  auto on_shutdown = [shutdown_complete](absl::Status status) {
+    RunEventEngineClosure(shutdown_complete, status);
+  };
+  grpc_core::RefCountedPtr<grpc_core::ResourceQuota> resource_quota;
+  {
+    void* tmp_quota = config.GetVoidPointer(GRPC_ARG_RESOURCE_QUOTA);
+    CHECK_NE(tmp_quota, nullptr);
+    resource_quota =
+        reinterpret_cast<grpc_core::ResourceQuota*>(tmp_quota)->Ref();
+  }
+  gpr_ref_init(&s->refs, 1);
+  gpr_mu_init(&s->mu);
+  s->ee_listener = new WindowsEventEngineListener(
+      engine_ptr->poller(), std::move(accept_cb), std::move(on_shutdown),
+      std::make_unique<MemoryQuotaBasedMemoryAllocatorFactory>(
+          resource_quota->memory_quota()),
+      engine_ptr->shared_from_this(), engine_ptr->thread_pool(), config);
+  s->active_ports = -1;
+  s->on_accept_cb = [](void* /* arg */, grpc_endpoint* /* ep */,
+                       grpc_pollset* /* accepting_pollset */,
+                       grpc_tcp_server_acceptor* /* acceptor */) {
+    grpc_core::Crash("iomgr on_accept_cb callback should be unused");
+  };
+  s->on_accept_cb_arg = nullptr;
+  s->head = nullptr;
+  s->tail = nullptr;
+  s->shutdown_starting.head = nullptr;
+  s->shutdown_starting.tail = nullptr;
+  s->shutdown_complete = grpc_core::NewClosure([](absl::Status) {
+    grpc_core::Crash("iomgr shutdown_complete callback should be unused");
+  });
+  *server = s;
+  return absl::OkStatus();
+}
+
+static void event_engine_start(grpc_tcp_server* s,
+                               const std::vector<grpc_pollset*>* /*pollsets*/) {
+  CHECK(s->ee_listener->Start().ok());
+}
+
+static grpc_error_handle event_engine_add_port(
+    grpc_tcp_server* s, const grpc_resolved_address* addr, int* port) {
+  CHECK_NE(addr, nullptr);
+  CHECK_NE(port, nullptr);
+  auto ee_addr = CreateResolvedAddress(*addr);
+  auto out_port = s->ee_listener->Bind(ee_addr);
+  *port = out_port.ok() ? *out_port : -1;
+  return out_port.status();
+}
+
+static grpc_core::TcpServerFdHandler* event_engine_create_fd_handler(
+    grpc_tcp_server* /* s */) {
+  return nullptr;
+}
+
+static unsigned event_engine_port_fd_count(grpc_tcp_server* /* s */,
+                                           unsigned /* port_index */) {
+  return 0;
+}
+
+static int event_engine_port_fd(grpc_tcp_server* /* s */,
+                                unsigned /* port_index */,
+                                unsigned /* fd_index */) {
+  return -1;
+}
+
+static grpc_tcp_server* event_engine_ref(grpc_tcp_server* s) {
+  gpr_ref_non_zero(&s->refs);
+  return s;
+}
+
+static void event_engine_shutdown_listeners(grpc_tcp_server* s) {
+  s->ee_listener->ShutdownListeners();
+}
+
+static void event_engine_unref(grpc_tcp_server* s) {
+  if (gpr_unref(&s->refs)) {
+    event_engine_shutdown_listeners(s);
+    gpr_mu_lock(&s->mu);
+    grpc_core::ExecCtx::RunList(DEBUG_LOCATION, &s->shutdown_starting);
+    gpr_mu_unlock(&s->mu);
+    gpr_mu_destroy(&s->mu);
+    delete s->ee_listener;
+    gpr_free(s);
+  }
+}
+
+static void event_engine_shutdown_starting_add(
+    grpc_tcp_server* s, grpc_closure* shutdown_starting) {
+  gpr_mu_lock(&s->mu);
+  grpc_closure_list_append(&s->shutdown_starting, shutdown_starting,
+                           absl::OkStatus());
+  gpr_mu_unlock(&s->mu);
+}
+
+}  // namespace
+
+grpc_tcp_server_vtable grpc_windows_event_engine_tcp_server_vtable = {
+    event_engine_create,        event_engine_start,
+    event_engine_add_port,      event_engine_create_fd_handler,
+    event_engine_port_fd_count, event_engine_port_fd,
+    event_engine_ref,           event_engine_shutdown_starting_add,
+    event_engine_unref,         event_engine_shutdown_listeners,
+    tcp_pre_allocated_fd,       tcp_set_pre_allocated_fd};
+
 #endif  // GRPC_WINSOCK_SOCKET
