@@ -16,17 +16,14 @@
 //
 //
 
-#ifndef GRPC_CORE_LIB_SURFACE_API_TRACE_H
-#define GRPC_CORE_LIB_SURFACE_API_TRACE_H
-
-#include <grpc/support/port_platform.h>
+#ifndef GRPC_SRC_CORE_LIB_SURFACE_API_TRACE_H
+#define GRPC_SRC_CORE_LIB_SURFACE_API_TRACE_H
 
 #include <grpc/support/log.h>
+#include <grpc/support/port_platform.h>
 
 #include "src/core/lib/debug/trace.h"
 
-extern grpc_core::TraceFlag grpc_api_trace;
-
 // Provide unwrapping macros because we're in C89 and variadic macros weren't
 // introduced until C99...
 #define GRPC_API_TRACE_UNWRAP0()
@@ -46,8 +43,8 @@ extern grpc_core::TraceFlag grpc_api_trace;
 // Due to the limitations of C89's preprocessor, the arity of the var-arg list
 // 'nargs' must be specified.
 #define GRPC_API_TRACE(fmt, nargs, args)                      \
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_api_trace)) {              \
+  if (GRPC_TRACE_FLAG_ENABLED(api)) {                         \
     gpr_log(GPR_INFO, fmt GRPC_API_TRACE_UNWRAP##nargs args); \
   }
 
-#endif  // GRPC_CORE_LIB_SURFACE_API_TRACE_H
+#endif  // GRPC_SRC_CORE_LIB_SURFACE_API_TRACE_H
