@@ -1,55 +1,59 @@
-/*
- *
- * Copyright 2018 gRPC authors.
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
- *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
- *
- */
-
-#include <grpc/support/port_platform.h>
+//
+//
+// Copyright 2018 gRPC authors.
+//
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
+//
 
 #include "src/core/tsi/alts/zero_copy_frame_protector/alts_grpc_record_protocol_common.h"
 
 #include <string.h>
 
+#include "absl/log/check.h"
+
 #include <grpc/support/alloc.h>
 #include <grpc/support/log.h>
+#include <grpc/support/port_platform.h>
 
-#include "src/core/lib/gpr/useful.h"
+#include "src/core/lib/gprpp/crash.h"
 #include "src/core/lib/iomgr/exec_ctx.h"
 #include "src/core/lib/slice/slice_internal.h"
+#include "src/core/util/useful.h"
 
 const size_t kInitialIovecBufferSize = 8;
 
-/* Makes sure iovec_buf in alts_grpc_record_protocol is large enough.  */
+// Makes sure iovec_buf in alts_grpc_record_protocol is large enough.
 static void ensure_iovec_buf_size(alts_grpc_record_protocol* rp,
                                   const grpc_slice_buffer* sb) {
-  GPR_ASSERT(rp != nullptr && sb != nullptr);
+  CHECK(rp != nullptr);
+  CHECK_NE(sb, nullptr);
   if (sb->count <= rp->iovec_buf_length) {
     return;
   }
-  /* At least double the iovec buffer size.  */
+  // At least double the iovec buffer size.
   rp->iovec_buf_length = std::max(sb->count, 2 * rp->iovec_buf_length);
   rp->iovec_buf = static_cast<iovec_t*>(
       gpr_realloc(rp->iovec_buf, rp->iovec_buf_length * sizeof(iovec_t)));
 }
 
-/* --- Implementation of methods defined in tsi_grpc_record_protocol_common.h.
- * --- */
+// --- Implementation of methods defined in tsi_grpc_record_protocol_common.h.
+// ---
 
 void alts_grpc_record_protocol_convert_slice_buffer_to_iovec(
     alts_grpc_record_protocol* rp, const grpc_slice_buffer* sb) {
-  GPR_ASSERT(rp != nullptr && sb != nullptr);
+  CHECK(rp != nullptr);
+  CHECK_NE(sb, nullptr);
   ensure_iovec_buf_size(rp, sb);
   for (size_t i = 0; i < sb->count; i++) {
     rp->iovec_buf[i].iov_base = GRPC_SLICE_START_PTR(sb->slices[i]);
@@ -59,7 +63,8 @@ void alts_grpc_record_protocol_convert_slice_buffer_to_iovec(
 
 void alts_grpc_record_protocol_copy_slice_buffer(const grpc_slice_buffer* src,
                                                  unsigned char* dst) {
-  GPR_ASSERT(src != nullptr && dst != nullptr);
+  CHECK(src != nullptr);
+  CHECK_NE(dst, nullptr);
   for (size_t i = 0; i < src->count; i++) {
     size_t slice_length = GRPC_SLICE_LENGTH(src->slices[i]);
     memcpy(dst, GRPC_SLICE_START_PTR(src->slices[i]), slice_length);
@@ -77,8 +82,8 @@ iovec_t alts_grpc_record_protocol_get_header_iovec(
   if (rp->header_sb.count == 1) {
     header_iovec.iov_base = GRPC_SLICE_START_PTR(rp->header_sb.slices[0]);
   } else {
-    /* Frame header is in multiple slices, copies the header bytes from slice
-     * buffer to a single flat buffer.  */
+    // Frame header is in multiple slices, copies the header bytes from slice
+    // buffer to a single flat buffer.
     alts_grpc_record_protocol_copy_slice_buffer(&rp->header_sb, rp->header_buf);
     header_iovec.iov_base = rp->header_buf;
   }
@@ -95,7 +100,7 @@ tsi_result alts_grpc_record_protocol_init(alts_grpc_record_protocol* rp,
             "Invalid nullptr arguments to alts_grpc_record_protocol init.");
     return TSI_INVALID_ARGUMENT;
   }
-  /* Creates alts_iovec_record_protocol.  */
+  // Creates alts_iovec_record_protocol.
   char* error_details = nullptr;
   grpc_status_code status = alts_iovec_record_protocol_create(
       crypter, overflow_size, is_client, is_integrity_only, is_protect,
@@ -106,26 +111,25 @@ tsi_result alts_grpc_record_protocol_init(alts_grpc_record_protocol* rp,
     gpr_free(error_details);
     return TSI_INTERNAL_ERROR;
   }
-  /* Allocates header slice buffer.  */
+  // Allocates header slice buffer.
   grpc_slice_buffer_init(&rp->header_sb);
-  /* Allocates header buffer.  */
+  // Allocates header buffer.
   rp->header_length = alts_iovec_record_protocol_get_header_length();
   rp->header_buf = static_cast<unsigned char*>(gpr_malloc(rp->header_length));
   rp->tag_length = alts_iovec_record_protocol_get_tag_length(rp->iovec_rp);
-  /* Allocates iovec buffer.  */
+  // Allocates iovec buffer.
   rp->iovec_buf_length = kInitialIovecBufferSize;
   rp->iovec_buf =
       static_cast<iovec_t*>(gpr_malloc(rp->iovec_buf_length * sizeof(iovec_t)));
   return TSI_OK;
 }
 
-/* --- Implementation of methods defined in tsi_grpc_record_protocol.h. --- */
+// --- Implementation of methods defined in tsi_grpc_record_protocol.h. ---
 tsi_result alts_grpc_record_protocol_protect(
     alts_grpc_record_protocol* self, grpc_slice_buffer* unprotected_slices,
     grpc_slice_buffer* protected_slices) {
-  if (grpc_core::ExecCtx::Get() == nullptr || self == nullptr ||
-      self->vtable == nullptr || unprotected_slices == nullptr ||
-      protected_slices == nullptr) {
+  if (self == nullptr || self->vtable == nullptr ||
+      unprotected_slices == nullptr || protected_slices == nullptr) {
     return TSI_INVALID_ARGUMENT;
   }
   if (self->vtable->protect == nullptr) {
@@ -137,9 +141,8 @@ tsi_result alts_grpc_record_protocol_protect(
 tsi_result alts_grpc_record_protocol_unprotect(
     alts_grpc_record_protocol* self, grpc_slice_buffer* protected_slices,
     grpc_slice_buffer* unprotected_slices) {
-  if (grpc_core::ExecCtx::Get() == nullptr || self == nullptr ||
-      self->vtable == nullptr || protected_slices == nullptr ||
-      unprotected_slices == nullptr) {
+  if (self == nullptr || self->vtable == nullptr ||
+      protected_slices == nullptr || unprotected_slices == nullptr) {
     return TSI_INVALID_ARGUMENT;
   }
   if (self->vtable->unprotect == nullptr) {
@@ -162,8 +165,8 @@ void alts_grpc_record_protocol_destroy(alts_grpc_record_protocol* self) {
   gpr_free(self);
 }
 
-/* Integrity-only and privacy-integrity share the same implementation. No need
- * to call vtable.  */
+// Integrity-only and privacy-integrity share the same implementation. No need
+// to call vtable.
 size_t alts_grpc_record_protocol_max_unprotected_data_size(
     const alts_grpc_record_protocol* self, size_t max_protected_frame_size) {
   if (self == nullptr) {
