@@ -23,11 +23,12 @@
 #include <memory>
 #include <utility>
 
+#include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "upb/base/string_view.h"
 #include "upb/mem/arena.hpp"
 
 #include <grpc/slice.h>
-#include <grpc/support/log.h>
 #include <grpcpp/impl/rpc_method.h>
 #include <grpcpp/impl/rpc_service_method.h>
 #include <grpcpp/impl/server_callback_handlers.h>
@@ -109,7 +110,7 @@ void DefaultHealthCheckService::UnregisterWatch(
 
 DefaultHealthCheckService::HealthCheckServiceImpl*
 DefaultHealthCheckService::GetHealthCheckService() {
-  GPR_ASSERT(impl_ == nullptr);
+  CHECK(impl_ == nullptr);
   impl_ = std::make_unique<HealthCheckServiceImpl>(this);
   return impl_.get();
 }
@@ -258,8 +259,8 @@ DefaultHealthCheckService::HealthCheckServiceImpl::WatchReactor::WatchReactor(
     ++service_->num_watches_;
   }
   bool success = DecodeRequest(*request, &service_name_);
-  gpr_log(GPR_DEBUG, "[HCS %p] watcher %p \"%s\": watch call started", service_,
-          this, service_name_.c_str());
+  VLOG(2) << "[HCS " << service_ << "] watcher " << this << " \""
+          << service_name_ << "\": watch call started";
   if (!success) {
     MaybeFinishLocked(Status(StatusCode::INTERNAL, "could not parse request"));
     return;
@@ -270,15 +271,14 @@ DefaultHealthCheckService::HealthCheckServiceImpl::WatchReactor::WatchReactor(
 
 void DefaultHealthCheckService::HealthCheckServiceImpl::WatchReactor::
     SendHealth(ServingStatus status) {
-  gpr_log(GPR_DEBUG,
-          "[HCS %p] watcher %p \"%s\": SendHealth() for ServingStatus %d",
-          service_, this, service_name_.c_str(), status);
+  VLOG(2) << "[HCS " << service_ << "] watcher " << this << " \""
+          << service_name_ << "\": SendHealth() for ServingStatus " << status;
   grpc::internal::MutexLock lock(&mu_);
   // If there's already a send in flight, cache the new status, and
   // we'll start a new send for it when the one in flight completes.
   if (write_pending_) {
-    gpr_log(GPR_DEBUG, "[HCS %p] watcher %p \"%s\": queuing write", service_,
-            this, service_name_.c_str());
+    VLOG(2) << "[HCS " << service_ << "] watcher " << this << " \""
+            << service_name_ << "\": queuing write";
     pending_status_ = status;
     return;
   }
@@ -306,17 +306,16 @@ void DefaultHealthCheckService::HealthCheckServiceImpl::WatchReactor::
         Status(StatusCode::INTERNAL, "could not encode response"));
     return;
   }
-  gpr_log(GPR_DEBUG,
-          "[HCS %p] watcher %p \"%s\": starting write for ServingStatus %d",
-          service_, this, service_name_.c_str(), status);
+  VLOG(2) << "[HCS " << service_ << "] watcher " << this << " \""
+          << service_name_ << "\": starting write for ServingStatus " << status;
   write_pending_ = true;
   StartWrite(&response_);
 }
 
 void DefaultHealthCheckService::HealthCheckServiceImpl::WatchReactor::
     OnWriteDone(bool ok) {
-  gpr_log(GPR_DEBUG, "[HCS %p] watcher %p \"%s\": OnWriteDone(): ok=%d",
-          service_, this, service_name_.c_str(), ok);
+  VLOG(2) << "[HCS " << service_ << "] watcher " << this << " \""
+          << service_name_ << "\": OnWriteDone(): ok=" << ok;
   response_.Clear();
   grpc::internal::MutexLock lock(&mu_);
   if (!ok) {
@@ -340,8 +339,8 @@ void DefaultHealthCheckService::HealthCheckServiceImpl::WatchReactor::
 }
 
 void DefaultHealthCheckService::HealthCheckServiceImpl::WatchReactor::OnDone() {
-  gpr_log(GPR_DEBUG, "[HCS %p] watcher %p \"%s\": OnDone()", service_, this,
-          service_name_.c_str());
+  VLOG(2) << "[HCS " << service_ << "] watcher " << this << " \""
+          << service_name_ << "\": OnDone()";
   service_->database_->UnregisterWatch(service_name_, this);
   {
     grpc::internal::MutexLock lock(&service_->mu_);
@@ -355,13 +354,13 @@ void DefaultHealthCheckService::HealthCheckServiceImpl::WatchReactor::OnDone() {
 
 void DefaultHealthCheckService::HealthCheckServiceImpl::WatchReactor::
     MaybeFinishLocked(Status status) {
-  gpr_log(GPR_DEBUG,
-          "[HCS %p] watcher %p \"%s\": MaybeFinishLocked() with code=%d msg=%s",
-          service_, this, service_name_.c_str(), status.error_code(),
-          status.error_message().c_str());
+  VLOG(2) << "[HCS " << service_ << "] watcher " << this << " \""
+          << service_name_
+          << "\": MaybeFinishLocked() with code=" << status.error_code()
+          << " msg=" << status.error_message();
   if (!finish_called_) {
-    gpr_log(GPR_DEBUG, "[HCS %p] watcher %p \"%s\": actually calling Finish()",
-            service_, this, service_name_.c_str());
+    VLOG(2) << "[HCS " << service_ << "] watcher " << this << " \""
+            << service_name_ << "\": actually calling Finish()";
     finish_called_ = true;
     Finish(status);
   }
