@@ -29,12 +29,14 @@
 // configurations and assess whether such a change is correct and desirable.
 //
 
-#include <string.h>
-
 #include <algorithm>
+#include <memory>
 #include <string>
 #include <vector>
 
+#include "absl/log/check.h"
+#include "absl/memory/memory.h"
+#include "absl/strings/string_view.h"
 #include "gtest/gtest.h"
 
 #include <grpc/grpc.h>
@@ -45,30 +47,55 @@
 #include "src/core/lib/channel/channel_stack.h"
 #include "src/core/lib/channel/channel_stack_builder_impl.h"
 #include "src/core/lib/config/core_configuration.h"
+#include "src/core/lib/experiments/experiments.h"
+#include "src/core/lib/iomgr/endpoint.h"
 #include "src/core/lib/iomgr/exec_ctx.h"
+#include "src/core/lib/iomgr/iomgr_fwd.h"
 #include "src/core/lib/surface/channel_init.h"
 #include "src/core/lib/surface/channel_stack_type.h"
-#include "src/core/lib/transport/transport_fwd.h"
-#include "src/core/lib/transport/transport_impl.h"
-#include "test/core/util/test_config.h"
+#include "src/core/lib/transport/transport.h"
+#include "test/core/test_util/test_config.h"
+
+namespace {
+class FakeTransport final : public grpc_core::Transport {
+ public:
+  explicit FakeTransport(absl::string_view transport_name)
+      : transport_name_(transport_name) {}
+
+  grpc_core::FilterStackTransport* filter_stack_transport() override {
+    return nullptr;
+  }
+  grpc_core::ClientTransport* client_transport() override { return nullptr; }
+  grpc_core::ServerTransport* server_transport() override { return nullptr; }
+
+  absl::string_view GetTransportName() const override {
+    return transport_name_;
+  }
+  void SetPollset(grpc_stream*, grpc_pollset*) override {}
+  void SetPollsetSet(grpc_stream*, grpc_pollset_set*) override {}
+  void PerformOp(grpc_transport_op*) override {}
+  void Orphan() override {}
+
+ private:
+  absl::string_view transport_name_;
+};
+}  // namespace
 
 std::vector<std::string> MakeStack(const char* transport_name,
-                                   const grpc_core::ChannelArgs& channel_args,
+                                   grpc_core::ChannelArgs channel_args,
                                    grpc_channel_stack_type channel_stack_type) {
   // create phony channel stack
+  std::unique_ptr<FakeTransport> fake_transport;
+  if (transport_name != nullptr) {
+    fake_transport = absl::make_unique<FakeTransport>(transport_name);
+    channel_args = channel_args.SetObject(fake_transport.get());
+  }
   grpc_core::ChannelStackBuilderImpl builder("test", channel_stack_type,
                                              channel_args);
-  grpc_transport_vtable fake_transport_vtable;
-  memset(&fake_transport_vtable, 0, sizeof(grpc_transport_vtable));
-  fake_transport_vtable.name = transport_name;
-  grpc_transport fake_transport = {&fake_transport_vtable};
   builder.SetTarget("foo.test.google.fr");
-  if (transport_name != nullptr) {
-    builder.SetTransport(&fake_transport);
-  }
   {
     grpc_core::ExecCtx exec_ctx;
-    GPR_ASSERT(grpc_core::CoreConfiguration::Get().channel_init().CreateStack(
+    CHECK(grpc_core::CoreConfiguration::Get().channel_init().CreateStack(
         &builder));
   }
 
@@ -110,31 +137,31 @@ TEST(ChannelStackFilters, LooksAsExpected) {
 
   // tests with a default stack
 
-  EXPECT_EQ(MakeStack("unknown", no_args, GRPC_CLIENT_DIRECT_CHANNEL),
-            std::vector<std::string>(
-                {"authority", "message_size", "deadline", "connected"}));
+  EXPECT_EQ(
+      MakeStack("unknown", no_args, GRPC_CLIENT_DIRECT_CHANNEL),
+      std::vector<std::string>({"authority", "message_size", "connected"}));
   EXPECT_EQ(
       MakeStack("unknown", no_args, GRPC_CLIENT_SUBCHANNEL),
       std::vector<std::string>({"authority", "message_size", "connected"}));
   EXPECT_EQ(MakeStack("unknown", no_args, GRPC_SERVER_CHANNEL),
-            std::vector<std::string>({"server", "message_size", "deadline",
-                                      "server_call_tracer", "connected"}));
+            std::vector<std::string>(
+                {"server", "message_size", "server_call_tracer", "connected"}));
 
   EXPECT_EQ(
       MakeStack("chttp2", no_args, GRPC_CLIENT_DIRECT_CHANNEL),
-      std::vector<std::string>({"authority", "message_size", "deadline",
-                                "http-client", "compression", "connected"}));
+      std::vector<std::string>({"authority", "message_size", "http-client",
+                                "compression", "connected"}));
   EXPECT_EQ(
       MakeStack("chttp2", no_args, GRPC_CLIENT_SUBCHANNEL),
       std::vector<std::string>({"authority", "message_size", "http-client",
                                 "compression", "connected"}));
 
   EXPECT_EQ(MakeStack("chttp2", no_args, GRPC_SERVER_CHANNEL),
-            std::vector<std::string>({"server", "message_size", "deadline",
-                                      "http-server", "compression",
-                                      "server_call_tracer", "connected"}));
+            std::vector<std::string>({"server", "message_size", "http-server",
+                                      "compression", "server_call_tracer",
+                                      "connected"}));
   EXPECT_EQ(MakeStack(nullptr, no_args, GRPC_CLIENT_CHANNEL),
-            std::vector<std::string>({"client-channel"}));
+            std::vector<std::string>({"client_idle", "client-channel"}));
 }
 
 int main(int argc, char** argv) {
