@@ -28,6 +28,8 @@
 #include <utility>
 #include <vector>
 
+#include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "absl/strings/escaping.h"
 #include "absl/strings/str_cat.h"
 #include "absl/strings/str_format.h"
@@ -40,7 +42,6 @@
 #include <grpc/grpc.h>
 #include <grpc/slice.h>
 #include <grpc/slice_buffer.h>
-#include <grpc/support/log.h>
 #include <grpc/support/time.h>
 
 #include "src/core/lib/compression/message_compress.h"
@@ -48,7 +49,7 @@
 #include "src/core/lib/gprpp/debug_location.h"
 #include "src/core/lib/gprpp/match.h"
 #include "src/core/lib/surface/event_string.h"
-#include "test/core/util/test_config.h"
+#include "test/core/test_util/test_config.h"
 
 // a set of metadata we expect to find on an event
 typedef struct metadata {
@@ -125,9 +126,8 @@ int raw_byte_buffer_eq_slice(grpc_byte_buffer* rbb, grpc_slice b) {
        0 == memcmp(GRPC_SLICE_START_PTR(a), GRPC_SLICE_START_PTR(b),
                    GRPC_SLICE_LENGTH(a));
   if (!ok) {
-    gpr_log(GPR_ERROR,
-            "SLICE MISMATCH: left_length=%" PRIuPTR " right_length=%" PRIuPTR,
-            GRPC_SLICE_LENGTH(a), GRPC_SLICE_LENGTH(b));
+    LOG(ERROR) << "SLICE MISMATCH: left_length=" << GRPC_SLICE_LENGTH(a)
+               << " right_length=" << GRPC_SLICE_LENGTH(b);
     std::string out;
     const char* a_str = reinterpret_cast<const char*>(GRPC_SLICE_START_PTR(a));
     const char* b_str = reinterpret_cast<const char*>(GRPC_SLICE_START_PTR(b));
@@ -150,7 +150,7 @@ int raw_byte_buffer_eq_slice(grpc_byte_buffer* rbb, grpc_slice b) {
                         absl::CEscape(absl::string_view(&b_str[i], 1)),
                         "\u001b[0m");
       }
-      gpr_log(GPR_ERROR, "%s", out.c_str());
+      LOG(ERROR) << out;
     }
   }
   grpc_slice_unref(a);
@@ -163,9 +163,9 @@ int byte_buffer_eq_slice(grpc_byte_buffer* bb, grpc_slice b) {
   if (bb->data.raw.compression > GRPC_COMPRESS_NONE) {
     grpc_slice_buffer decompressed_buffer;
     grpc_slice_buffer_init(&decompressed_buffer);
-    GPR_ASSERT(grpc_msg_decompress(bb->data.raw.compression,
-                                   &bb->data.raw.slice_buffer,
-                                   &decompressed_buffer));
+    CHECK(grpc_msg_decompress(bb->data.raw.compression,
+                              &bb->data.raw.slice_buffer,
+                              &decompressed_buffer));
     grpc_byte_buffer* rbb = grpc_raw_byte_buffer_create(
         decompressed_buffer.slices, decompressed_buffer.count);
     int ret_val = raw_byte_buffer_eq_slice(rbb, b);
@@ -364,8 +364,7 @@ void CqVerifier::Verify(Duration timeout, SourceLocation location) {
   while (!expectations_.empty()) {
     must_log = std::exchange(added_expectations_, false) || must_log;
     if (log_verifications_ && must_log) {
-      gpr_log(GPR_ERROR, "Verify %s for %s", ToShortString().c_str(),
-              timeout.ToString().c_str());
+      LOG(ERROR) << "Verify " << ToShortString() << " for " << timeout;
     }
     must_log = false;
     grpc_event ev = Step(deadline);
@@ -422,12 +421,11 @@ bool CqVerifier::AllMaybes() const {
 
 void CqVerifier::VerifyEmpty(Duration timeout, SourceLocation location) {
   if (log_verifications_) {
-    gpr_log(GPR_ERROR, "Verify empty completion queue for %s",
-            timeout.ToString().c_str());
+    LOG(ERROR) << "Verify empty completion queue for " << timeout;
   }
   const gpr_timespec deadline =
       gpr_time_add(gpr_now(GPR_CLOCK_MONOTONIC), timeout.as_timespec());
-  GPR_ASSERT(expectations_.empty());
+  CHECK(expectations_.empty());
   grpc_event ev = Step(deadline);
   if (ev.type != GRPC_QUEUE_TIMEOUT) {
     FailUnexpectedEvent(&ev, location);
