@@ -21,20 +21,23 @@
 #include <limits.h>
 #include <string.h>
 
-#include <algorithm>
+#include <atomic>
 #include <memory>
 #include <string>
 #include <vector>
 
+#include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "absl/status/status.h"
 #include "absl/status/statusor.h"
+#include "absl/strings/escaping.h"
 #include "absl/strings/str_cat.h"
+#include "absl/strings/strip.h"
 
 #include <grpc/grpc.h>
 #include <grpc/slice.h>
 #include <grpc/slice_buffer.h>
 #include <grpc/support/alloc.h>
-#include <grpc/support/log.h>
 #include <grpc/support/sync.h>
 
 #include "src/core/lib/address_utils/sockaddr_utils.h"
@@ -42,12 +45,12 @@
 #include "src/core/lib/channel/channel_args_preconditioning.h"
 #include "src/core/lib/config/core_configuration.h"
 #include "src/core/lib/event_engine/channel_args_endpoint_config.h"
+#include "src/core/lib/event_engine/default_event_engine.h"
 #include "src/core/lib/gprpp/host_port.h"
 #include "src/core/lib/gprpp/memory.h"
 #include "src/core/lib/gprpp/status_helper.h"
 #include "src/core/lib/gprpp/thd.h"
 #include "src/core/lib/gprpp/time.h"
-#include "src/core/lib/http/parser.h"
 #include "src/core/lib/iomgr/closure.h"
 #include "src/core/lib/iomgr/combiner.h"
 #include "src/core/lib/iomgr/endpoint.h"
@@ -61,26 +64,47 @@
 #include "src/core/lib/iomgr/sockaddr.h"
 #include "src/core/lib/iomgr/tcp_client.h"
 #include "src/core/lib/iomgr/tcp_server.h"
-#include "src/core/lib/slice/b64.h"
-#include "test/core/util/port.h"
+#include "src/core/util/http_client/parser.h"
+#include "test/core/test_util/port.h"
 
 struct grpc_end2end_http_proxy {
   grpc_end2end_http_proxy()
-      : server(nullptr), channel_args(nullptr), mu(nullptr), combiner(nullptr) {
-    gpr_ref_init(&users, 1);
-    combiner = grpc_combiner_create();
-  }
+      : combiner(grpc_combiner_create(
+            grpc_event_engine::experimental::GetDefaultEventEngine())) {}
   std::string proxy_name;
+  std::atomic<bool> is_shutdown{false};
+  std::atomic<size_t> users{1};
   grpc_core::Thread thd;
-  grpc_tcp_server* server;
-  const grpc_channel_args* channel_args;
-  gpr_mu* mu;
+  grpc_tcp_server* server = nullptr;
+  const grpc_channel_args* channel_args = nullptr;
+  gpr_mu* mu = nullptr;
   std::vector<grpc_pollset*> pollset;
-  gpr_refcount users;
-
-  grpc_core::Combiner* combiner;
+  grpc_core::Combiner* combiner = nullptr;
 };
 
+namespace {
+
+// Sometimes, on_accept may be called after thread_main has returned, and the
+// proxy will have already been destroyed. This value is reset every time a
+// proxy fixture is created, and it prevents a crash due to a repeated unref.
+std::atomic<bool> proxy_destroyed{false};
+
+void proxy_ref(grpc_end2end_http_proxy* proxy) { proxy->users.fetch_add(1); }
+
+// Returns the remaining number of outstanding refs
+size_t proxy_unref(grpc_end2end_http_proxy* proxy) {
+  if (proxy_destroyed.load()) return -1;
+  size_t ref_count = proxy->users.fetch_sub(1) - 1;
+  if (ref_count == 0) {
+    proxy_destroyed.store(true);
+    GRPC_COMBINER_UNREF(proxy->combiner, "test");
+    delete proxy;
+  }
+  return ref_count;
+}
+
+}  // namespace
+
 //
 // Connection handling
 //
@@ -137,9 +161,9 @@ static void proxy_connection_ref(proxy_connection* conn,
 static void proxy_connection_unref(proxy_connection* conn,
                                    const char* /*reason*/) {
   if (gpr_unref(&conn->refcount)) {
-    gpr_log(GPR_DEBUG, "endpoints: %p %p", conn->client_endpoint,
-            conn->server_endpoint);
-    grpc_endpoint_destroy(conn->client_endpoint);
+    if (conn->client_endpoint != nullptr) {
+      grpc_endpoint_destroy(conn->client_endpoint);
+    }
     if (conn->server_endpoint != nullptr) {
       grpc_endpoint_destroy(conn->server_endpoint);
     }
@@ -152,7 +176,7 @@ static void proxy_connection_unref(proxy_connection* conn,
     grpc_slice_buffer_destroy(&conn->server_write_buffer);
     grpc_http_parser_destroy(&conn->http_parser);
     grpc_http_request_destroy(&conn->http_request);
-    gpr_unref(&conn->proxy->users);
+    proxy_unref(conn->proxy);
     gpr_free(conn);
   }
 }
@@ -204,13 +228,16 @@ static void proxy_connection_failed(proxy_connection* conn,
     }
   }
   // If we decided to shut down either one and have not yet done so, do so.
-  if (shutdown_client && !conn->client_shutdown) {
-    grpc_endpoint_shutdown(conn->client_endpoint, error);
+  if (shutdown_client && !conn->client_shutdown &&
+      conn->client_endpoint != nullptr) {
+    grpc_endpoint_destroy(conn->client_endpoint);
+    conn->client_endpoint = nullptr;
     conn->client_shutdown = true;
   }
   if (shutdown_server && !conn->server_shutdown &&
-      (conn->server_endpoint != nullptr)) {
-    grpc_endpoint_shutdown(conn->server_endpoint, error);
+      conn->server_endpoint != nullptr) {
+    grpc_endpoint_destroy(conn->server_endpoint);
+    conn->server_endpoint = nullptr;
     conn->server_shutdown = true;
   }
   // Unref the connection.
@@ -228,8 +255,8 @@ static void on_client_write_done_locked(void* arg, grpc_error_handle error) {
     return;
   }
   if (conn->server_read_failed) {
-    grpc_endpoint_shutdown(conn->client_endpoint,
-                           absl::UnknownError("Client shutdown"));
+    grpc_endpoint_destroy(conn->client_endpoint);
+    conn->client_endpoint = nullptr;
     // No more writes.  Unref the connection.
     proxy_connection_unref(conn, "client_write");
     return;
@@ -238,7 +265,8 @@ static void on_client_write_done_locked(void* arg, grpc_error_handle error) {
   grpc_slice_buffer_reset_and_unref(&conn->client_write_buffer);
   // If more data was read from the server since we started this write,
   // write that data now.
-  if (conn->client_deferred_write_buffer.length > 0) {
+  if (conn->client_deferred_write_buffer.length > 0 &&
+      conn->client_endpoint != nullptr) {
     grpc_slice_buffer_move_into(&conn->client_deferred_write_buffer,
                                 &conn->client_write_buffer);
     conn->client_is_writing = true;
@@ -270,10 +298,9 @@ static void on_server_write_done_locked(void* arg, grpc_error_handle error) {
                             "HTTP proxy server write", error);
     return;
   }
-
   if (conn->client_read_failed) {
-    grpc_endpoint_shutdown(conn->server_endpoint,
-                           absl::UnknownError("Server shutdown"));
+    grpc_endpoint_destroy(conn->server_endpoint);
+    conn->server_endpoint = nullptr;
     // No more writes.  Unref the connection.
     proxy_connection_unref(conn, "server_write");
     return;
@@ -282,7 +309,8 @@ static void on_server_write_done_locked(void* arg, grpc_error_handle error) {
   grpc_slice_buffer_reset_and_unref(&conn->server_write_buffer);
   // If more data was read from the client since we started this write,
   // write that data now.
-  if (conn->server_deferred_write_buffer.length > 0) {
+  if (conn->server_deferred_write_buffer.length > 0 &&
+      conn->server_endpoint != nullptr) {
     grpc_slice_buffer_move_into(&conn->server_deferred_write_buffer,
                                 &conn->server_write_buffer);
     conn->server_is_writing = true;
@@ -323,7 +351,7 @@ static void on_client_read_done_locked(void* arg, grpc_error_handle error) {
   if (conn->server_is_writing) {
     grpc_slice_buffer_move_into(&conn->client_read_buffer,
                                 &conn->server_deferred_write_buffer);
-  } else {
+  } else if (conn->server_endpoint != nullptr) {
     grpc_slice_buffer_move_into(&conn->client_read_buffer,
                                 &conn->server_write_buffer);
     proxy_connection_ref(conn, "client_read");
@@ -334,6 +362,10 @@ static void on_client_read_done_locked(void* arg, grpc_error_handle error) {
                         &conn->on_server_write_done, nullptr,
                         /*max_frame_size=*/INT_MAX);
   }
+  if (conn->client_endpoint == nullptr) {
+    proxy_connection_unref(conn, "client_read");
+    return;
+  }
   // Read more data.
   GRPC_CLOSURE_INIT(&conn->on_client_read_done, on_client_read_done, conn,
                     grpc_schedule_on_exec_ctx);
@@ -368,7 +400,7 @@ static void on_server_read_done_locked(void* arg, grpc_error_handle error) {
   if (conn->client_is_writing) {
     grpc_slice_buffer_move_into(&conn->server_read_buffer,
                                 &conn->client_deferred_write_buffer);
-  } else {
+  } else if (conn->client_endpoint != nullptr) {
     grpc_slice_buffer_move_into(&conn->server_read_buffer,
                                 &conn->client_write_buffer);
     proxy_connection_ref(conn, "server_read");
@@ -379,6 +411,10 @@ static void on_server_read_done_locked(void* arg, grpc_error_handle error) {
                         &conn->on_client_write_done, nullptr,
                         /*max_frame_size=*/INT_MAX);
   }
+  if (conn->server_endpoint == nullptr) {
+    proxy_connection_unref(conn, "server_read");
+    return;
+  }
   // Read more data.
   GRPC_CLOSURE_INIT(&conn->on_server_read_done, on_server_read_done, conn,
                     grpc_schedule_on_exec_ctx);
@@ -470,19 +506,14 @@ static void on_server_connect_done(void* arg, grpc_error_handle error) {
 /// Basic <base64_encoded_expected_cred>
 /// Returns true if it matches, false otherwise
 ///
-static bool proxy_auth_header_matches(char* proxy_auth_header_val,
-                                      char* expected_cred) {
-  GPR_ASSERT(proxy_auth_header_val != nullptr);
-  GPR_ASSERT(expected_cred != nullptr);
-  if (strncmp(proxy_auth_header_val, "Basic ", 6) != 0) {
+static bool proxy_auth_header_matches(absl::string_view proxy_auth_header_val,
+                                      absl::string_view expected_cred) {
+  if (!absl::ConsumePrefix(&proxy_auth_header_val, "Basic ")) {
     return false;
   }
-  proxy_auth_header_val += 6;
-  grpc_slice decoded_slice = grpc_base64_decode(proxy_auth_header_val, 0);
-  const bool header_matches =
-      grpc_slice_str_cmp(decoded_slice, expected_cred) == 0;
-  grpc_slice_unref(decoded_slice);
-  return header_matches;
+  std::string decoded;
+  absl::Base64Unescape(proxy_auth_header_val, &decoded);
+  return expected_cred == decoded;
 }
 
 // Callback to read the HTTP CONNECT request.
@@ -493,8 +524,8 @@ static bool proxy_auth_header_matches(char* proxy_auth_header_val,
 // which will cause the client connection to be dropped.
 static void on_read_request_done_locked(void* arg, grpc_error_handle error) {
   proxy_connection* conn = static_cast<proxy_connection*>(arg);
-  gpr_log(GPR_DEBUG, "on_read_request_done: %p %s", conn,
-          grpc_core::StatusToString(error).c_str());
+  VLOG(2) << "on_read_request_done: " << conn << " "
+          << grpc_core::StatusToString(error);
   if (!error.ok()) {
     proxy_connection_failed(conn, SETUP_FAILED, "HTTP proxy read request",
                             error);
@@ -559,7 +590,7 @@ static void on_read_request_done_locked(void* arg, grpc_error_handle error) {
     proxy_connection_failed(conn, SETUP_FAILED, "HTTP proxy DNS lookup", error);
     return;
   }
-  GPR_ASSERT(!addresses_or->empty());
+  CHECK(!addresses_or->empty());
   // Connect to requested address.
   // The connection callback inherits our reference to conn.
   const grpc_core::Timestamp deadline =
@@ -586,10 +617,19 @@ static void on_accept(void* arg, grpc_endpoint* endpoint,
                       grpc_pollset* /*accepting_pollset*/,
                       grpc_tcp_server_acceptor* acceptor) {
   gpr_free(acceptor);
+  if (proxy_destroyed.load()) {
+    grpc_endpoint_destroy(endpoint);
+    return;
+  }
   grpc_end2end_http_proxy* proxy = static_cast<grpc_end2end_http_proxy*>(arg);
+  proxy_ref(proxy);
+  if (proxy->is_shutdown.load()) {
+    grpc_endpoint_destroy(endpoint);
+    proxy_unref(proxy);
+    return;
+  }
   // Instantiate proxy_connection.
   proxy_connection* conn = grpc_core::Zalloc<proxy_connection>();
-  gpr_ref(&proxy->users);
   conn->client_endpoint = endpoint;
   conn->proxy = proxy;
   gpr_ref_init(&conn->refcount, 1);
@@ -621,26 +661,28 @@ static void thread_main(void* arg) {
   grpc_end2end_http_proxy* proxy = static_cast<grpc_end2end_http_proxy*>(arg);
   grpc_core::ExecCtx exec_ctx;
   do {
-    gpr_ref(&proxy->users);
+    proxy_ref(proxy);
     grpc_pollset_worker* worker = nullptr;
     gpr_mu_lock(proxy->mu);
-    GRPC_LOG_IF_ERROR("grpc_pollset_work",
-                      grpc_pollset_work(proxy->pollset[0], &worker,
-                                        grpc_core::Timestamp::Now() +
-                                            grpc_core::Duration::Seconds(1)));
+    GRPC_LOG_IF_ERROR(
+        "grpc_pollset_work",
+        grpc_pollset_work(proxy->pollset[0], &worker,
+                          grpc_core::Timestamp::Now() +
+                              grpc_core::Duration::Milliseconds(100)));
     gpr_mu_unlock(proxy->mu);
     grpc_core::ExecCtx::Get()->Flush();
-  } while (!gpr_unref(&proxy->users));
+  } while (proxy_unref(proxy) > 1 || !proxy->is_shutdown.load());
 }
 
 grpc_end2end_http_proxy* grpc_end2end_http_proxy_create(
     const grpc_channel_args* args) {
   grpc_core::ExecCtx exec_ctx;
   grpc_end2end_http_proxy* proxy = new grpc_end2end_http_proxy();
+  proxy_destroyed.store(false);
   // Construct proxy address.
   const int proxy_port = grpc_pick_unused_port_or_die();
   proxy->proxy_name = grpc_core::JoinHostPort("localhost", proxy_port);
-  gpr_log(GPR_INFO, "Proxy address: %s", proxy->proxy_name.c_str());
+  VLOG(2) << "Proxy address: " << proxy->proxy_name;
   // Create TCP server.
   auto channel_args = grpc_core::CoreConfiguration::Get()
                           .channel_args_preconditioning()
@@ -650,7 +692,7 @@ grpc_end2end_http_proxy* grpc_end2end_http_proxy_create(
       nullptr,
       grpc_event_engine::experimental::ChannelArgsEndpointConfig(channel_args),
       on_accept, proxy, &proxy->server);
-  GPR_ASSERT(error.ok());
+  CHECK_OK(error);
   // Bind to port.
   grpc_resolved_address resolved_addr;
   grpc_sockaddr_in* addr =
@@ -661,8 +703,8 @@ grpc_end2end_http_proxy* grpc_end2end_http_proxy_create(
   grpc_sockaddr_set_port(&resolved_addr, proxy_port);
   int port;
   error = grpc_tcp_server_add_port(proxy->server, &resolved_addr, &port);
-  GPR_ASSERT(error.ok());
-  GPR_ASSERT(port == proxy_port);
+  CHECK_OK(error);
+  CHECK(port == proxy_port);
   // Start server.
   auto* pollset = static_cast<grpc_pollset*>(gpr_zalloc(grpc_pollset_size()));
   grpc_pollset_init(pollset, &proxy->mu);
@@ -682,7 +724,7 @@ static void destroy_pollset(void* arg, grpc_error_handle /*error*/) {
 }
 
 void grpc_end2end_http_proxy_destroy(grpc_end2end_http_proxy* proxy) {
-  gpr_unref(&proxy->users);  // Signal proxy thread to shutdown.
+  proxy->is_shutdown.store(true);
   grpc_core::ExecCtx exec_ctx;
   proxy->thd.Join();
   grpc_tcp_server_shutdown_listeners(proxy->server);
@@ -691,8 +733,7 @@ void grpc_end2end_http_proxy_destroy(grpc_end2end_http_proxy* proxy) {
   grpc_pollset_shutdown(proxy->pollset[0],
                         GRPC_CLOSURE_CREATE(destroy_pollset, proxy->pollset[0],
                                             grpc_schedule_on_exec_ctx));
-  GRPC_COMBINER_UNREF(proxy->combiner, "test");
-  delete proxy;
+  proxy_unref(proxy);
 }
 
 const char* grpc_end2end_http_proxy_get_proxy_name(
