@@ -34,8 +34,8 @@ namespace {
 
 void TestMaxMessageLengthOnClientOnRequest(CoreEnd2endTest& test) {
   auto c = test.NewClientCall("/service/method").Create();
-  CoreEnd2endTest::IncomingStatusOnClient server_status;
-  CoreEnd2endTest::IncomingMetadata server_initial_metadata;
+  IncomingStatusOnClient server_status;
+  IncomingMetadata server_initial_metadata;
   c.NewBatch(1)
       .SendInitialMetadata({})
       .SendMessage("hello world")
@@ -45,13 +45,14 @@ void TestMaxMessageLengthOnClientOnRequest(CoreEnd2endTest& test) {
   test.Expect(1, true);
   test.Step();
   EXPECT_EQ(server_status.status(), GRPC_STATUS_RESOURCE_EXHAUSTED);
-  EXPECT_EQ(server_status.message(), "Sent message larger than max (11 vs. 5)");
+  EXPECT_EQ(server_status.message(),
+            "CLIENT: Sent message larger than max (11 vs. 5)");
 }
 
 void TestMaxMessageLengthOnServerOnRequest(CoreEnd2endTest& test) {
   auto c = test.NewClientCall("/service/method").Create();
-  CoreEnd2endTest::IncomingStatusOnClient server_status;
-  CoreEnd2endTest::IncomingMetadata server_initial_metadata;
+  IncomingStatusOnClient server_status;
+  IncomingMetadata server_initial_metadata;
   c.NewBatch(1)
       .SendInitialMetadata({})
       .SendMessage("hello world")
@@ -61,8 +62,8 @@ void TestMaxMessageLengthOnServerOnRequest(CoreEnd2endTest& test) {
   auto s = test.RequestCall(101);
   test.Expect(101, true);
   test.Step();
-  CoreEnd2endTest::IncomingCloseOnServer client_close;
-  CoreEnd2endTest::IncomingMessage client_message;
+  IncomingCloseOnServer client_close;
+  IncomingMessage client_message;
   s.NewBatch(102).RecvCloseOnServer(client_close).RecvMessage(client_message);
   test.Expect(102, true);
   test.Expect(1, true);
@@ -71,14 +72,14 @@ void TestMaxMessageLengthOnServerOnRequest(CoreEnd2endTest& test) {
   EXPECT_TRUE(client_close.was_cancelled());
   EXPECT_EQ(server_status.status(), GRPC_STATUS_RESOURCE_EXHAUSTED);
   EXPECT_EQ(server_status.message(),
-            "Received message larger than max (11 vs. 5)");
+            "SERVER: Received message larger than max (11 vs. 5)");
 }
 
 void TestMaxMessageLengthOnClientOnResponse(CoreEnd2endTest& test) {
   auto c = test.NewClientCall("/service/method").Create();
-  CoreEnd2endTest::IncomingStatusOnClient server_status;
-  CoreEnd2endTest::IncomingMetadata server_initial_metadata;
-  CoreEnd2endTest::IncomingMessage server_message;
+  IncomingStatusOnClient server_status;
+  IncomingMetadata server_initial_metadata;
+  IncomingMessage server_message;
   c.NewBatch(1)
       .SendInitialMetadata({})
       .SendCloseFromClient()
@@ -88,7 +89,7 @@ void TestMaxMessageLengthOnClientOnResponse(CoreEnd2endTest& test) {
   auto s = test.RequestCall(101);
   test.Expect(101, true);
   test.Step();
-  CoreEnd2endTest::IncomingCloseOnServer client_close;
+  IncomingCloseOnServer client_close;
   s.NewBatch(102)
       .SendInitialMetadata({})
       .RecvCloseOnServer(client_close)
@@ -100,14 +101,14 @@ void TestMaxMessageLengthOnClientOnResponse(CoreEnd2endTest& test) {
   EXPECT_EQ(s.method(), "/service/method");
   EXPECT_EQ(server_status.status(), GRPC_STATUS_RESOURCE_EXHAUSTED);
   EXPECT_EQ(server_status.message(),
-            "Received message larger than max (11 vs. 5)");
+            "CLIENT: Received message larger than max (11 vs. 5)");
 }
 
 void TestMaxMessageLengthOnServerOnResponse(CoreEnd2endTest& test) {
   auto c = test.NewClientCall("/service/method").Create();
-  CoreEnd2endTest::IncomingStatusOnClient server_status;
-  CoreEnd2endTest::IncomingMetadata server_initial_metadata;
-  CoreEnd2endTest::IncomingMessage server_message;
+  IncomingStatusOnClient server_status;
+  IncomingMetadata server_initial_metadata;
+  IncomingMessage server_message;
   c.NewBatch(1)
       .SendInitialMetadata({})
       .SendCloseFromClient()
@@ -117,7 +118,7 @@ void TestMaxMessageLengthOnServerOnResponse(CoreEnd2endTest& test) {
   auto s = test.RequestCall(101);
   test.Expect(101, true);
   test.Step();
-  CoreEnd2endTest::IncomingCloseOnServer client_close;
+  IncomingCloseOnServer client_close;
   s.NewBatch(102)
       .SendInitialMetadata({})
       .RecvCloseOnServer(client_close)
@@ -128,7 +129,8 @@ void TestMaxMessageLengthOnServerOnResponse(CoreEnd2endTest& test) {
   test.Step();
   EXPECT_EQ(s.method(), "/service/method");
   EXPECT_EQ(server_status.status(), GRPC_STATUS_RESOURCE_EXHAUSTED);
-  EXPECT_EQ(server_status.message(), "Sent message larger than max (11 vs. 5)");
+  EXPECT_EQ(server_status.message(),
+            "SERVER: Sent message larger than max (11 vs. 5)");
 }
 
 CORE_END2END_TEST(CoreEnd2endTest,
@@ -276,7 +278,7 @@ CORE_END2END_TEST(Http2Test, MaxMessageLengthOnServerOnRequestWithCompression) {
   EXPECT_TRUE(client_close.was_cancelled());
   EXPECT_EQ(server_status.status(), GRPC_STATUS_RESOURCE_EXHAUSTED);
   EXPECT_THAT(server_status.message(),
-              StartsWith("Received message larger than max"));
+              StartsWith("SERVER: Received message larger than max"));
 }
 
 CORE_END2END_TEST(Http2Test,
@@ -311,7 +313,7 @@ CORE_END2END_TEST(Http2Test,
   EXPECT_EQ(s.method(), "/service/method");
   EXPECT_EQ(server_status.status(), GRPC_STATUS_RESOURCE_EXHAUSTED);
   EXPECT_THAT(server_status.message(),
-              StartsWith("Received message larger than max"));
+              StartsWith("CLIENT: Received message larger than max"));
 }
 
 }  // namespace
