@@ -21,7 +21,6 @@
 #include <regex>
 #include <string>
 #include <utility>
-#include <vector>
 
 #include "absl/strings/str_cat.h"
 #include "absl/strings/string_view.h"
@@ -39,7 +38,8 @@
 void gpr_default_log(gpr_log_func_args* args);
 
 namespace grpc_core {
-
+// This test is currently broken.
+// The whole test will be re-written as a part of gpr to absl log conversion.
 class Verifier {
  public:
   Verifier() {
@@ -54,10 +54,12 @@ class Verifier {
           static_cast<gpr_log_severity>(GPR_LOG_SEVERITY_ERROR + 1);
     }
     grpc_tracer_set_enabled("all", 0);
-    gpr_set_log_verbosity(GPR_LOG_SEVERITY_DEBUG);
+    grpc_set_absl_verbosity_debug();
+    // This is broken. Replace with an absl log sink.
     gpr_set_log_function(DispatchLog);
   }
   ~Verifier() {
+    // This is broken. Replace with an absl log sink.
     gpr_set_log_function(gpr_default_log);
     saved_trace_flags_.Restore();
     gpr_set_log_verbosity(saved_severity_);
@@ -74,7 +76,9 @@ class Verifier {
   static void NoLog(gpr_log_func_args* args) {
     static const auto* const allowed_logs_by_module =
         new std::map<absl::string_view, std::regex>(
-            {{"cq_verifier.cc", std::regex("^Verify .* for [0-9]+ms")}});
+            {{"cq_verifier.cc", std::regex("^Verify .* for [0-9]+ms")},
+             {"chttp2_transport.cc",
+              std::regex("Sending goaway.*Channel Destroyed")}});
     absl::string_view filename = args->file;
     auto slash = filename.rfind('/');
     if (slash != absl::string_view::npos) {
@@ -86,7 +90,7 @@ class Verifier {
     }
     auto it = allowed_logs_by_module->find(filename);
     if (it != allowed_logs_by_module->end() &&
-        std::regex_match(args->message, it->second)) {
+        std::regex_search(args->message, it->second)) {
       gpr_default_log(args);
       return;
     }
@@ -112,8 +116,8 @@ std::atomic<gpr_log_func> Verifier::g_log_func_(gpr_default_log);
 void SimpleRequest(CoreEnd2endTest& test) {
   auto c = test.NewClientCall("/foo").Timeout(Duration::Seconds(5)).Create();
   EXPECT_NE(c.GetPeer(), absl::nullopt);
-  CoreEnd2endTest::IncomingMetadata server_initial_metadata;
-  CoreEnd2endTest::IncomingStatusOnClient server_status;
+  IncomingMetadata server_initial_metadata;
+  IncomingStatusOnClient server_status;
   c.NewBatch(1)
       .SendInitialMetadata({})
       .SendCloseFromClient()
@@ -124,7 +128,7 @@ void SimpleRequest(CoreEnd2endTest& test) {
   test.Step();
   EXPECT_NE(c.GetPeer(), absl::nullopt);
   EXPECT_NE(s.GetPeer(), absl::nullopt);
-  CoreEnd2endTest::IncomingCloseOnServer client_close;
+  IncomingCloseOnServer client_close;
   s.NewBatch(102)
       .SendInitialMetadata({})
       .SendStatusFromServer(GRPC_STATUS_UNIMPLEMENTED, "xyz", {})
@@ -139,6 +143,9 @@ void SimpleRequest(CoreEnd2endTest& test) {
 }
 
 CORE_END2END_TEST(NoLoggingTest, NoLoggingTest) {
+// This test makes sure that we dont get spammy logs when making an rpc
+// especially when rpcs are successful.
+
 // TODO(hork): remove when the listener flake is identified
 #ifdef GPR_WINDOWS
   if (IsEventEngineListenerEnabled()) {
