@@ -16,6 +16,7 @@
 
 #include <string.h>
 
+#include <memory>
 #include <new>
 
 #include "absl/status/status.h"
@@ -25,17 +26,14 @@
 #include <grpc/impl/channel_arg_names.h>
 #include <grpc/status.h>
 
-#include "src/core/lib/channel/channel_args.h"
 #include "src/core/lib/channel/channel_fwd.h"
 #include "src/core/lib/channel/channel_stack.h"
-#include "src/core/lib/channel/channel_stack_builder.h"
 #include "src/core/lib/config/core_configuration.h"
 #include "src/core/lib/gprpp/status_helper.h"
 #include "src/core/lib/gprpp/time.h"
 #include "src/core/lib/iomgr/call_combiner.h"
 #include "src/core/lib/iomgr/closure.h"
 #include "src/core/lib/iomgr/error.h"
-#include "src/core/lib/surface/channel_init.h"
 #include "src/core/lib/surface/channel_stack_type.h"
 #include "src/core/lib/transport/metadata_batch.h"
 #include "src/core/lib/transport/transport.h"
@@ -116,7 +114,6 @@ class FailFirstTenCallsFilter {
 
 grpc_channel_filter FailFirstTenCallsFilter::kFilterVtable = {
     CallData::StartTransportStreamOpBatch,
-    nullptr,
     grpc_channel_next_op,
     sizeof(CallData),
     CallData::Init,
@@ -133,19 +130,11 @@ grpc_channel_filter FailFirstTenCallsFilter::kFilterVtable = {
 // Tests transparent retries when the call was never sent out on the wire.
 CORE_END2END_TEST(RetryTest, RetryTransparentNotSentOnWire) {
   CoreConfiguration::RegisterBuilder([](CoreConfiguration::Builder* builder) {
-    builder->channel_init()->RegisterStage(
-        GRPC_CLIENT_SUBCHANNEL, GRPC_CHANNEL_INIT_BUILTIN_PRIORITY + 1,
-        [](ChannelStackBuilder* builder) {
-          // Skip on proxy (which explicitly disables retries).
-          if (!builder->channel_args()
-                   .GetBool(GRPC_ARG_ENABLE_RETRIES)
-                   .value_or(true)) {
-            return true;
-          }
-          // Install filter.
-          builder->PrependFilter(&FailFirstTenCallsFilter::kFilterVtable);
-          return true;
-        });
+    builder->channel_init()
+        ->RegisterFilter(GRPC_CLIENT_SUBCHANNEL,
+                         &FailFirstTenCallsFilter::kFilterVtable)
+        // Skip on proxy (which explicitly disables retries).
+        .IfChannelArg(GRPC_ARG_ENABLE_RETRIES, true);
   });
   auto c =
       NewClientCall("/service/method").Timeout(Duration::Minutes(1)).Create();
