@@ -21,9 +21,10 @@
 #include <algorithm>
 #include <cerrno>
 #include <cstring>
-#include <initializer_list>
 #include <memory>
 
+#include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "absl/status/status.h"
 #include "absl/strings/str_cat.h"
 #include "absl/strings/str_format.h"
@@ -32,7 +33,6 @@
 
 #include <grpc/event_engine/event_engine.h>
 #include <grpc/support/alloc.h>
-#include <grpc/support/log.h>
 
 #include "src/core/lib/address_utils/sockaddr_utils.h"
 #include "src/core/lib/gprpp/crash.h"
@@ -139,7 +139,7 @@ std::string ReadBytes(int sockfd, int& saved_errno, int num_expected_bytes) {
                               num_expected_bytes - read_data.length());
     if (saved_errno == EAGAIN &&
         read_data.length() < static_cast<size_t>(num_expected_bytes)) {
-      GPR_ASSERT(BlockUntilReadable(sockfd).ok());
+      CHECK_OK(BlockUntilReadable(sockfd));
     } else if (saved_errno != 0 && num_expected_bytes > 0) {
       read_data.clear();
       break;
@@ -178,10 +178,10 @@ int WriteBytes(int sockfd, int& saved_errno, std::string write_bytes) {
     saved_errno = 0;
     ret = TryWriteBytes(sockfd, saved_errno, write_bytes);
     if (saved_errno == EAGAIN && ret < static_cast<int>(write_bytes.length())) {
-      GPR_ASSERT(ret >= 0);
-      GPR_ASSERT(BlockUntilWritable(sockfd).ok());
+      CHECK_GE(ret, 0);
+      CHECK_OK(BlockUntilWritable(sockfd));
     } else if (saved_errno != 0) {
-      GPR_ASSERT(ret < 0);
+      CHECK_LT(ret, 0);
       return ret;
     }
     write_bytes = write_bytes.substr(ret, std::string::npos);
@@ -234,7 +234,7 @@ PosixOracleEndpoint::~PosixOracleEndpoint() {
 bool PosixOracleEndpoint::Read(absl::AnyInvocable<void(absl::Status)> on_read,
                                SliceBuffer* buffer, const ReadArgs* args) {
   grpc_core::MutexLock lock(&mu_);
-  GPR_ASSERT(buffer != nullptr);
+  CHECK_NE(buffer, nullptr);
   int read_hint_bytes =
       args != nullptr ? std::max(1, static_cast<int>(args->read_hint_bytes))
                       : 0;
@@ -248,14 +248,14 @@ bool PosixOracleEndpoint::Write(
     absl::AnyInvocable<void(absl::Status)> on_writable, SliceBuffer* data,
     const WriteArgs* /*args*/) {
   grpc_core::MutexLock lock(&mu_);
-  GPR_ASSERT(data != nullptr);
+  CHECK_NE(data, nullptr);
   write_ops_channel_ = WriteOperation(data, std::move(on_writable));
   write_op_signal_->Notify();
   return false;
 }
 
 void PosixOracleEndpoint::ProcessReadOperations() {
-  gpr_log(GPR_INFO, "Starting thread to process read ops ...");
+  LOG(INFO) << "Starting thread to process read ops ...";
   while (true) {
     read_op_signal_->WaitForNotification();
     read_op_signal_ = std::make_unique<grpc_core::Notification>();
@@ -273,11 +273,11 @@ void PosixOracleEndpoint::ProcessReadOperations() {
                                               grpc_core::StrError(saved_errno)))
                            : absl::OkStatus());
   }
-  gpr_log(GPR_INFO, "Shutting down read ops thread ...");
+  LOG(INFO) << "Shutting down read ops thread ...";
 }
 
 void PosixOracleEndpoint::ProcessWriteOperations() {
-  gpr_log(GPR_INFO, "Starting thread to process write ops ...");
+  LOG(INFO) << "Starting thread to process write ops ...";
   while (true) {
     write_op_signal_->WaitForNotification();
     write_op_signal_ = std::make_unique<grpc_core::Notification>();
@@ -293,7 +293,7 @@ void PosixOracleEndpoint::ProcessWriteOperations() {
                                         grpc_core::StrError(saved_errno)))
                      : absl::OkStatus());
   }
-  gpr_log(GPR_INFO, "Shutting down write ops thread ...");
+  LOG(INFO) << "Shutting down write ops thread ...";
 }
 
 PosixOracleListener::PosixOracleListener(
@@ -311,7 +311,7 @@ PosixOracleListener::PosixOracleListener(
 
 absl::Status PosixOracleListener::Start() {
   grpc_core::MutexLock lock(&mu_);
-  GPR_ASSERT(!listener_fds_.empty());
+  CHECK(!listener_fds_.empty());
   if (std::exchange(is_started_, true)) {
     return absl::InternalError("Cannot start listener more than once ...");
   }
@@ -335,14 +335,14 @@ PosixOracleListener::~PosixOracleListener() {
     shutdown(listener_fds_[i], SHUT_RDWR);
   }
   // Send a STOP message over the pipe.
-  GPR_ASSERT(write(pipefd_[1], kStopMessage, strlen(kStopMessage)) != -1);
+  CHECK(write(pipefd_[1], kStopMessage, strlen(kStopMessage)) != -1);
   serve_.Join();
   on_shutdown_(absl::OkStatus());
 }
 
 void PosixOracleListener::HandleIncomingConnections() {
-  gpr_log(GPR_INFO, "Starting accept thread ...");
-  GPR_ASSERT(!listener_fds_.empty());
+  LOG(INFO) << "Starting accept thread ...";
+  CHECK(!listener_fds_.empty());
   int nfds = listener_fds_.size();
   // Add one extra file descriptor to poll the pipe fd.
   ++nfds;
@@ -371,17 +371,16 @@ void PosixOracleListener::HandleIncomingConnections() {
       // pfds[i].fd has a readable event.
       int client_sock_fd = accept(pfds[i].fd, nullptr, nullptr);
       if (client_sock_fd < 0) {
-        gpr_log(GPR_ERROR,
-                "Error accepting new connection: %s. Ignoring connection "
-                "attempt ...",
-                grpc_core::StrError(errno).c_str());
+        LOG(ERROR) << "Error accepting new connection: "
+                   << grpc_core::StrError(errno)
+                   << ". Ignoring connection attempt ...";
         continue;
       }
       on_accept_(PosixOracleEndpoint::Create(client_sock_fd),
                  memory_allocator_factory_->CreateMemoryAllocator("test"));
     }
   }
-  gpr_log(GPR_INFO, "Shutting down accept thread ...");
+  LOG(INFO) << "Shutting down accept thread ...";
   gpr_free(pfds);
 }
 
