@@ -19,15 +19,15 @@
 #include <stdint.h>
 #include <string.h>
 
+#include "absl/log/check.h"
 #include "absl/status/statusor.h"
 #include "absl/strings/string_view.h"
 
 #include <grpc/support/json.h>
 #include <grpc/support/log.h>
 
-#include "src/core/lib/json/json.h"
-#include "src/core/lib/json/json_reader.h"
-#include "src/core/lib/json/json_writer.h"
+#include "src/core/util/json/json_reader.h"
+#include "src/core/util/json/json_writer.h"
 
 bool squelch = true;
 bool leak_check = true;
@@ -38,8 +38,8 @@ extern "C" int LLVMFuzzerTestOneInput(const uint8_t* data, size_t size) {
   if (json.ok()) {
     auto text2 = grpc_core::JsonDump(*json);
     auto json2 = grpc_core::JsonParse(text2);
-    GPR_ASSERT(json2.ok());
-    GPR_ASSERT(*json == *json2);
+    CHECK(json2.ok());
+    CHECK(*json == *json2);
   }
   return 0;
 }
