@@ -16,28 +16,31 @@
 //
 //
 
-#include <grpc/support/port_platform.h>
-
 #include "src/core/lib/slice/slice.h"
 
 #include <inttypes.h>
 #include <string.h>
 
 #include <algorithm>
+#include <functional>
 #include <memory>
 #include <random>
 #include <string>
 #include <vector>
 
+#include "absl/log/log.h"
 #include "absl/strings/string_view.h"
 #include "gtest/gtest.h"
 
 #include <grpc/slice.h>
 #include <grpc/support/log.h>
+#include <grpc/support/port_platform.h>
 
 #include "src/core/lib/gprpp/memory.h"
+#include "src/core/lib/gprpp/no_destruct.h"
 #include "src/core/lib/slice/slice_internal.h"
-#include "test/core/util/build.h"
+#include "src/core/lib/slice/slice_refcount.h"
+#include "test/core/test_util/build.h"
 
 TEST(GrpcSliceTest, MallocReturnsSomethingSensible) {
   // Calls grpc_slice_create for various lengths and verifies the internals for
@@ -180,7 +183,7 @@ TEST_P(GrpcSliceSizedTest, SliceSplitHeadWorks) {
   grpc_slice head, tail;
   size_t i;
 
-  gpr_log(GPR_INFO, "length=%" PRIuPTR, length);
+  LOG(INFO) << "length=" << length;
 
   // Create a slice in which each byte is equal to the distance from it to the
   // beginning of the slice.
@@ -209,7 +212,7 @@ TEST_P(GrpcSliceSizedTest, SliceSplitTailWorks) {
   grpc_slice head, tail;
   size_t i;
 
-  gpr_log(GPR_INFO, "length=%" PRIuPTR, length);
+  LOG(INFO) << "length=" << length;
 
   // Create a slice in which each byte is equal to the distance from it to the
   // beginning of the slice.
@@ -347,10 +350,40 @@ INSTANTIATE_TEST_SUITE_P(SliceSizedTest, SliceSizedTest,
                            }
                            return out;
                          }()),
-                         [](const testing::TestParamInfo<size_t>& info) {
+                         [](const ::testing::TestParamInfo<size_t>& info) {
                            return std::to_string(info.param);
                          });
 
+class TakeUniquelyOwnedTest
+    : public ::testing::TestWithParam<std::function<Slice()>> {};
+
+TEST_P(TakeUniquelyOwnedTest, TakeUniquelyOwned) {
+  auto owned = GetParam()().TakeUniquelyOwned();
+  auto* refcount = owned.c_slice().refcount;
+  if (refcount != nullptr && refcount != grpc_slice_refcount::NoopRefcount()) {
+    EXPECT_TRUE(refcount->IsUnique());
+  }
+}
+
+INSTANTIATE_TEST_SUITE_P(
+    TakeUniquelyOwnedTest, TakeUniquelyOwnedTest,
+    ::testing::Values(
+        []() {
+          static const NoDestruct<std::string> big('a', 1024);
+          return Slice::FromStaticBuffer(big->data(), big->size());
+        },
+        []() {
+          static const NoDestruct<std::string> big('a', 1024);
+          return Slice::FromCopiedBuffer(big->data(), big->size());
+        },
+        []() {
+          static const NoDestruct<std::string> big('a', 1024);
+          static const NoDestruct<Slice> big_slice(
+              Slice::FromCopiedBuffer(big->data(), big->size()));
+          return big_slice->Ref();
+        },
+        []() { return Slice::FromStaticString("hello"); }));
+
 size_t SumSlice(const Slice& slice) {
   size_t x = 0;
   for (size_t i = 0; i < slice.size(); ++i) {
@@ -441,28 +474,6 @@ TEST(SliceTest, SliceCastWorks) {
   EXPECT_EQ(&other, &test);
 }
 
-TEST(SliceTest, MutableSliceCastWorks) {
-  using ::grpc_event_engine::experimental::internal::SliceCast;
-  Slice test = Slice::FromCopiedString("hello world!");
-  grpc_slice& slice = SliceCast<grpc_slice>(test);
-  EXPECT_EQ(&slice, &test.c_slice());
-  slice = grpc_slice_from_static_string("goodbye world!");
-  EXPECT_EQ(test.as_string_view(), "goodbye world!");
-
-  MutableSlice& m_cpp_slice = SliceCast<MutableSlice>(test);
-  EXPECT_EQ(&m_cpp_slice.c_slice(), &test.c_slice());
-  m_cpp_slice = MutableSlice::FromCopiedString("hello world again!");
-  // Change the first byte.
-  m_cpp_slice[0] = 'e';
-  EXPECT_EQ(test.as_string_view(), "eello world again!");
-
-  MutableSlice& m_c_slice = SliceCast<MutableSlice>(slice);
-  EXPECT_EQ(&m_c_slice.c_slice(), &slice);
-  // Restore the first byte.
-  GRPC_SLICE_START_PTR(slice)[0] = 'h';
-  EXPECT_EQ(m_c_slice.as_string_view(), "hello world again!");
-}
-
 }  // namespace
 }  // namespace grpc_core
 
