@@ -18,6 +18,9 @@
 
 #include <string.h>
 
+#include <memory>
+
+#include "absl/log/log.h"
 #include "gtest/gtest.h"
 
 #include <grpc/byte_buffer.h>
@@ -25,11 +28,8 @@
 #include <grpc/compression.h>
 #include <grpc/grpc.h>
 #include <grpc/slice.h>
-#include <grpc/support/log.h>
 
-#include "test/core/util/test_config.h"
-
-#define LOG_TEST(x) gpr_log(GPR_INFO, "%s", x)
+#include "test/core/test_util/test_config.h"
 
 TEST(GrpcByteBufferReaderTest, TestReadOneSlice) {
   grpc_slice slice;
@@ -38,7 +38,7 @@ TEST(GrpcByteBufferReaderTest, TestReadOneSlice) {
   grpc_slice first_slice, second_slice;
   int first_code, second_code;
 
-  LOG_TEST("test_read_one_slice");
+  LOG(INFO) << "test_read_one_slice";
   slice = grpc_slice_from_copied_string("test");
   buffer = grpc_raw_byte_buffer_create(&slice, 1);
   grpc_slice_unref(slice);
@@ -60,7 +60,7 @@ TEST(GrpcByteBufferReaderTest, TestReadOneSliceMalloc) {
   grpc_slice first_slice, second_slice;
   int first_code, second_code;
 
-  LOG_TEST("test_read_one_slice_malloc");
+  LOG(INFO) << "test_read_one_slice_malloc";
   slice = grpc_slice_malloc(4);
   memcpy(GRPC_SLICE_START_PTR(slice), "test", 4);
   buffer = grpc_raw_byte_buffer_create(&slice, 1);
@@ -83,7 +83,7 @@ TEST(GrpcByteBufferReaderTest, TestReadNoneCompressedSlice) {
   grpc_slice first_slice, second_slice;
   int first_code, second_code;
 
-  LOG_TEST("test_read_none_compressed_slice");
+  LOG(INFO) << "test_read_none_compressed_slice";
   slice = grpc_slice_from_copied_string("test");
   buffer = grpc_raw_byte_buffer_create(&slice, 1);
   grpc_slice_unref(slice);
@@ -106,7 +106,7 @@ TEST(GrpcByteBufferReaderTest, TestPeekOneSlice) {
   grpc_slice* second_slice;
   int first_code, second_code;
 
-  LOG_TEST("test_peek_one_slice");
+  LOG(INFO) << "test_peek_one_slice";
   slice = grpc_slice_from_copied_string("test");
   buffer = grpc_raw_byte_buffer_create(&slice, 1);
   grpc_slice_unref(slice);
@@ -128,7 +128,7 @@ TEST(GrpcByteBufferReaderTest, TestPeekOneSliceMalloc) {
   grpc_slice* second_slice;
   int first_code, second_code;
 
-  LOG_TEST("test_peek_one_slice_malloc");
+  LOG(INFO) << "test_peek_one_slice_malloc";
   slice = grpc_slice_malloc(4);
   memcpy(GRPC_SLICE_START_PTR(slice), "test", 4);
   buffer = grpc_raw_byte_buffer_create(&slice, 1);
@@ -151,7 +151,7 @@ TEST(GrpcByteBufferReaderTest, TestPeekNoneCompressedSlice) {
   grpc_slice* second_slice;
   int first_code, second_code;
 
-  LOG_TEST("test_peek_none_compressed_slice");
+  LOG(INFO) << "test_peek_none_compressed_slice";
   slice = grpc_slice_from_copied_string("test");
   buffer = grpc_raw_byte_buffer_create(&slice, 1);
   grpc_slice_unref(slice);
@@ -170,7 +170,7 @@ TEST(GrpcByteBufferReaderTest, TestByteBufferFromReader) {
   grpc_byte_buffer *buffer, *buffer_from_reader;
   grpc_byte_buffer_reader reader;
 
-  LOG_TEST("test_byte_buffer_from_reader");
+  LOG(INFO) << "test_byte_buffer_from_reader";
   slice = grpc_slice_malloc(4);
   memcpy(GRPC_SLICE_START_PTR(slice), "test", 4);
   buffer = grpc_raw_byte_buffer_create(&slice, 1);
@@ -199,7 +199,7 @@ TEST(GrpcByteBufferReaderTest, TestReadall) {
   grpc_byte_buffer_reader reader;
   grpc_slice slice_out;
 
-  LOG_TEST("test_readall");
+  LOG(INFO) << "test_readall";
 
   memset(lotsa_as, 'a', 512 * sizeof(lotsa_as[0]));
   memset(lotsa_bs, 'b', 1024 * sizeof(lotsa_bs[0]));
@@ -233,7 +233,7 @@ TEST(GrpcByteBufferReaderTest, TestByteBufferCopy) {
   grpc_byte_buffer_reader reader;
   grpc_slice slice_out;
 
-  LOG_TEST("test_byte_buffer_copy");
+  LOG(INFO) << "test_byte_buffer_copy";
 
   memset(lotsa_as, 'a', 512 * sizeof(lotsa_as[0]));
   memset(lotsa_bs, 'b', 1024 * sizeof(lotsa_bs[0]));
