@@ -19,6 +19,8 @@
 #include "gmock/gmock.h"
 #include "gtest/gtest.h"
 
+#include "test/core/promise/poll_matcher.h"
+
 using testing::Mock;
 using testing::StrictMock;
 
@@ -57,34 +59,6 @@ class MockActivity : public Activity, public Wakeable {
   std::unique_ptr<ScopedActivity> scoped_activity_;
 };
 
-MATCHER(IsPending, "") {
-  if (arg.ready()) {
-    *result_listener << "is ready";
-    return false;
-  }
-  return true;
-}
-
-MATCHER(IsReady, "") {
-  if (arg.pending()) {
-    *result_listener << "is pending";
-    return false;
-  }
-  return true;
-}
-
-MATCHER_P(IsReady, value, "") {
-  if (arg.pending()) {
-    *result_listener << "is pending";
-    return false;
-  }
-  if (arg.value() != value) {
-    *result_listener << "is " << ::testing::PrintToString(arg.value());
-    return false;
-  }
-  return true;
-}
-
 }  // namespace
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -119,7 +93,7 @@ namespace filters_detail {
 
 TEST(StackDataTest, Empty) {
   StackData d;
-  EXPECT_EQ(d.call_data_alignment, 0u);
+  EXPECT_EQ(d.call_data_alignment, 1u);
   EXPECT_EQ(d.call_data_size, 0u);
 }
 
@@ -354,10 +328,7 @@ TEST(StackDataTest, InstantClientInitialMetadataReturningVoid) {
   EXPECT_EQ(d.client_initial_metadata.ops[0].poll, nullptr);
   EXPECT_EQ(d.client_initial_metadata.ops[0].early_destroy, nullptr);
   // Check promise init
-  auto memory_allocator =
-      MakeMemoryQuota("test-quota")->CreateMemoryAllocator("foo");
-  auto arena = MakeScopedArena(1024, &memory_allocator);
-  auto md = Arena::MakePooled<ClientMetadata>(arena.get());
+  auto md = Arena::MakePooled<ClientMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   char call_data;
   auto r = d.client_initial_metadata.ops[0].promise_init(
@@ -392,10 +363,7 @@ TEST(StackDataTest, InstantClientInitialMetadataReturningVoidTakingChannelPtr) {
   EXPECT_EQ(d.client_initial_metadata.ops[0].poll, nullptr);
   EXPECT_EQ(d.client_initial_metadata.ops[0].early_destroy, nullptr);
   // Check promise init
-  auto memory_allocator =
-      MakeMemoryQuota("test-quota")->CreateMemoryAllocator("foo");
-  auto arena = MakeScopedArena(1024, &memory_allocator);
-  auto md = Arena::MakePooled<ClientMetadata>(arena.get());
+  auto md = Arena::MakePooled<ClientMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   char call_data;
   auto r = d.client_initial_metadata.ops[0].promise_init(
@@ -439,10 +407,8 @@ TEST(StackDataTest, InstantClientInitialMetadataReturningAbslStatus) {
   // Check promise init
   void* call_data = gpr_malloc_aligned(d.call_data_size, d.call_data_alignment);
   d.filter_constructor[0].call_init(call_data, &f1);
-  auto memory_allocator =
-      MakeMemoryQuota("test-quota")->CreateMemoryAllocator("foo");
-  auto arena = MakeScopedArena(1024, &memory_allocator);
-  auto md = Arena::MakePooled<ClientMetadata>(arena.get());
+  auto arena = SimpleArenaAllocator()->MakeArena();
+  auto md = Arena::MakePooled<ClientMetadata>();
   promise_detail::Context<Arena> ctx(arena.get());
   // A succeeding call
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
@@ -453,7 +419,7 @@ TEST(StackDataTest, InstantClientInitialMetadataReturningAbslStatus) {
   EXPECT_EQ(r.value().ok->get_pointer(HttpPathMetadata())->as_string_view(),
             "hello");
   // A failing call
-  md = Arena::MakePooled<ClientMetadata>(arena.get());
+  md = Arena::MakePooled<ClientMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   r = d.client_initial_metadata.ops[0].promise_init(
       nullptr, call_data, d.client_initial_metadata.ops[0].channel_data,
@@ -499,10 +465,8 @@ TEST(StackDataTest,
   // Check promise init
   void* call_data = gpr_malloc_aligned(d.call_data_size, d.call_data_alignment);
   d.filter_constructor[0].call_init(call_data, &f1);
-  auto memory_allocator =
-      MakeMemoryQuota("test-quota")->CreateMemoryAllocator("foo");
-  auto arena = MakeScopedArena(1024, &memory_allocator);
-  auto md = Arena::MakePooled<ClientMetadata>(arena.get());
+  auto arena = SimpleArenaAllocator()->MakeArena();
+  auto md = Arena::MakePooled<ClientMetadata>();
   promise_detail::Context<Arena> ctx(arena.get());
   // A succeeding call
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
@@ -513,7 +477,7 @@ TEST(StackDataTest,
   EXPECT_EQ(r.value().ok->get_pointer(HttpPathMetadata())->as_string_view(),
             "hello");
   // A failing call
-  md = Arena::MakePooled<ClientMetadata>(arena.get());
+  md = Arena::MakePooled<ClientMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   r = d.client_initial_metadata.ops[0].promise_init(
       nullptr, call_data, d.client_initial_metadata.ops[0].channel_data,
@@ -558,10 +522,8 @@ TEST(StackDataTest, InstantClientInitialMetadataReturningServerMetadata) {
   // Check promise init
   void* call_data = gpr_malloc_aligned(d.call_data_size, d.call_data_alignment);
   d.filter_constructor[0].call_init(call_data, &f1);
-  auto memory_allocator =
-      MakeMemoryQuota("test-quota")->CreateMemoryAllocator("foo");
-  auto arena = MakeScopedArena(1024, &memory_allocator);
-  auto md = Arena::MakePooled<ClientMetadata>(arena.get());
+  auto arena = SimpleArenaAllocator()->MakeArena();
+  auto md = Arena::MakePooled<ClientMetadata>();
   promise_detail::Context<Arena> ctx(arena.get());
   // A succeeding call
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
@@ -572,7 +534,7 @@ TEST(StackDataTest, InstantClientInitialMetadataReturningServerMetadata) {
   EXPECT_EQ(r.value().ok->get_pointer(HttpPathMetadata())->as_string_view(),
             "hello");
   // A failing call
-  md = Arena::MakePooled<ClientMetadata>(arena.get());
+  md = Arena::MakePooled<ClientMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   r = d.client_initial_metadata.ops[0].promise_init(
       nullptr, call_data, d.client_initial_metadata.ops[0].channel_data,
@@ -620,10 +582,8 @@ TEST(StackDataTest,
   // Check promise init
   void* call_data = gpr_malloc_aligned(d.call_data_size, d.call_data_alignment);
   d.filter_constructor[0].call_init(call_data, &f1);
-  auto memory_allocator =
-      MakeMemoryQuota("test-quota")->CreateMemoryAllocator("foo");
-  auto arena = MakeScopedArena(1024, &memory_allocator);
-  auto md = Arena::MakePooled<ClientMetadata>(arena.get());
+  auto arena = SimpleArenaAllocator()->MakeArena();
+  auto md = Arena::MakePooled<ClientMetadata>();
   promise_detail::Context<Arena> ctx(arena.get());
   // A succeeding call
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
@@ -634,7 +594,7 @@ TEST(StackDataTest,
   EXPECT_EQ(r.value().ok->get_pointer(HttpPathMetadata())->as_string_view(),
             "hello");
   // A failing call
-  md = Arena::MakePooled<ClientMetadata>(arena.get());
+  md = Arena::MakePooled<ClientMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   r = d.client_initial_metadata.ops[0].promise_init(
       nullptr, call_data, d.client_initial_metadata.ops[0].channel_data,
@@ -680,10 +640,8 @@ TEST(StackDataTest, PromiseClientInitialMetadataReturningAbslStatus) {
   // Check promise init
   void* call_data = gpr_malloc_aligned(d.call_data_size, d.call_data_alignment);
   d.filter_constructor[0].call_init(call_data, &f1);
-  auto memory_allocator =
-      MakeMemoryQuota("test-quota")->CreateMemoryAllocator("foo");
-  auto arena = MakeScopedArena(1024, &memory_allocator);
-  auto md = Arena::MakePooled<ClientMetadata>(arena.get());
+  auto arena = SimpleArenaAllocator()->MakeArena();
+  auto md = Arena::MakePooled<ClientMetadata>();
   promise_detail::Context<Arena> ctx(arena.get());
   // A succeeding call
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
@@ -701,7 +659,7 @@ TEST(StackDataTest, PromiseClientInitialMetadataReturningAbslStatus) {
   EXPECT_EQ(r.value().ok->get_pointer(HttpPathMetadata())->as_string_view(),
             "hello");
   // A failing call
-  md = Arena::MakePooled<ClientMetadata>(arena.get());
+  md = Arena::MakePooled<ClientMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   r = d.client_initial_metadata.ops[0].promise_init(
       promise_data, call_data, d.client_initial_metadata.ops[0].channel_data,
@@ -714,7 +672,7 @@ TEST(StackDataTest, PromiseClientInitialMetadataReturningAbslStatus) {
   EXPECT_EQ(r.value().ok, nullptr);
   EXPECT_EQ(r.value().error->get(GrpcStatusMetadata()), GRPC_STATUS_CANCELLED);
   // A cancelled call
-  md = Arena::MakePooled<ClientMetadata>(arena.get());
+  md = Arena::MakePooled<ClientMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   r = d.client_initial_metadata.ops[0].promise_init(
       promise_data, call_data, d.client_initial_metadata.ops[0].channel_data,
@@ -763,10 +721,8 @@ TEST(StackDataTest,
   // Check promise init
   void* call_data = gpr_malloc_aligned(d.call_data_size, d.call_data_alignment);
   d.filter_constructor[0].call_init(call_data, &f1);
-  auto memory_allocator =
-      MakeMemoryQuota("test-quota")->CreateMemoryAllocator("foo");
-  auto arena = MakeScopedArena(1024, &memory_allocator);
-  auto md = Arena::MakePooled<ClientMetadata>(arena.get());
+  auto arena = SimpleArenaAllocator()->MakeArena();
+  auto md = Arena::MakePooled<ClientMetadata>();
   promise_detail::Context<Arena> ctx(arena.get());
   // A succeeding call
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
@@ -784,7 +740,7 @@ TEST(StackDataTest,
   EXPECT_EQ(r.value().ok->get_pointer(HttpPathMetadata())->as_string_view(),
             "hello");
   // A failing call
-  md = Arena::MakePooled<ClientMetadata>(arena.get());
+  md = Arena::MakePooled<ClientMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   r = d.client_initial_metadata.ops[0].promise_init(
       promise_data, call_data, d.client_initial_metadata.ops[0].channel_data,
@@ -797,7 +753,7 @@ TEST(StackDataTest,
   EXPECT_EQ(r.value().ok, nullptr);
   EXPECT_EQ(r.value().error->get(GrpcStatusMetadata()), GRPC_STATUS_CANCELLED);
   // A cancelled call
-  md = Arena::MakePooled<ClientMetadata>(arena.get());
+  md = Arena::MakePooled<ClientMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   r = d.client_initial_metadata.ops[0].promise_init(
       promise_data, call_data, d.client_initial_metadata.ops[0].channel_data,
@@ -833,10 +789,8 @@ TEST(StackDataTest, InstantServerInitialMetadataReturningVoid) {
   EXPECT_EQ(d.server_initial_metadata.ops[0].poll, nullptr);
   EXPECT_EQ(d.server_initial_metadata.ops[0].early_destroy, nullptr);
   // Check promise init
-  auto memory_allocator =
-      MakeMemoryQuota("test-quota")->CreateMemoryAllocator("foo");
-  auto arena = MakeScopedArena(1024, &memory_allocator);
-  auto md = Arena::MakePooled<ServerMetadata>(arena.get());
+  auto arena = SimpleArenaAllocator()->MakeArena();
+  auto md = Arena::MakePooled<ServerMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   char call_data;
   auto r = d.server_initial_metadata.ops[0].promise_init(
@@ -870,9 +824,7 @@ TEST(StackDataTest, InstantClientToServerMessagesReturningVoid) {
   EXPECT_EQ(d.client_to_server_messages.ops[0].poll, nullptr);
   EXPECT_EQ(d.client_to_server_messages.ops[0].early_destroy, nullptr);
   // Check promise init
-  auto memory_allocator =
-      MakeMemoryQuota("test-quota")->CreateMemoryAllocator("foo");
-  auto arena = MakeScopedArena(1024, &memory_allocator);
+  auto arena = SimpleArenaAllocator()->MakeArena();
   auto message = Arena::MakePooled<Message>(SliceBuffer(), 0);
   char call_data;
   auto r = d.client_to_server_messages.ops[0].promise_init(
@@ -905,9 +857,7 @@ TEST(StackDataTest, InstantServerToClientMessagesReturningVoid) {
   EXPECT_EQ(d.server_to_client_messages.ops[0].poll, nullptr);
   EXPECT_EQ(d.server_to_client_messages.ops[0].early_destroy, nullptr);
   // Check promise init
-  auto memory_allocator =
-      MakeMemoryQuota("test-quota")->CreateMemoryAllocator("foo");
-  auto arena = MakeScopedArena(1024, &memory_allocator);
+  auto arena = SimpleArenaAllocator()->MakeArena();
   auto message = Arena::MakePooled<Message>(SliceBuffer(), 0);
   char call_data;
   auto r = d.server_to_client_messages.ops[0].promise_init(
@@ -940,10 +890,8 @@ TEST(StackDataTest, InstantServerTrailingMetadataReturningVoid) {
   EXPECT_EQ(d.server_trailing_metadata.ops[0].poll, nullptr);
   EXPECT_EQ(d.server_trailing_metadata.ops[0].early_destroy, nullptr);
   // Check promise init
-  auto memory_allocator =
-      MakeMemoryQuota("test-quota")->CreateMemoryAllocator("foo");
-  auto arena = MakeScopedArena(1024, &memory_allocator);
-  auto md = Arena::MakePooled<ServerMetadata>(arena.get());
+  auto arena = SimpleArenaAllocator()->MakeArena();
+  auto md = Arena::MakePooled<ServerMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   char call_data;
   auto r = d.server_trailing_metadata.ops[0].promise_init(
@@ -980,10 +928,8 @@ TEST(StackDataTest,
   EXPECT_EQ(d.server_trailing_metadata.ops[0].poll, nullptr);
   EXPECT_EQ(d.server_trailing_metadata.ops[0].early_destroy, nullptr);
   // Check promise init
-  auto memory_allocator =
-      MakeMemoryQuota("test-quota")->CreateMemoryAllocator("foo");
-  auto arena = MakeScopedArena(1024, &memory_allocator);
-  auto md = Arena::MakePooled<ServerMetadata>(arena.get());
+  auto arena = SimpleArenaAllocator()->MakeArena();
+  auto md = Arena::MakePooled<ServerMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   char call_data;
   auto r = d.server_trailing_metadata.ops[0].promise_init(
@@ -1070,12 +1016,10 @@ TEST(OperationExecutorTest, InstantTwo) {
   d.filter_constructor[0].call_init(call_data1, &f1);
   d.filter_constructor[1].call_init(call_data2, &f2);
   OperationExecutor<ClientMetadataHandle> transformer;
-  auto memory_allocator =
-      MakeMemoryQuota("test-quota")->CreateMemoryAllocator("foo");
-  auto arena = MakeScopedArena(1024, &memory_allocator);
+  auto arena = SimpleArenaAllocator()->MakeArena();
   promise_detail::Context<Arena> ctx(arena.get());
   // First call succeeds
-  auto md = Arena::MakePooled<ServerMetadata>(arena.get());
+  auto md = Arena::MakePooled<ServerMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   auto r =
       transformer.Start(&d.client_initial_metadata, std::move(md), call_data1);
@@ -1083,7 +1027,7 @@ TEST(OperationExecutorTest, InstantTwo) {
   EXPECT_EQ(r.value().ok->get_pointer(HttpPathMetadata())->as_string_view(),
             "world");
   // Second fails
-  md = Arena::MakePooled<ServerMetadata>(arena.get());
+  md = Arena::MakePooled<ServerMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   r = transformer.Start(&d.client_initial_metadata, std::move(md), call_data1);
   EXPECT_TRUE(r.ready());
@@ -1135,12 +1079,10 @@ TEST(OperationExecutorTest, PromiseTwo) {
   d.filter_constructor[0].call_init(call_data1, &f1);
   d.filter_constructor[1].call_init(call_data2, &f2);
   OperationExecutor<ClientMetadataHandle> transformer;
-  auto memory_allocator =
-      MakeMemoryQuota("test-quota")->CreateMemoryAllocator("foo");
-  auto arena = MakeScopedArena(1024, &memory_allocator);
+  auto arena = SimpleArenaAllocator()->MakeArena();
   promise_detail::Context<Arena> ctx(arena.get());
   // First call succeeds after two sets of two step delays.
-  auto md = Arena::MakePooled<ServerMetadata>(arena.get());
+  auto md = Arena::MakePooled<ServerMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   auto r =
       transformer.Start(&d.client_initial_metadata, std::move(md), call_data1);
@@ -1156,7 +1098,7 @@ TEST(OperationExecutorTest, PromiseTwo) {
   EXPECT_EQ(r.value().ok->get_pointer(HttpPathMetadata())->as_string_view(),
             "world");
   // Second fails after one set of two step delays.
-  md = Arena::MakePooled<ServerMetadata>(arena.get());
+  md = Arena::MakePooled<ServerMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   r = transformer.Start(&d.client_initial_metadata, std::move(md), call_data1);
   EXPECT_FALSE(r.ready());
@@ -1207,11 +1149,9 @@ TEST(InfallibleOperationExecutor, InstantTwo) {
   ASSERT_EQ(d.server_trailing_metadata.ops.size(), 2u);
   void* call_data = gpr_malloc_aligned(d.call_data_size, d.call_data_alignment);
   InfallibleOperationExecutor<ServerMetadataHandle> transformer;
-  auto memory_allocator =
-      MakeMemoryQuota("test-quota")->CreateMemoryAllocator("foo");
-  auto arena = MakeScopedArena(1024, &memory_allocator);
+  auto arena = SimpleArenaAllocator()->MakeArena();
   promise_detail::Context<Arena> ctx(arena.get());
-  auto md = Arena::MakePooled<ServerMetadata>(arena.get());
+  auto md = Arena::MakePooled<ServerMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   auto r =
       transformer.Start(&d.server_trailing_metadata, std::move(md), call_data);
@@ -1249,7 +1189,7 @@ TEST(PipeStateTest, OnePull) {
   ps.BeginPush();
   Mock::VerifyAndClearExpectations(&activity);
   // now we should see a value on the pull poll
-  EXPECT_THAT(ps.PollPull(), IsReady(Success{}));
+  EXPECT_THAT(ps.PollPull(), IsReady(true));
   // push should be pending though!
   EXPECT_THAT(ps.PollPush(), IsPending());
   // ack the pull, should see a wakeup
@@ -1257,7 +1197,7 @@ TEST(PipeStateTest, OnePull) {
   ps.AckPull();
   Mock::VerifyAndClearExpectations(&activity);
   // now the push is complete
-  EXPECT_THAT(ps.PollPush(), IsReady(Success{}));
+  EXPECT_THAT(ps.PollPush(), IsReady(Success()));
   ps.DropPush();
   ps.DropPull();
   EXPECT_FALSE(ps.holds_error());
@@ -1276,7 +1216,7 @@ TEST(PipeStateTest, StartThenPull) {
   ps.BeginPush();
   Mock::VerifyAndClearExpectations(&activity);
   // now we should see a value on the pull poll
-  EXPECT_THAT(ps.PollPull(), IsReady(Success{}));
+  EXPECT_THAT(ps.PollPull(), IsReady(true));
   // push should be pending though!
   EXPECT_THAT(ps.PollPush(), IsPending());
   // ack the pull, should see a wakeup
@@ -1284,7 +1224,7 @@ TEST(PipeStateTest, StartThenPull) {
   ps.AckPull();
   Mock::VerifyAndClearExpectations(&activity);
   // now the push is complete
-  EXPECT_THAT(ps.PollPush(), IsReady(Success{}));
+  EXPECT_THAT(ps.PollPush(), IsReady(Success()));
   ps.DropPush();
   ps.DropPull();
   EXPECT_FALSE(ps.holds_error());
@@ -1300,7 +1240,7 @@ TEST(PipeStateTest, PushFirst) {
   // push should be pending
   EXPECT_THAT(ps.PollPush(), IsPending());
   // pull should immediately see a value
-  EXPECT_THAT(ps.PollPull(), IsReady(Success{}));
+  EXPECT_THAT(ps.PollPull(), IsReady(true));
   // push should still be pending though!
   EXPECT_THAT(ps.PollPush(), IsPending());
   // ack the pull, should see a wakeup
@@ -1308,7 +1248,7 @@ TEST(PipeStateTest, PushFirst) {
   ps.AckPull();
   Mock::VerifyAndClearExpectations(&activity);
   // now the push is complete
-  EXPECT_THAT(ps.PollPush(), IsReady(Success{}));
+  EXPECT_THAT(ps.PollPush(), IsReady(Success()));
   ps.DropPush();
   ps.DropPull();
   EXPECT_FALSE(ps.holds_error());
@@ -1345,7 +1285,7 @@ TEST(PipeStateTest, DropProcessing) {
   activity.Activate();
   ps.Start();
   ps.BeginPush();
-  EXPECT_THAT(ps.PollPull(), IsReady(Success{}));
+  EXPECT_THAT(ps.PollPull(), IsReady(true));
   ps.DropPull();
   EXPECT_TRUE(ps.holds_error());
   EXPECT_THAT(ps.PollPull(), IsReady(Failure()));
@@ -1363,6 +1303,7 @@ TEST(CallFiltersTest, CanBuildStack) {
       void OnClientInitialMetadata(ClientMetadata&) {}
       void OnServerInitialMetadata(ServerMetadata&) {}
       void OnClientToServerMessage(Message&) {}
+      void OnClientToServerHalfClose() {}
       void OnServerToClientMessage(Message&) {}
       void OnServerTrailingMetadata(ServerMetadata&) {}
       void OnFinalize(const grpc_call_final_info*) {}
@@ -1387,6 +1328,10 @@ TEST(CallFiltersTest, UnaryCall) {
       void OnClientToServerMessage(Message&, Filter* f) {
         f->steps.push_back(absl::StrCat(f->label, ":OnClientToServerMessage"));
       }
+      void OnClientToServerHalfClose(Filter* f) {
+        f->steps.push_back(
+            absl::StrCat(f->label, ":OnClientToServerHalfClose"));
+      }
       void OnServerToClientMessage(Message&, Filter* f) {
         f->steps.push_back(absl::StrCat(f->label, ":OnServerToClientMessage"));
       }
@@ -1408,24 +1353,16 @@ TEST(CallFiltersTest, UnaryCall) {
   CallFilters::StackBuilder builder;
   builder.Add(&f1);
   builder.Add(&f2);
-  CallFilters filters(builder.Build());
-  auto memory_allocator =
-      MakeMemoryQuota("test-quota")->CreateMemoryAllocator("foo");
-  auto arena = MakeScopedArena(1024, &memory_allocator);
+  auto arena = SimpleArenaAllocator()->MakeArena();
+  CallFilters filters(Arena::MakePooled<ClientMetadata>());
+  filters.SetStack(builder.Build());
   promise_detail::Context<Arena> ctx(arena.get());
   StrictMock<MockActivity> activity;
   activity.Activate();
-  // Push client initial metadata
-  auto push_client_initial_metadata = filters.PushClientInitialMetadata(
-      Arena::MakePooled<ClientMetadata>(arena.get()));
-  EXPECT_THAT(push_client_initial_metadata(), IsPending());
+  // Pull client initial metadata
   auto pull_client_initial_metadata = filters.PullClientInitialMetadata();
-  // Pull client initial metadata, expect a wakeup
-  EXPECT_CALL(activity, WakeupRequested());
   EXPECT_THAT(pull_client_initial_metadata(), IsReady());
   Mock::VerifyAndClearExpectations(&activity);
-  // Push should be done
-  EXPECT_THAT(push_client_initial_metadata(), IsReady(Success{}));
   // Push client to server message
   auto push_client_to_server_message = filters.PushClientToServerMessage(
       Arena::MakePooled<Message>(SliceBuffer(), 0));
@@ -1438,8 +1375,8 @@ TEST(CallFiltersTest, UnaryCall) {
   // Push should be done
   EXPECT_THAT(push_client_to_server_message(), IsReady(Success{}));
   // Push server initial metadata
-  auto push_server_initial_metadata = filters.PushServerInitialMetadata(
-      Arena::MakePooled<ServerMetadata>(arena.get()));
+  auto push_server_initial_metadata =
+      filters.PushServerInitialMetadata(Arena::MakePooled<ServerMetadata>());
   EXPECT_THAT(push_server_initial_metadata(), IsPending());
   auto pull_server_initial_metadata = filters.PullServerInitialMetadata();
   // Pull server initial metadata, expect a wakeup
@@ -1460,8 +1397,7 @@ TEST(CallFiltersTest, UnaryCall) {
   // Push should be done
   EXPECT_THAT(push_server_to_client_message(), IsReady(Success{}));
   // Push server trailing metadata
-  filters.PushServerTrailingMetadata(
-      Arena::MakePooled<ServerMetadata>(arena.get()));
+  filters.PushServerTrailingMetadata(Arena::MakePooled<ServerMetadata>());
   // Pull server trailing metadata
   auto pull_server_trailing_metadata = filters.PullServerTrailingMetadata();
   // Should be done
