@@ -16,8 +16,6 @@
 //
 //
 
-#include <grpc/support/port_platform.h>
-
 #include <limits.h>
 #include <stdint.h>
 #include <stdlib.h>
@@ -27,10 +25,11 @@
 #include <memory>
 #include <string>
 #include <thread>
+#include <tuple>
 
 #include "absl/base/thread_annotations.h"
+#include "absl/log/check.h"
 #include "absl/status/status.h"
-#include "absl/strings/match.h"
 #include "absl/strings/str_cat.h"
 #include "absl/strings/string_view.h"
 #include "absl/time/clock.h"
@@ -38,17 +37,19 @@
 #include "gtest/gtest.h"
 
 #include <grpc/grpc.h>
+#include <grpc/impl/channel_arg_names.h>
 #include <grpc/slice.h>
 #include <grpc/slice_buffer.h>
 #include <grpc/status.h>
 #include <grpc/support/log.h>
+#include <grpc/support/port_platform.h>
 
+#include "src/core/channelz/channelz.h"
 #include "src/core/ext/transport/chttp2/transport/chttp2_transport.h"
 #include "src/core/ext/transport/chttp2/transport/frame_goaway.h"
 #include "src/core/ext/transport/chttp2/transport/frame_ping.h"
 #include "src/core/lib/channel/channel_args.h"
-#include "src/core/lib/channel/channelz.h"
-#include "src/core/lib/gpr/useful.h"
+#include "src/core/lib/gprpp/crash.h"
 #include "src/core/lib/gprpp/notification.h"
 #include "src/core/lib/gprpp/sync.h"
 #include "src/core/lib/iomgr/closure.h"
@@ -59,9 +60,10 @@
 #include "src/core/lib/slice/slice.h"
 #include "src/core/lib/slice/slice_internal.h"
 #include "src/core/lib/surface/completion_queue.h"
-#include "src/core/lib/surface/server.h"
+#include "src/core/server/server.h"
+#include "src/core/util/useful.h"
 #include "test/core/end2end/cq_verifier.h"
-#include "test/core/util/test_config.h"
+#include "test/core/test_util/test_config.h"
 
 namespace grpc_core {
 namespace {
@@ -95,10 +97,11 @@ class GracefulShutdownTest : public ::testing::Test {
     auto* transport = grpc_create_chttp2_transport(core_server->channel_args(),
                                                    fds_.server, false);
     grpc_endpoint_add_to_pollset(fds_.server, grpc_cq_pollset(cq_));
-    GPR_ASSERT(core_server->SetupTransport(transport, nullptr,
-                                           core_server->channel_args(),
-                                           nullptr) == absl::OkStatus());
-    grpc_chttp2_transport_start_reading(transport, nullptr, nullptr, nullptr);
+    CHECK(core_server->SetupTransport(transport, nullptr,
+                                      core_server->channel_args(),
+                                      nullptr) == absl::OkStatus());
+    grpc_chttp2_transport_start_reading(transport, nullptr, nullptr, nullptr,
+                                        nullptr);
     // Start polling on the client
     Notification client_poller_thread_started_notification;
     client_poll_thread_ = std::make_unique<std::thread>(
@@ -114,10 +117,10 @@ class GracefulShutdownTest : public ::testing::Test {
           }
           client_poller_thread_started_notification.Notify();
           while (!shutdown_) {
-            GPR_ASSERT(grpc_completion_queue_next(
-                           client_cq, grpc_timeout_milliseconds_to_deadline(10),
-                           nullptr)
-                           .type == GRPC_QUEUE_TIMEOUT);
+            CHECK(grpc_completion_queue_next(
+                      client_cq, grpc_timeout_milliseconds_to_deadline(10),
+                      nullptr)
+                      .type == GRPC_QUEUE_TIMEOUT);
           }
           grpc_completion_queue_destroy(client_cq);
         });
@@ -129,6 +132,8 @@ class GracefulShutdownTest : public ::testing::Test {
     // Start reading on the client
     grpc_slice_buffer_init(&read_buffer_);
     GRPC_CLOSURE_INIT(&on_read_done_, OnReadDone, this, nullptr);
+    GRPC_CLOSURE_INIT(&on_read_done_scheduler_, OnReadDoneScheduler, this,
+                      nullptr);
     grpc_endpoint_read(fds_.client, &read_buffer_, &on_read_done_, false,
                        /*min_progress_size=*/1);
   }
@@ -137,13 +142,15 @@ class GracefulShutdownTest : public ::testing::Test {
   void ShutdownAndDestroy() {
     shutdown_ = true;
     ExecCtx exec_ctx;
-    grpc_endpoint_shutdown(fds_.client, GRPC_ERROR_CREATE("Client shutdown"));
+    {
+      MutexLock lock(&ep_destroy_mu_);
+      grpc_endpoint_destroy(fds_.client);
+      fds_.client = nullptr;
+    }
     ExecCtx::Get()->Flush();
     client_poll_thread_->join();
-    GPR_ASSERT(read_end_notification_.WaitForNotificationWithTimeout(
+    CHECK(read_end_notification_.WaitForNotificationWithTimeout(
         absl::Seconds(5)));
-    grpc_endpoint_destroy(fds_.client);
-    ExecCtx::Get()->Flush();
     // Shutdown and destroy server
     grpc_server_shutdown_and_notify(server_, cq_, Tag(1000));
     cqv_->Expect(Tag(1000), true);
@@ -164,27 +171,56 @@ class GracefulShutdownTest : public ::testing::Test {
         }
         self->read_cv_.SignalAll();
       }
-      grpc_slice_buffer_reset_and_unref(&self->read_buffer_);
-      grpc_endpoint_read(self->fds_.client, &self->read_buffer_,
-                         &self->on_read_done_, false, /*min_progress_size=*/1);
-    } else {
-      grpc_slice_buffer_destroy(&self->read_buffer_);
-      self->read_end_notification_.Notify();
+      MutexLock lock(&self->ep_destroy_mu_);
+      if (self->fds_.client != nullptr) {
+        grpc_slice_buffer_reset_and_unref(&self->read_buffer_);
+        grpc_endpoint_read(self->fds_.client, &self->read_buffer_,
+                           &self->on_read_done_scheduler_, false,
+                           /*min_progress_size=*/1);
+        return;
+      }
     }
+    grpc_slice_buffer_destroy(&self->read_buffer_);
+    self->read_end_notification_.Notify();
+  }
+
+  // Do async hop for OnReadDone() in case grpc_endpoint_read() invokes
+  // us synchronously while we're holding the lock.
+  static void OnReadDoneScheduler(void* arg, grpc_error_handle error) {
+    GracefulShutdownTest* self = static_cast<GracefulShutdownTest*>(arg);
+    ExecCtx::Run(DEBUG_LOCATION, &self->on_read_done_, std::move(error));
   }
 
   // Waits for \a bytes to show up in read_bytes_
   void WaitForReadBytes(absl::string_view bytes) {
-    std::atomic<bool> done{false};
     auto start_time = absl::Now();
-    {
-      MutexLock lock(&mu_);
-      while (!absl::StrContains(read_bytes_, bytes)) {
-        ASSERT_LT(absl::Now() - start_time, absl::Seconds(60));
-        read_cv_.WaitWithTimeout(&mu_, absl::Seconds(5));
+    MutexLock lock(&mu_);
+    while (true) {
+      auto where = read_bytes_.find(std::string(bytes));
+      if (where != std::string::npos) {
+        read_bytes_ = read_bytes_.substr(where + bytes.size());
+        break;
       }
+      ASSERT_LT(absl::Now() - start_time, absl::Seconds(60));
+      read_cv_.WaitWithTimeout(&mu_, absl::Seconds(5));
     }
-    done = true;
+  }
+
+  std::string WaitForNBytes(size_t bytes) {
+    auto start_time = absl::Now();
+    MutexLock lock(&mu_);
+    while (read_bytes_.size() < bytes) {
+      EXPECT_LT(absl::Now() - start_time, absl::Seconds(60));
+      read_cv_.WaitWithTimeout(&mu_, absl::Seconds(5));
+    }
+    std::string result = read_bytes_.substr(0, bytes);
+    read_bytes_ = read_bytes_.substr(bytes);
+    return result;
+  }
+
+  void WaitForClose() {
+    ASSERT_TRUE(read_end_notification_.WaitForNotificationWithTimeout(
+        absl::Minutes(1)));
   }
 
   void WaitForGoaway(uint32_t last_stream_id, uint32_t error_code = 0,
@@ -200,9 +236,20 @@ class GracefulShutdownTest : public ::testing::Test {
     WaitForReadBytes(expected_bytes);
   }
 
-  void WaitForPing(uint64_t opaque_data) {
-    grpc_slice ping_slice = grpc_chttp2_ping_create(0, opaque_data);
-    WaitForReadBytes(StringViewFromSlice(ping_slice));
+  uint64_t WaitForPing() {
+    grpc_slice ping_slice = grpc_chttp2_ping_create(0, 0);
+    auto whole_ping = StringViewFromSlice(ping_slice);
+    CHECK(whole_ping.size() == 9 + 8);
+    WaitForReadBytes(whole_ping.substr(0, 9));
+    std::string ping = WaitForNBytes(8);
+    return (static_cast<uint64_t>(static_cast<uint8_t>(ping[0])) << 56) |
+           (static_cast<uint64_t>(static_cast<uint8_t>(ping[1])) << 48) |
+           (static_cast<uint64_t>(static_cast<uint8_t>(ping[2])) << 40) |
+           (static_cast<uint64_t>(static_cast<uint8_t>(ping[3])) << 32) |
+           (static_cast<uint64_t>(static_cast<uint8_t>(ping[4])) << 24) |
+           (static_cast<uint64_t>(static_cast<uint8_t>(ping[5])) << 16) |
+           (static_cast<uint64_t>(static_cast<uint8_t>(ping[6])) << 8) |
+           (static_cast<uint64_t>(static_cast<uint8_t>(ping[7])));
   }
 
   void SendPingAck(uint64_t opaque_data) {
@@ -231,16 +278,21 @@ class GracefulShutdownTest : public ::testing::Test {
     grpc_endpoint_write(fds_.client, buffer, &on_write_done_, nullptr,
                         /*max_frame_size=*/INT_MAX);
     ExecCtx::Get()->Flush();
-    GPR_ASSERT(on_write_done_notification_.WaitForNotificationWithTimeout(
+    CHECK(on_write_done_notification_.WaitForNotificationWithTimeout(
         absl::Seconds(5)));
   }
 
   static void OnWriteDone(void* arg, grpc_error_handle error) {
-    GPR_ASSERT(error.ok());
+    if (!error.ok()) {
+      Crash(absl::StrCat("Write failed: ", error.ToString()));
+    }
     Notification* on_write_done_notification_ = static_cast<Notification*>(arg);
     on_write_done_notification_->Notify();
   }
 
+  // Held when destroying fds_.client so we know not to start another read.
+  Mutex ep_destroy_mu_;
+
   grpc_endpoint_pair fds_;
   grpc_server* server_ = nullptr;
   grpc_completion_queue* cq_ = nullptr;
@@ -248,6 +300,7 @@ class GracefulShutdownTest : public ::testing::Test {
   std::unique_ptr<std::thread> client_poll_thread_;
   std::atomic<bool> shutdown_{false};
   grpc_closure on_read_done_;
+  grpc_closure on_read_done_scheduler_;
   Mutex mu_;
   CondVar read_cv_;
   Notification read_end_notification_;
@@ -262,9 +315,9 @@ TEST_F(GracefulShutdownTest, GracefulGoaway) {
   // Wait for first goaway
   WaitForGoaway((1u << 31) - 1);
   // Wait for the ping
-  WaitForPing(0);
+  uint64_t ping_id = WaitForPing();
   // Reply to the ping
-  SendPingAck(0);
+  SendPingAck(ping_id);
   // Wait for final goaway
   WaitForGoaway(0);
   // The shutdown should successfully complete.
@@ -281,13 +334,13 @@ TEST_F(GracefulShutdownTest, RequestStartedBeforeFinalGoaway) {
   grpc_metadata_array_init(&request_metadata_recv);
   error = grpc_server_request_call(server_, &s, &call_details,
                                    &request_metadata_recv, cq_, cq_, Tag(100));
-  GPR_ASSERT(GRPC_CALL_OK == error);
+  CHECK_EQ(error, GRPC_CALL_OK);
   // Initiate shutdown on the server
   grpc_server_shutdown_and_notify(server_, cq_, Tag(1));
   // Wait for first goaway
   WaitForGoaway((1u << 31) - 1);
   // Wait for the ping
-  WaitForPing(0);
+  uint64_t ping_id = WaitForPing();
   // Start a request
   constexpr char kRequestFrame[] =
       "\x00\x00\xbe\x01\x05\x00\x00\x00\x01"
@@ -303,7 +356,7 @@ TEST_F(GracefulShutdownTest, RequestStartedBeforeFinalGoaway) {
       "\x10\x0auser-agent\x17grpc-c/0.12.0.0 (linux)";
   Write(absl::string_view(kRequestFrame, sizeof(kRequestFrame) - 1));
   // Reply to the ping
-  SendPingAck(0);
+  SendPingAck(ping_id);
   // Wait for final goaway with last stream ID 1 to show that the HTTP2
   // transport accepted the stream.
   WaitForGoaway(1);
@@ -328,7 +381,7 @@ TEST_F(GracefulShutdownTest, RequestStartedAfterFinalGoawayIsIgnored) {
   grpc_metadata_array_init(&request_metadata_recv);
   error = grpc_server_request_call(server_, &s, &call_details,
                                    &request_metadata_recv, cq_, cq_, Tag(100));
-  GPR_ASSERT(GRPC_CALL_OK == error);
+  CHECK_EQ(error, GRPC_CALL_OK);
   // Send the request from the client.
   constexpr char kRequestFrame[] =
       "\x00\x00\xbe\x01\x05\x00\x00\x00\x01"
@@ -351,9 +404,9 @@ TEST_F(GracefulShutdownTest, RequestStartedAfterFinalGoawayIsIgnored) {
   // Wait for first goaway
   WaitForGoaway((1u << 31) - 1);
   // Wait for the ping
-  WaitForPing(0);
+  uint64_t ping_id = WaitForPing();
   // Reply to the ping
-  SendPingAck(0);
+  SendPingAck(ping_id);
   // Wait for final goaway
   WaitForGoaway(1);
 
@@ -398,7 +451,7 @@ TEST_F(GracefulShutdownTest, RequestStartedAfterFinalGoawayIsIgnored) {
   op++;
   error = grpc_call_start_batch(s, ops, static_cast<size_t>(op - ops), Tag(101),
                                 nullptr);
-  GPR_ASSERT(GRPC_CALL_OK == error);
+  CHECK_EQ(error, GRPC_CALL_OK);
   cqv_->Expect(Tag(101), true);
   // The shutdown should successfully complete.
   cqv_->Expect(Tag(1), true);
@@ -417,9 +470,9 @@ TEST_F(GracefulShutdownTest, UnresponsiveClient) {
   // Wait for first goaway
   WaitForGoaway((1u << 31) - 1);
   // Wait for the ping
-  WaitForPing(0);
+  std::ignore = WaitForPing();
   // Wait for final goaway without sending a ping ACK.
-  WaitForGoaway(0);
+  WaitForClose();
   EXPECT_GE(absl::Now() - initial_time,
             absl::Seconds(20) -
                 absl::Seconds(
