@@ -12,6 +12,8 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+#include <memory>
+
 #include "gtest/gtest.h"
 
 #include <grpc/grpc.h>
@@ -30,9 +32,8 @@
 #include "src/core/lib/iomgr/endpoint.h"
 #include "src/core/lib/iomgr/exec_ctx.h"
 #include "src/core/lib/resource_quota/resource_quota.h"
-#include "src/core/lib/transport/transport.h"
-#include "test/core/util/mock_endpoint.h"
-#include "test/core/util/test_config.h"
+#include "test/core/test_util/mock_endpoint.h"
+#include "test/core/test_util/test_config.h"
 
 namespace grpc_core {
 namespace {
@@ -40,17 +41,15 @@ namespace {
 class ConfigurationTest : public ::testing::Test {
  protected:
   ConfigurationTest() {
-    mock_endpoint_ = grpc_mock_endpoint_create(DiscardWrite);
+    auto engine = grpc_event_engine::experimental::GetDefaultEventEngine();
+    mock_endpoint_ = grpc_mock_endpoint_create(engine);
+    grpc_mock_endpoint_finish_put_reads(mock_endpoint_);
     args_ = args_.SetObject(ResourceQuota::Default());
-    args_ = args_.SetObject(
-        grpc_event_engine::experimental::GetDefaultEventEngine());
+    args_ = args_.SetObject(std::move(engine));
   }
 
   grpc_endpoint* mock_endpoint_ = nullptr;
   ChannelArgs args_;
-
- private:
-  static void DiscardWrite(grpc_slice /*slice*/) {}
 };
 
 TEST_F(ConfigurationTest, ClientKeepaliveDefaults) {
@@ -61,7 +60,7 @@ TEST_F(ConfigurationTest, ClientKeepaliveDefaults) {
   EXPECT_EQ(t->keepalive_timeout, Duration::Infinity());
   EXPECT_EQ(t->keepalive_permit_without_calls, false);
   EXPECT_EQ(t->ping_rate_policy.TestOnlyMaxPingsWithoutData(), 2);
-  grpc_transport_destroy(&t->base);
+  t->Orphan();
 }
 
 TEST_F(ConfigurationTest, ClientKeepaliveExplicitArgs) {
@@ -76,7 +75,7 @@ TEST_F(ConfigurationTest, ClientKeepaliveExplicitArgs) {
   EXPECT_EQ(t->keepalive_timeout, Duration::Seconds(10));
   EXPECT_EQ(t->keepalive_permit_without_calls, true);
   EXPECT_EQ(t->ping_rate_policy.TestOnlyMaxPingsWithoutData(), 3);
-  grpc_transport_destroy(&t->base);
+  t->Orphan();
 }
 
 TEST_F(ConfigurationTest, ServerKeepaliveDefaults) {
@@ -91,7 +90,7 @@ TEST_F(ConfigurationTest, ServerKeepaliveDefaults) {
   EXPECT_EQ(t->ping_abuse_policy.TestOnlyMinPingIntervalWithoutData(),
             Duration::Minutes(5));
   EXPECT_EQ(t->ping_abuse_policy.TestOnlyMaxPingStrikes(), 2);
-  grpc_transport_destroy(&t->base);
+  t->Orphan();
 }
 
 TEST_F(ConfigurationTest, ServerKeepaliveExplicitArgs) {
@@ -113,7 +112,7 @@ TEST_F(ConfigurationTest, ServerKeepaliveExplicitArgs) {
   EXPECT_EQ(t->ping_abuse_policy.TestOnlyMinPingIntervalWithoutData(),
             Duration::Seconds(20));
   EXPECT_EQ(t->ping_abuse_policy.TestOnlyMaxPingStrikes(), 0);
-  grpc_transport_destroy(&t->base);
+  t->Orphan();
 }
 
 // This test modifies the defaults of the client side settings, so it would
@@ -136,7 +135,7 @@ TEST_F(ConfigurationTest, ModifyClientDefaults) {
   EXPECT_EQ(t->keepalive_timeout, Duration::Seconds(10));
   EXPECT_EQ(t->keepalive_permit_without_calls, true);
   EXPECT_EQ(t->ping_rate_policy.TestOnlyMaxPingsWithoutData(), 3);
-  grpc_transport_destroy(&t->base);
+  t->Orphan();
 }
 
 // This test modifies the defaults of the client side settings, so it would
@@ -165,7 +164,7 @@ TEST_F(ConfigurationTest, ModifyServerDefaults) {
   EXPECT_EQ(t->ping_abuse_policy.TestOnlyMinPingIntervalWithoutData(),
             Duration::Seconds(20));
   EXPECT_EQ(t->ping_abuse_policy.TestOnlyMaxPingStrikes(), 0);
-  grpc_transport_destroy(&t->base);
+  t->Orphan();
 }
 
 }  // namespace
