@@ -16,8 +16,6 @@
 //
 //
 
-#include <grpc/support/port_platform.h>
-
 #include <limits.h>
 #include <stdint.h>
 #include <stdlib.h>
@@ -31,6 +29,7 @@
 #include <vector>
 
 #include "absl/base/thread_annotations.h"
+#include "absl/log/check.h"
 #include "absl/status/status.h"
 #include "absl/strings/match.h"
 #include "absl/strings/str_cat.h"
@@ -39,6 +38,7 @@
 #include "absl/types/optional.h"
 #include "gtest/gtest.h"
 
+#include <grpc/credentials.h>
 #include <grpc/grpc.h>
 #include <grpc/grpc_security.h>
 #include <grpc/impl/channel_arg_names.h>
@@ -48,6 +48,7 @@
 #include <grpc/status.h>
 #include <grpc/support/alloc.h>
 #include <grpc/support/log.h>
+#include <grpc/support/port_platform.h>
 
 #include "src/core/ext/transport/chttp2/transport/chttp2_transport.h"
 #include "src/core/ext/transport/chttp2/transport/frame_goaway.h"
@@ -55,7 +56,6 @@
 #include "src/core/lib/channel/channel_fwd.h"
 #include "src/core/lib/channel/channel_stack.h"
 #include "src/core/lib/config/core_configuration.h"
-#include "src/core/lib/gpr/useful.h"
 #include "src/core/lib/gprpp/debug_location.h"
 #include "src/core/lib/gprpp/host_port.h"
 #include "src/core/lib/gprpp/notification.h"
@@ -72,10 +72,11 @@
 #include "src/core/lib/surface/channel_stack_type.h"
 #include "src/core/lib/transport/metadata_batch.h"
 #include "src/core/lib/transport/transport.h"
+#include "src/core/util/useful.h"
 #include "test/core/end2end/cq_verifier.h"
-#include "test/core/util/port.h"
-#include "test/core/util/test_config.h"
-#include "test/core/util/test_tcp_server.h"
+#include "test/core/test_util/port.h"
+#include "test/core/test_util/test_config.h"
+#include "test/core/test_util/test_tcp_server.h"
 
 namespace grpc_core {
 namespace {
@@ -198,8 +199,6 @@ class TrailingMetadataRecordingFilter {
 
 grpc_channel_filter TrailingMetadataRecordingFilter::kFilterVtable = {
     CallData::StartTransportStreamOpBatch,
-    nullptr,
-    nullptr,
     grpc_channel_next_op,
     sizeof(CallData),
     CallData::Init,
@@ -230,6 +229,8 @@ class StreamsNotSeenTest : public ::testing::Test {
     TrailingMetadataRecordingFilter::reset_state();
     grpc_slice_buffer_init(&read_buffer_);
     GRPC_CLOSURE_INIT(&on_read_done_, OnReadDone, this, nullptr);
+    GRPC_CLOSURE_INIT(&on_read_done_scheduler_, OnReadDoneScheduler, this,
+                      nullptr);
     // Start the test tcp server
     port_ = grpc_pick_unused_port_or_die();
     test_tcp_server_init(&server_, OnConnect, this);
@@ -267,7 +268,7 @@ class StreamsNotSeenTest : public ::testing::Test {
       state = grpc_channel_check_connectivity_state(channel_, false);
     }
     ExecCtx::Get()->Flush();
-    GPR_ASSERT(
+    CHECK(
         connect_notification_.WaitForNotificationWithTimeout(absl::Seconds(1)));
   }
 
@@ -281,11 +282,10 @@ class StreamsNotSeenTest : public ::testing::Test {
     } while (ev.type != GRPC_QUEUE_SHUTDOWN);
     grpc_completion_queue_destroy(cq_);
     grpc_channel_destroy(channel_);
-    grpc_endpoint_shutdown(tcp_, GRPC_ERROR_CREATE("Test Shutdown"));
+    if (tcp_ != nullptr) grpc_endpoint_destroy(tcp_);
     ExecCtx::Get()->Flush();
-    GPR_ASSERT(read_end_notification_.WaitForNotificationWithTimeout(
+    CHECK(read_end_notification_.WaitForNotificationWithTimeout(
         absl::Seconds(5)));
-    grpc_endpoint_destroy(tcp_);
     shutdown_ = true;
     server_poll_thread_->join();
     test_tcp_server_destroy(&server_);
@@ -356,15 +356,20 @@ class StreamsNotSeenTest : public ::testing::Test {
     Notification on_write_done_notification_;
     GRPC_CLOSURE_INIT(&on_write_done_, OnWriteDone,
                       &on_write_done_notification_, nullptr);
-    grpc_endpoint_write(tcp_, buffer, &on_write_done_, nullptr,
-                        /*max_frame_size=*/INT_MAX);
+    {
+      MutexLock lock(&tcp_destroy_mu_);
+      if (tcp_ != nullptr) {
+        grpc_endpoint_write(tcp_, buffer, &on_write_done_, nullptr,
+                            /*max_frame_size=*/INT_MAX);
+      }
+    }
     ExecCtx::Get()->Flush();
-    GPR_ASSERT(on_write_done_notification_.WaitForNotificationWithTimeout(
+    CHECK(on_write_done_notification_.WaitForNotificationWithTimeout(
         absl::Seconds(5)));
   }
 
   static void OnWriteDone(void* arg, grpc_error_handle error) {
-    GPR_ASSERT(error.ok());
+    CHECK_OK(error);
     Notification* on_write_done_notification_ = static_cast<Notification*>(arg);
     on_write_done_notification_->Notify();
   }
@@ -380,13 +385,30 @@ class StreamsNotSeenTest : public ::testing::Test {
         }
         self->read_cv_.SignalAll();
       }
-      grpc_slice_buffer_reset_and_unref(&self->read_buffer_);
-      grpc_endpoint_read(self->tcp_, &self->read_buffer_, &self->on_read_done_,
-                         false, /*min_progress_size=*/1);
-    } else {
-      grpc_slice_buffer_destroy(&self->read_buffer_);
-      self->read_end_notification_.Notify();
+      MutexLock lock(&self->tcp_destroy_mu_);
+      if (self->tcp_ != nullptr) {
+        grpc_slice_buffer_reset_and_unref(&self->read_buffer_);
+        grpc_endpoint_read(self->tcp_, &self->read_buffer_,
+                           &self->on_read_done_scheduler_, false,
+                           /*min_progress_size=*/1);
+        return;
+      }
     }
+    grpc_slice_buffer_destroy(&self->read_buffer_);
+    self->read_end_notification_.Notify();
+  }
+
+  // Async hop for OnReadDone(), in case grpc_endpoint_read() invokes
+  // the callback synchronously while holding the lock.
+  static void OnReadDoneScheduler(void* arg, grpc_error_handle error) {
+    StreamsNotSeenTest* self = static_cast<StreamsNotSeenTest*>(arg);
+    ExecCtx::Run(DEBUG_LOCATION, &self->on_read_done_, std::move(error));
+  }
+
+  void CloseServerConnection() {
+    MutexLock lock(&tcp_destroy_mu_);
+    grpc_endpoint_destroy(tcp_);
+    tcp_ = nullptr;
   }
 
   // Waits for \a bytes to show up in read_bytes_
@@ -396,7 +418,7 @@ class StreamsNotSeenTest : public ::testing::Test {
       while (!done) {
         grpc_event ev = grpc_completion_queue_next(
             cq_, grpc_timeout_milliseconds_to_deadline(10), nullptr);
-        GPR_ASSERT(ev.type == GRPC_QUEUE_TIMEOUT);
+        CHECK(ev.type == GRPC_QUEUE_TIMEOUT);
       }
     });
     {
@@ -415,11 +437,14 @@ class StreamsNotSeenTest : public ::testing::Test {
   int port_;
   test_tcp_server server_;
   std::unique_ptr<std::thread> server_poll_thread_;
+  // Guards destroying tcp_, so that we know not to start the next read/write.
+  Mutex tcp_destroy_mu_;
   grpc_endpoint* tcp_ = nullptr;
   Notification connect_notification_;
   grpc_slice_buffer read_buffer_;
   grpc_closure on_write_done_;
   grpc_closure on_read_done_;
+  grpc_closure on_read_done_scheduler_;
   Notification read_end_notification_;
   std::string read_bytes_ ABSL_GUARDED_BY(mu_);
   grpc_channel* channel_ = nullptr;
@@ -440,7 +465,7 @@ TEST_F(StreamsNotSeenTest, StartStreamBeforeGoaway) {
       grpc_channel_create_call(channel_, nullptr, GRPC_PROPAGATE_DEFAULTS, cq_,
                                grpc_slice_from_static_string("/foo"), nullptr,
                                grpc_timeout_seconds_to_deadline(1), nullptr);
-  GPR_ASSERT(c);
+  CHECK(c);
   grpc_metadata_array initial_metadata_recv;
   grpc_metadata_array trailing_metadata_recv;
   grpc_metadata_array_init(&initial_metadata_recv);
@@ -487,7 +512,7 @@ TEST_F(StreamsNotSeenTest, StartStreamBeforeGoaway) {
   op++;
   error = grpc_call_start_batch(c, ops, static_cast<size_t>(op - ops), Tag(102),
                                 nullptr);
-  GPR_ASSERT(GRPC_CALL_OK == error);
+  CHECK_EQ(error, GRPC_CALL_OK);
   cqv_->Expect(Tag(102), true);
   cqv_->Verify();
   // Verify status and metadata
@@ -514,7 +539,7 @@ TEST_F(StreamsNotSeenTest, TransportDestroyed) {
       grpc_channel_create_call(channel_, nullptr, GRPC_PROPAGATE_DEFAULTS, cq_,
                                grpc_slice_from_static_string("/foo"), nullptr,
                                grpc_timeout_seconds_to_deadline(1), nullptr);
-  GPR_ASSERT(c);
+  CHECK(c);
   grpc_metadata_array initial_metadata_recv;
   grpc_metadata_array trailing_metadata_recv;
   grpc_metadata_array_init(&initial_metadata_recv);
@@ -542,7 +567,7 @@ TEST_F(StreamsNotSeenTest, TransportDestroyed) {
   cqv_->Expect(Tag(101), true);
   cqv_->Verify();
   // Shutdown the server endpoint
-  grpc_endpoint_shutdown(tcp_, GRPC_ERROR_CREATE("Server shutdown"));
+  CloseServerConnection();
   memset(ops, 0, sizeof(ops));
   op = ops;
   op->op = GRPC_OP_RECV_INITIAL_METADATA;
@@ -560,7 +585,7 @@ TEST_F(StreamsNotSeenTest, TransportDestroyed) {
   op++;
   error = grpc_call_start_batch(c, ops, static_cast<size_t>(op - ops), Tag(102),
                                 nullptr);
-  GPR_ASSERT(GRPC_CALL_OK == error);
+  CHECK_EQ(error, GRPC_CALL_OK);
   cqv_->Expect(Tag(102), true);
   cqv_->Verify();
   // Verify status and metadata
@@ -588,7 +613,7 @@ TEST_F(StreamsNotSeenTest, StartStreamAfterGoaway) {
       grpc_channel_create_call(channel_, nullptr, GRPC_PROPAGATE_DEFAULTS, cq_,
                                grpc_slice_from_static_string("/foo"), nullptr,
                                grpc_timeout_seconds_to_deadline(1), nullptr);
-  GPR_ASSERT(c);
+  CHECK(c);
   grpc_metadata_array initial_metadata_recv;
   grpc_metadata_array trailing_metadata_recv;
   grpc_metadata_array_init(&initial_metadata_recv);
@@ -625,7 +650,7 @@ TEST_F(StreamsNotSeenTest, StartStreamAfterGoaway) {
   op++;
   error = grpc_call_start_batch(c, ops, static_cast<size_t>(op - ops), Tag(101),
                                 nullptr);
-  GPR_ASSERT(GRPC_CALL_OK == error);
+  CHECK_EQ(error, GRPC_CALL_OK);
   cqv_->Expect(Tag(101), true);
   cqv_->Verify();
   // Verify status and metadata
@@ -661,7 +686,7 @@ TEST_F(ZeroConcurrencyTest, StartStreamBeforeGoaway) {
       grpc_channel_create_call(channel_, nullptr, GRPC_PROPAGATE_DEFAULTS, cq_,
                                grpc_slice_from_static_string("/foo"), nullptr,
                                grpc_timeout_seconds_to_deadline(5), nullptr);
-  GPR_ASSERT(c);
+  CHECK(c);
   grpc_metadata_array initial_metadata_recv;
   grpc_metadata_array trailing_metadata_recv;
   grpc_metadata_array_init(&initial_metadata_recv);
@@ -703,7 +728,7 @@ TEST_F(ZeroConcurrencyTest, StartStreamBeforeGoaway) {
   // the transport. If that no longer holds true, we might need to drive the cq
   // for some time to make sure that the RPC reaches the HTTP2 layer.
   SendGoaway(0);
-  GPR_ASSERT(GRPC_CALL_OK == error);
+  CHECK_EQ(error, GRPC_CALL_OK);
   cqv_->Expect(Tag(101), true);
   cqv_->Verify();
   // Verify status and metadata
@@ -729,7 +754,7 @@ TEST_F(ZeroConcurrencyTest, TransportDestroyed) {
       grpc_channel_create_call(channel_, nullptr, GRPC_PROPAGATE_DEFAULTS, cq_,
                                grpc_slice_from_static_string("/foo"), nullptr,
                                grpc_timeout_seconds_to_deadline(5), nullptr);
-  GPR_ASSERT(c);
+  CHECK(c);
   grpc_metadata_array initial_metadata_recv;
   grpc_metadata_array trailing_metadata_recv;
   grpc_metadata_array_init(&initial_metadata_recv);
@@ -767,8 +792,8 @@ TEST_F(ZeroConcurrencyTest, TransportDestroyed) {
   op++;
   error = grpc_call_start_batch(c, ops, static_cast<size_t>(op - ops), Tag(101),
                                 nullptr);
-  grpc_endpoint_shutdown(tcp_, GRPC_ERROR_CREATE("Server shutdown"));
-  GPR_ASSERT(GRPC_CALL_OK == error);
+  CloseServerConnection();
+  CHECK_EQ(error, GRPC_CALL_OK);
   cqv_->Expect(Tag(101), true);
   cqv_->Verify();
   // Verify status and metadata
