@@ -22,6 +22,8 @@
 
 #include <gtest/gtest.h>
 
+#include "absl/log/log.h"
+
 #include <grpc/grpc.h>
 #include <grpc/support/time.h>
 #include <grpcpp/channel.h>
@@ -37,19 +39,13 @@
 #include "src/core/lib/surface/api_trace.h"
 #include "src/proto/grpc/testing/duplicate/echo_duplicate.grpc.pb.h"
 #include "src/proto/grpc/testing/echo.grpc.pb.h"
-#include "test/core/util/port.h"
-#include "test/core/util/test_config.h"
+#include "test/core/test_util/port.h"
+#include "test/core/test_util/test_config.h"
 
-#if defined(__APPLE__)
-// Use less # of threads on Mac because its test machines are less powerful
-// to finish the test on time. (context: b/185231823)
-const int kNumThreads = 100;  // Number of threads
-#else
-const int kNumThreads = 300;  // Number of threads
-#endif
+const int kNumThreads = 10;  // Number of threads
 const int kNumAsyncSendThreads = 2;
-const int kNumAsyncReceiveThreads = 50;
-const int kNumAsyncServerThreads = 50;
+const int kNumAsyncReceiveThreads = 5;
+const int kNumAsyncServerThreads = 5;
 const int kNumRpcs = 1000;  // Number of RPCs per thread
 
 namespace grpc {
@@ -291,8 +287,8 @@ static void SendRpc(grpc::testing::EchoTestService::Stub* stub, int num_rpcs,
     if (!s.ok()) {
       if (!(allow_exhaustion &&
             s.error_code() == StatusCode::RESOURCE_EXHAUSTED)) {
-        gpr_log(GPR_ERROR, "RPC error: %d: %s", s.error_code(),
-                s.error_message().c_str());
+        LOG(ERROR) << "RPC error: " << s.error_code() << ": "
+                   << s.error_message();
       }
       gpr_atm_no_barrier_fetch_add(errors, gpr_atm{1});
     } else {
@@ -303,13 +299,8 @@ static void SendRpc(grpc::testing::EchoTestService::Stub* stub, int num_rpcs,
 
 typedef ::testing::Types<
     CommonStressTestSyncServer<CommonStressTestInsecure<TestServiceImpl>>,
-    CommonStressTestSyncServer<CommonStressTestInproc<TestServiceImpl, false>>,
-    CommonStressTestSyncServerLowThreadCount<
-        CommonStressTestInproc<TestServiceImpl, true>>,
     CommonStressTestAsyncServer<
-        CommonStressTestInsecure<grpc::testing::EchoTestService::AsyncService>>,
-    CommonStressTestAsyncServer<CommonStressTestInproc<
-        grpc::testing::EchoTestService::AsyncService, false>>>
+        CommonStressTestInsecure<grpc::testing::EchoTestService::AsyncService>>>
     CommonTypes;
 TYPED_TEST_SUITE(End2endTest, CommonTypes);
 TYPED_TEST(End2endTest, ThreadStress) {
@@ -317,20 +308,17 @@ TYPED_TEST(End2endTest, ThreadStress) {
   std::vector<std::thread> threads;
   gpr_atm errors;
   gpr_atm_rel_store(&errors, gpr_atm{0});
-  int num_threads = kNumThreads / grpc_test_slowdown_factor();
-  // The number of threads should be > 10 to be able to catch errors
-  ASSERT_GT(num_threads, 10);
-  threads.reserve(num_threads);
-  for (int i = 0; i < num_threads; ++i) {
+  threads.reserve(kNumThreads);
+  for (int i = 0; i < kNumThreads; ++i) {
     threads.emplace_back(SendRpc, this->common_.GetStub(), kNumRpcs,
                          this->common_.AllowExhaustion(), &errors);
   }
-  for (int i = 0; i < num_threads; ++i) {
+  for (int i = 0; i < kNumThreads; ++i) {
     threads[i].join();
   }
   uint64_t error_cnt = static_cast<uint64_t>(gpr_atm_no_barrier_load(&errors));
   if (error_cnt != 0) {
-    gpr_log(GPR_INFO, "RPC error count: %" PRIu64, error_cnt);
+    LOG(INFO) << "RPC error count: " << error_cnt;
   }
   // If this test allows resource exhaustion, expect that it actually sees some
   if (this->common_.AllowExhaustion()) {
@@ -389,7 +377,7 @@ class AsyncClientEnd2endTest : public ::testing::Test {
       if (!cq_.Next(&got_tag, &ok)) break;
       AsyncClientCall* call = static_cast<AsyncClientCall*>(got_tag);
       if (!ok) {
-        gpr_log(GPR_DEBUG, "Error: %d", call->status.error_code());
+        VLOG(2) << "Error: " << call->status.error_code();
       }
       delete call;
 
