@@ -20,11 +20,18 @@
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
 
+#include "absl/log/log.h"
 #include "absl/strings/match.h"
 #include "absl/strings/str_cat.h"
 
-#include "src/core/ext/filters/client_channel/backup_poller.h"
+#include "src/core/client_channel/backup_poller.h"
 #include "src/core/lib/address_utils/sockaddr_utils.h"
+#include "src/core/lib/config/config_vars.h"
+#include "src/core/lib/surface/call.h"
+#include "src/core/telemetry/call_tracer.h"
+#include "src/proto/grpc/testing/xds/v3/orca_load_report.pb.h"
+#include "test/core/test_util/fake_stats_plugin.h"
+#include "test/core/test_util/scoped_env_var.h"
 #include "test/cpp/end2end/connection_attempt_injector.h"
 #include "test/cpp/end2end/xds/xds_end2end_test_lib.h"
 
@@ -38,6 +45,8 @@ using ::envoy::config::core::v3::HealthStatus;
 using ::envoy::type::v3::FractionalPercent;
 
 using ClientStats = LrsServiceImpl::ClientStats;
+using OptionalLabelKey =
+    grpc_core::ClientCallTracer::CallAttemptTracer::OptionalLabelKey;
 
 constexpr char kLbDropType[] = "lb";
 constexpr char kThrottleDropType[] = "throttle";
@@ -300,6 +309,55 @@ TEST_P(CdsTest, ClusterChangeAfterAdsCallFails) {
   WaitForBackend(DEBUG_LOCATION, 1);
 }
 
+TEST_P(CdsTest, MetricLabels) {
+  // Injects a fake client call tracer factory. Try keep this at top.
+  grpc_core::FakeClientCallTracerFactory fake_client_call_tracer_factory;
+  CreateAndStartBackends(2);
+  // Populates EDS resources.
+  EdsResourceArgs args({{"locality0", CreateEndpointsForBackends(0, 1)},
+                        {"locality1", CreateEndpointsForBackends(1, 2)}});
+  balancer_->ads_service()->SetEdsResource(BuildEdsResource(args));
+  // Populates service labels to CDS resources.
+  auto cluster = default_cluster_;
+  auto& filter_map = *cluster.mutable_metadata()->mutable_filter_metadata();
+  auto& label_map =
+      *filter_map["com.google.csm.telemetry_labels"].mutable_fields();
+  *label_map["service_name"].mutable_string_value() = "myservice";
+  *label_map["service_namespace"].mutable_string_value() = "mynamespace";
+  balancer_->ads_service()->SetCdsResource(cluster);
+  ChannelArguments channel_args;
+  channel_args.SetPointer(GRPC_ARG_INJECT_FAKE_CLIENT_CALL_TRACER_FACTORY,
+                          &fake_client_call_tracer_factory);
+  ResetStub(/*failover_timeout_ms=*/0, &channel_args);
+  // Send an RPC to backend 0.
+  WaitForBackend(DEBUG_LOCATION, 0);
+  // Verify that the optional labels are recorded in the call tracer.
+  EXPECT_THAT(
+      fake_client_call_tracer_factory.GetLastFakeClientCallTracer()
+          ->GetLastCallAttemptTracer()
+          ->GetOptionalLabels(),
+      ::testing::ElementsAre(
+          ::testing::Pair(OptionalLabelKey::kXdsServiceName, "myservice"),
+          ::testing::Pair(OptionalLabelKey::kXdsServiceNamespace,
+                          "mynamespace"),
+          ::testing::Pair(OptionalLabelKey::kLocality,
+                          LocalityNameString("locality0"))));
+  // Send an RPC to backend 1.
+  WaitForBackend(DEBUG_LOCATION, 1);
+  // Verify that the optional labels are recorded in the call
+  // tracer.
+  EXPECT_THAT(
+      fake_client_call_tracer_factory.GetLastFakeClientCallTracer()
+          ->GetLastCallAttemptTracer()
+          ->GetOptionalLabels(),
+      ::testing::ElementsAre(
+          ::testing::Pair(OptionalLabelKey::kXdsServiceName, "myservice"),
+          ::testing::Pair(OptionalLabelKey::kXdsServiceNamespace,
+                          "mynamespace"),
+          ::testing::Pair(OptionalLabelKey::kLocality,
+                          LocalityNameString("locality1"))));
+}
+
 //
 // CDS deletion tests
 //
@@ -326,9 +384,9 @@ TEST_P(CdsDeletionTest, ClusterDeleted) {
   SendRpcsUntil(DEBUG_LOCATION, [](const RpcResult& result) {
     if (result.status.ok()) return true;  // Keep going.
     EXPECT_EQ(StatusCode::UNAVAILABLE, result.status.error_code());
-    EXPECT_EQ(absl::StrCat("CDS resource \"", kDefaultClusterName,
-                           "\" does not exist"),
-              result.status.error_message());
+    EXPECT_EQ(
+        absl::StrCat("CDS resource ", kDefaultClusterName, " does not exist"),
+        result.status.error_message());
     return false;
   });
   // Make sure we ACK'ed the update.
@@ -339,7 +397,7 @@ TEST_P(CdsDeletionTest, ClusterDeleted) {
 
 // Tests that we ignore Cluster deletions if configured to do so.
 TEST_P(CdsDeletionTest, ClusterDeletionIgnored) {
-  InitClient(BootstrapBuilder().SetIgnoreResourceDeletion());
+  InitClient(MakeBootstrapBuilder().SetIgnoreResourceDeletion());
   CreateAndStartBackends(2);
   // Bring up client pointing to backend 0 and wait for it to connect.
   EdsResourceArgs args({{"locality0", CreateEndpointsForBackends(0, 1)}});
@@ -409,6 +467,42 @@ TEST_P(EdsTest, Vanilla) {
             channel_->GetLoadBalancingPolicyName());
 }
 
+TEST_P(EdsTest, MultipleAddressesPerEndpoint) {
+  grpc_core::testing::ScopedExperimentalEnvVar env(
+      "GRPC_EXPERIMENTAL_XDS_DUALSTACK_ENDPOINTS");
+  const size_t kNumRpcsPerAddress = 10;
+  // Create 3 backends, but leave backend 0 unstarted.
+  CreateBackends(3);
+  StartBackend(1);
+  StartBackend(2);
+  // The first endpoint is backends 0 and 1, the second endpoint is backend 2.
+  EdsResourceArgs args({
+      {"locality0",
+       {CreateEndpoint(0, HealthStatus::UNKNOWN, 1, {1}), CreateEndpoint(2)}},
+  });
+  balancer_->ads_service()->SetEdsResource(BuildEdsResource(args));
+  // Initially, backend 0 is offline, so the first endpoint should
+  // connect to backend 1 instead.  Traffic should round-robin across
+  // backends 1 and 2.
+  WaitForAllBackends(DEBUG_LOCATION, 1);  // Wait for backends 1 and 2.
+  CheckRpcSendOk(DEBUG_LOCATION, kNumRpcsPerAddress * 2);
+  EXPECT_EQ(kNumRpcsPerAddress,
+            backends_[1]->backend_service()->request_count());
+  EXPECT_EQ(kNumRpcsPerAddress,
+            backends_[2]->backend_service()->request_count());
+  // Now start backend 0 and shutdown backend 1.
+  StartBackend(0);
+  ShutdownBackend(1);
+  // Wait for traffic to go to backend 0.
+  WaitForBackend(DEBUG_LOCATION, 0);
+  // Traffic should now round-robin across backends 0 and 2.
+  CheckRpcSendOk(DEBUG_LOCATION, kNumRpcsPerAddress * 2);
+  EXPECT_EQ(kNumRpcsPerAddress,
+            backends_[0]->backend_service()->request_count());
+  EXPECT_EQ(kNumRpcsPerAddress,
+            backends_[2]->backend_service()->request_count());
+}
+
 TEST_P(EdsTest, IgnoresUnhealthyEndpoints) {
   CreateAndStartBackends(2);
   const size_t kNumRpcsPerAddress = 100;
@@ -433,20 +527,36 @@ TEST_P(EdsTest, IgnoresUnhealthyEndpoints) {
   }
 }
 
-TEST_P(EdsTest, OneLocalityWithNoEndpoints) {
+// This tests the bug described in https://github.com/grpc/grpc/issues/32486.
+TEST_P(EdsTest, LocalityBecomesEmptyWithDeactivatedChildStateUpdate) {
   CreateAndStartBackends(1);
   // Initial EDS resource has one locality with no endpoints.
-  EdsResourceArgs::Locality empty_locality("locality0", {});
-  EdsResourceArgs args({std::move(empty_locality)});
+  EdsResourceArgs args({{"locality0", CreateEndpointsForBackends()}});
   balancer_->ads_service()->SetEdsResource(BuildEdsResource(args));
-  // RPCs should fail.
+  WaitForAllBackends(DEBUG_LOCATION);
+  // EDS update removes all endpoints from the locality.
+  EdsResourceArgs::Locality empty_locality("locality0", {});
+  args = EdsResourceArgs({std::move(empty_locality)});
+  balancer_->ads_service()->SetEdsResource(BuildEdsResource(args));
+  // Wait for RPCs to start failing.
   constexpr char kErrorMessage[] =
       "no children in weighted_target policy: "
       "EDS resource eds_service_name contains empty localities: "
       "\\[\\{region=\"xds_default_locality_region\", "
       "zone=\"xds_default_locality_zone\", sub_zone=\"locality0\"\\}\\]";
-  CheckRpcSendFailure(DEBUG_LOCATION, StatusCode::UNAVAILABLE, kErrorMessage);
-  // Send EDS resource that has an endpoint.
+  SendRpcsUntil(DEBUG_LOCATION, [&](const RpcResult& result) {
+    if (result.status.ok()) return true;
+    EXPECT_EQ(result.status.error_code(), StatusCode::UNAVAILABLE);
+    EXPECT_THAT(result.status.error_message(),
+                ::testing::MatchesRegex(kErrorMessage));
+    return false;
+  });
+  // Shut down backend.  This triggers a connectivity state update from the
+  // deactivated child of the weighted_target policy.
+  ShutdownAllBackends();
+  // Now restart the backend.
+  StartAllBackends();
+  // Re-add endpoint.
   args = EdsResourceArgs({{"locality0", CreateEndpointsForBackends()}});
   balancer_->ads_service()->SetEdsResource(BuildEdsResource(args));
   // RPCs should eventually succeed.
@@ -454,7 +564,13 @@ TEST_P(EdsTest, OneLocalityWithNoEndpoints) {
     if (!result.status.ok()) {
       EXPECT_EQ(result.status.error_code(), StatusCode::UNAVAILABLE);
       EXPECT_THAT(result.status.error_message(),
-                  ::testing::MatchesRegex(kErrorMessage));
+                  ::testing::MatchesRegex(absl::StrCat(
+                      // The error message we see here depends on whether
+                      // the client sees the EDS update before or after it
+                      // sees the backend come back up.
+                      MakeConnectionFailureRegex(
+                          "connections to all backends failing; last error: "),
+                      "|", kErrorMessage)));
     }
   });
 }
@@ -499,11 +615,8 @@ TEST_P(EdsTest, AllServersUnreachableFailFast) {
   // seconds, and we should disocver in that time that the target backend is
   // down.
   CheckRpcSendFailure(DEBUG_LOCATION, StatusCode::UNAVAILABLE,
-                      "connections to all backends failing; last error: "
-                      "(UNKNOWN: (ipv6:%5B::1%5D|ipv4:127.0.0.1):[0-9]+: "
-                      "Failed to connect to remote host: Connection refused|"
-                      "UNAVAILABLE: (ipv6:%5B::1%5D|ipv4:127.0.0.1):[0-9]+: "
-                      "Failed to connect to remote host: FD shutdown)",
+                      MakeConnectionFailureRegex(
+                          "connections to all backends failing; last error: "),
                       RpcOptions().set_timeout_ms(kRpcTimeoutMs));
 }
 
@@ -526,11 +639,8 @@ TEST_P(EdsTest, BackendsRestart) {
                                ::testing::Eq(GRPC_CHANNEL_CONNECTING)));
   // RPCs should fail.
   CheckRpcSendFailure(DEBUG_LOCATION, StatusCode::UNAVAILABLE,
-                      "connections to all backends failing; last error: "
-                      "(UNKNOWN: (ipv6:%5B::1%5D|ipv4:127.0.0.1):[0-9]+: "
-                      "Failed to connect to remote host: Connection refused|"
-                      "UNAVAILABLE: (ipv6:%5B::1%5D|ipv4:127.0.0.1):[0-9]+: "
-                      "Failed to connect to remote host: FD shutdown)");
+                      MakeConnectionFailureRegex(
+                          "connections to all backends failing; last error: "));
   // Restart all backends.  RPCs should start succeeding again.
   StartAllBackends();
   CheckRpcSendOk(DEBUG_LOCATION, 1,
@@ -606,7 +716,7 @@ TEST_P(EdsTest, KeepUsingLastDataIfBalancerGoesDown) {
 
 // Tests that the localities in a locality map are picked according to their
 // weights.
-TEST_P(EdsTest, WeightedRoundRobin) {
+TEST_P(EdsTest, LocalityWeights) {
   CreateAndStartBackends(2);
   const int kLocalityWeight0 = 2;
   const int kLocalityWeight1 = 8;
@@ -680,6 +790,32 @@ TEST_P(EdsTest, NoIntegerOverflowInLocalityWeights) {
               ::testing::DoubleNear(kLocalityWeightRate1, kErrorTolerance));
 }
 
+TEST_P(EdsTest, OneLocalityWithNoEndpoints) {
+  CreateAndStartBackends(1);
+  // Initial EDS resource has one locality with no endpoints.
+  EdsResourceArgs::Locality empty_locality("locality0", {});
+  EdsResourceArgs args({std::move(empty_locality)});
+  balancer_->ads_service()->SetEdsResource(BuildEdsResource(args));
+  // RPCs should fail.
+  constexpr char kErrorMessage[] =
+      "no children in weighted_target policy: "
+      "EDS resource eds_service_name contains empty localities: "
+      "\\[\\{region=\"xds_default_locality_region\", "
+      "zone=\"xds_default_locality_zone\", sub_zone=\"locality0\"\\}\\]";
+  CheckRpcSendFailure(DEBUG_LOCATION, StatusCode::UNAVAILABLE, kErrorMessage);
+  // Send EDS resource that has an endpoint.
+  args = EdsResourceArgs({{"locality0", CreateEndpointsForBackends()}});
+  balancer_->ads_service()->SetEdsResource(BuildEdsResource(args));
+  // RPCs should eventually succeed.
+  WaitForAllBackends(DEBUG_LOCATION, 0, 1, [&](const RpcResult& result) {
+    if (!result.status.ok()) {
+      EXPECT_EQ(result.status.error_code(), StatusCode::UNAVAILABLE);
+      EXPECT_THAT(result.status.error_message(),
+                  ::testing::MatchesRegex(kErrorMessage));
+    }
+  });
+}
+
 // Tests that we correctly handle a locality containing no endpoints.
 TEST_P(EdsTest, LocalityContainingNoEndpoints) {
   CreateAndStartBackends(2);
@@ -763,10 +899,10 @@ TEST_P(EdsTest, LocalityMapUpdateChurn) {
   balancer_->ads_service()->SetEdsResource(BuildEdsResource(args));
   // Wait for the first 3 backends to be ready.
   WaitForAllBackends(DEBUG_LOCATION, 0, 3);
-  gpr_log(GPR_INFO, "========= BEFORE FIRST BATCH ==========");
+  LOG(INFO) << "========= BEFORE FIRST BATCH ==========";
   // Send kNumRpcs RPCs.
   CheckRpcSendOk(DEBUG_LOCATION, kNumRpcs);
-  gpr_log(GPR_INFO, "========= DONE WITH FIRST BATCH ==========");
+  LOG(INFO) << "========= DONE WITH FIRST BATCH ==========";
   // The picking rates of the first 3 backends should be roughly equal to the
   // expectation.
   std::vector<double> locality_picked_rates;
@@ -777,8 +913,7 @@ TEST_P(EdsTest, LocalityMapUpdateChurn) {
   }
   const double kErrorTolerance = 0.2;
   for (size_t i = 0; i < 3; ++i) {
-    gpr_log(GPR_INFO, "Locality %" PRIuPTR " rate %f", i,
-            locality_picked_rates[i]);
+    LOG(INFO) << "Locality " << i << " rate " << locality_picked_rates[i];
     EXPECT_THAT(
         locality_picked_rates[i],
         ::testing::AllOf(
@@ -796,10 +931,10 @@ TEST_P(EdsTest, LocalityMapUpdateChurn) {
   // Wait until the locality update has been processed, as signaled by backend
   // 3 receiving a request.
   WaitForAllBackends(DEBUG_LOCATION, 3, 4);
-  gpr_log(GPR_INFO, "========= BEFORE SECOND BATCH ==========");
+  LOG(INFO) << "========= BEFORE SECOND BATCH ==========";
   // Send kNumRpcs RPCs.
   CheckRpcSendOk(DEBUG_LOCATION, kNumRpcs);
-  gpr_log(GPR_INFO, "========= DONE WITH SECOND BATCH ==========");
+  LOG(INFO) << "========= DONE WITH SECOND BATCH ==========";
   // Backend 0 no longer receives any request.
   EXPECT_EQ(0U, backends_[0]->backend_service()->request_count());
   // The picking rates of the last 3 backends should be roughly equal to the
@@ -811,8 +946,7 @@ TEST_P(EdsTest, LocalityMapUpdateChurn) {
         kNumRpcs);
   }
   for (size_t i = 1; i < 4; ++i) {
-    gpr_log(GPR_INFO, "Locality %" PRIuPTR " rate %f", i,
-            locality_picked_rates[i]);
+    LOG(INFO) << "Locality " << i << " rate " << locality_picked_rates[i];
     EXPECT_THAT(
         locality_picked_rates[i],
         ::testing::AllOf(
@@ -956,14 +1090,14 @@ TEST_P(EdsTest, DropConfigUpdate) {
   args.drop_categories = {{kLbDropType, kDropPerMillionForLb}};
   balancer_->ads_service()->SetEdsResource(BuildEdsResource(args));
   // Send kNumRpcsLbOnly RPCs and count the drops.
-  gpr_log(GPR_INFO, "========= BEFORE FIRST BATCH ==========");
+  LOG(INFO) << "========= BEFORE FIRST BATCH ==========";
   size_t num_drops = SendRpcsAndCountFailuresWithMessage(
       DEBUG_LOCATION, kNumRpcsLbOnly, StatusCode::UNAVAILABLE,
       kStatusMessageDropPrefix);
-  gpr_log(GPR_INFO, "========= DONE WITH FIRST BATCH ==========");
+  LOG(INFO) << "========= DONE WITH FIRST BATCH ==========";
   // The drop rate should be roughly equal to the expectation.
   double seen_drop_rate = static_cast<double>(num_drops) / kNumRpcsLbOnly;
-  gpr_log(GPR_INFO, "First batch drop rate %f", seen_drop_rate);
+  LOG(INFO) << "First batch drop rate " << seen_drop_rate;
   EXPECT_THAT(seen_drop_rate,
               ::testing::DoubleNear(kDropRateForLb, kErrorTolerance));
   // The second ADS response contains two drop categories, send an update EDS
@@ -993,14 +1127,14 @@ TEST_P(EdsTest, DropConfigUpdate) {
       },
       /*timeout_ms=*/40000);
   // Send kNumRpcsBoth RPCs and count the drops.
-  gpr_log(GPR_INFO, "========= BEFORE SECOND BATCH ==========");
+  LOG(INFO) << "========= BEFORE SECOND BATCH ==========";
   num_drops = SendRpcsAndCountFailuresWithMessage(DEBUG_LOCATION, kNumRpcsBoth,
                                                   StatusCode::UNAVAILABLE,
                                                   kStatusMessageDropPrefix);
-  gpr_log(GPR_INFO, "========= DONE WITH SECOND BATCH ==========");
+  LOG(INFO) << "========= DONE WITH SECOND BATCH ==========";
   // The new drop rate should be roughly equal to the expectation.
   seen_drop_rate = static_cast<double>(num_drops) / kNumRpcsBoth;
-  gpr_log(GPR_INFO, "Second batch drop rate %f", seen_drop_rate);
+  LOG(INFO) << "Second batch drop rate " << seen_drop_rate;
   EXPECT_THAT(seen_drop_rate, ::testing::DoubleNear(kDropRateForLbAndThrottle,
                                                     kErrorTolerance));
 }
@@ -1133,9 +1267,9 @@ TEST_P(FailoverTest, ReportsConnectingDuringFailover) {
   // Allow the connection attempt to complete.
   hold->Resume();
   // Now the RPC should complete successfully.
-  gpr_log(GPR_INFO, "=== WAITING FOR RPC TO FINISH ===");
+  LOG(INFO) << "=== WAITING FOR RPC TO FINISH ===";
   Status status = rpc.GetStatus();
-  gpr_log(GPR_INFO, "=== RPC FINISHED ===");
+  LOG(INFO) << "=== RPC FINISHED ===";
   EXPECT_TRUE(status.ok()) << "code=" << status.error_code()
                            << " message=" << status.error_message();
 }
@@ -1176,14 +1310,9 @@ TEST_P(FailoverTest, UpdateInitialUnavailable) {
       {"locality1", {MakeNonExistantEndpoint()}, kDefaultLocalityWeight, 1},
   });
   balancer_->ads_service()->SetEdsResource(BuildEdsResource(args));
-  constexpr char kErrorMessageRegex[] =
-      "connections to all backends failing; last error: "
-      "(UNKNOWN: (ipv6:%5B::1%5D|ipv4:127.0.0.1):[0-9]+: "
-      "Failed to connect to remote host: Connection refused|"
-      "UNAVAILABLE: (ipv6:%5B::1%5D|ipv4:127.0.0.1):[0-9]+: "
-      "Failed to connect to remote host: FD shutdown)";
   CheckRpcSendFailure(DEBUG_LOCATION, StatusCode::UNAVAILABLE,
-                      kErrorMessageRegex);
+                      MakeConnectionFailureRegex(
+                          "connections to all backends failing; last error: "));
   args = EdsResourceArgs({
       {"locality0", CreateEndpointsForBackends(0, 1), kDefaultLocalityWeight,
        0},
@@ -1195,7 +1324,8 @@ TEST_P(FailoverTest, UpdateInitialUnavailable) {
     if (!result.status.ok()) {
       EXPECT_EQ(result.status.error_code(), StatusCode::UNAVAILABLE);
       EXPECT_THAT(result.status.error_message(),
-                  ::testing::MatchesRegex(kErrorMessageRegex));
+                  ::testing::MatchesRegex(MakeConnectionFailureRegex(
+                      "connections to all backends failing; last error: ")));
     }
   });
 }
@@ -1336,6 +1466,18 @@ INSTANTIATE_TEST_SUITE_P(
     ::testing::Values(XdsTestType().set_enable_load_reporting()),
     &XdsTestType::Name);
 
+MATCHER_P2(LoadMetricEq, num_requests_finished_with_metric, total_metric_value,
+           "equals LoadMetric") {
+  bool match = true;
+  match &= ::testing::ExplainMatchResult(num_requests_finished_with_metric,
+                                         arg.num_requests_finished_with_metric,
+                                         result_listener);
+  match &=
+      ::testing::ExplainMatchResult(::testing::DoubleEq(total_metric_value),
+                                    arg.total_metric_value, result_listener);
+  return match;
+}
+
 // Tests that the load report received at the balancer is correct.
 TEST_P(ClientLoadReportingTest, Vanilla) {
   CreateAndStartBackends(4);
@@ -1350,11 +1492,19 @@ TEST_P(ClientLoadReportingTest, Vanilla) {
   size_t num_warmup_rpcs =
       WaitForAllBackends(DEBUG_LOCATION, 0, 4, /*check_status=*/nullptr,
                          WaitForBackendOptions().set_reset_counters(false));
-  // Send kNumRpcsPerAddress RPCs per server.
-  CheckRpcSendOk(DEBUG_LOCATION, kNumRpcsPerAddress * backends_.size());
+  // Send kNumRpcsPerAddress RPCs per server with named metrics.
+  xds::data::orca::v3::OrcaLoadReport backend_metrics;
+  auto& named_metrics = (*backend_metrics.mutable_named_metrics());
+  named_metrics["foo"] = 1.0;
+  named_metrics["bar"] = 2.0;
+  CheckRpcSendOk(DEBUG_LOCATION, kNumRpcsPerAddress * backends_.size(),
+                 RpcOptions().set_backend_metrics(backend_metrics));
+  named_metrics["foo"] = 0.3;
+  named_metrics["bar"] = 0.4;
   for (size_t i = 0; i < kNumFailuresPerAddress * backends_.size(); ++i) {
     CheckRpcSendFailure(DEBUG_LOCATION, StatusCode::FAILED_PRECONDITION, "",
-                        RpcOptions().set_server_fail(true));
+                        RpcOptions().set_server_fail(true).set_backend_metrics(
+                            backend_metrics));
   }
   const size_t total_successful_rpcs_sent =
       (kNumRpcsPerAddress * backends_.size()) + num_warmup_rpcs;
@@ -1386,6 +1536,8 @@ TEST_P(ClientLoadReportingTest, Vanilla) {
                              ::testing::Pair("locality1", ::testing::_)));
   size_t num_successful_rpcs = 0;
   size_t num_failed_rpcs = 0;
+  std::map<std::string, ClientStats::LocalityStats::LoadMetric>
+      named_metrics_total;
   for (const auto& p : client_stats.locality_stats()) {
     EXPECT_EQ(p.second.total_requests_in_progress, 0U);
     EXPECT_EQ(
@@ -1393,10 +1545,30 @@ TEST_P(ClientLoadReportingTest, Vanilla) {
         p.second.total_successful_requests + p.second.total_error_requests);
     num_successful_rpcs += p.second.total_successful_requests;
     num_failed_rpcs += p.second.total_error_requests;
+    for (const auto& s : p.second.load_metrics) {
+      named_metrics_total[s.first] += s.second;
+    }
   }
   EXPECT_EQ(num_successful_rpcs, total_successful_rpcs_sent);
   EXPECT_EQ(num_failed_rpcs, total_failed_rpcs_sent);
   EXPECT_EQ(num_successful_rpcs + num_failed_rpcs, total_rpcs_sent);
+  EXPECT_THAT(
+      named_metrics_total,
+      ::testing::UnorderedElementsAre(
+          ::testing::Pair(
+              "foo",
+              LoadMetricEq(
+                  (kNumRpcsPerAddress + kNumFailuresPerAddress) *
+                      backends_.size(),
+                  (kNumRpcsPerAddress * backends_.size()) * 1.0 +
+                      (kNumFailuresPerAddress * backends_.size()) * 0.3)),
+          ::testing::Pair(
+              "bar",
+              LoadMetricEq(
+                  (kNumRpcsPerAddress + kNumFailuresPerAddress) *
+                      backends_.size(),
+                  (kNumRpcsPerAddress * backends_.size()) * 2.0 +
+                      (kNumFailuresPerAddress * backends_.size()) * 0.4))));
   // The LRS service got a single request, and sent a single response.
   EXPECT_EQ(1U, balancer_->lrs_service()->request_count());
   EXPECT_EQ(1U, balancer_->lrs_service()->response_count());
@@ -1413,10 +1585,18 @@ TEST_P(ClientLoadReportingTest, SendAllClusters) {
   // Wait until all backends are ready.
   size_t num_warmup_rpcs = WaitForAllBackends(DEBUG_LOCATION);
   // Send kNumRpcsPerAddress RPCs per server.
-  CheckRpcSendOk(DEBUG_LOCATION, kNumRpcsPerAddress * backends_.size());
+  xds::data::orca::v3::OrcaLoadReport backend_metrics;
+  auto& named_metrics = (*backend_metrics.mutable_named_metrics());
+  named_metrics["foo"] = 1.0;
+  named_metrics["bar"] = 2.0;
+  CheckRpcSendOk(DEBUG_LOCATION, kNumRpcsPerAddress * backends_.size(),
+                 RpcOptions().set_backend_metrics(backend_metrics));
+  named_metrics["foo"] = 0.3;
+  named_metrics["bar"] = 0.4;
   for (size_t i = 0; i < kNumFailuresPerAddress * backends_.size(); ++i) {
     CheckRpcSendFailure(DEBUG_LOCATION, StatusCode::FAILED_PRECONDITION, "",
-                        RpcOptions().set_server_fail(true));
+                        RpcOptions().set_server_fail(true).set_backend_metrics(
+                            backend_metrics));
   }
   // Check that each backend got the right number of requests.
   for (size_t i = 0; i < backends_.size(); ++i) {
@@ -1437,6 +1617,29 @@ TEST_P(ClientLoadReportingTest, SendAllClusters) {
   EXPECT_EQ(kNumFailuresPerAddress * backends_.size(),
             client_stats.total_error_requests());
   EXPECT_EQ(0U, client_stats.total_dropped_requests());
+  EXPECT_THAT(
+      client_stats.locality_stats(),
+      ::testing::ElementsAre(::testing::Pair(
+          "locality0",
+          ::testing::Field(
+              &ClientStats::LocalityStats::load_metrics,
+              ::testing::UnorderedElementsAre(
+                  ::testing::Pair(
+                      "foo",
+                      LoadMetricEq(
+                          (kNumRpcsPerAddress + kNumFailuresPerAddress) *
+                              backends_.size(),
+                          (kNumRpcsPerAddress * backends_.size()) * 1.0 +
+                              (kNumFailuresPerAddress * backends_.size()) *
+                                  0.3)),
+                  ::testing::Pair(
+                      "bar",
+                      LoadMetricEq(
+                          (kNumRpcsPerAddress + kNumFailuresPerAddress) *
+                              backends_.size(),
+                          (kNumRpcsPerAddress * backends_.size()) * 2.0 +
+                              (kNumFailuresPerAddress * backends_.size()) *
+                                  0.4)))))));
   // The LRS service got a single request, and sent a single response.
   EXPECT_EQ(1U, balancer_->lrs_service()->request_count());
   EXPECT_EQ(1U, balancer_->lrs_service()->response_count());
@@ -1476,6 +1679,11 @@ TEST_P(ClientLoadReportingTest, BalancerRestart) {
   EXPECT_EQ(0U, client_stats.total_requests_in_progress());
   EXPECT_EQ(0U, client_stats.total_error_requests());
   EXPECT_EQ(0U, client_stats.total_dropped_requests());
+  EXPECT_THAT(client_stats.locality_stats(),
+              ::testing::ElementsAre(::testing::Pair(
+                  "locality0",
+                  ::testing::Field(&ClientStats::LocalityStats::load_metrics,
+                                   ::testing::IsEmpty()))));
   // Shut down the balancer.
   balancer_->Shutdown();
   // We should continue using the last EDS response we received from the
@@ -1498,7 +1706,12 @@ TEST_P(ClientLoadReportingTest, BalancerRestart) {
   // This tells us that we're now using the new serverlist.
   num_rpcs += WaitForAllBackends(DEBUG_LOCATION, 2, 4);
   // Send one RPC per backend.
-  CheckRpcSendOk(DEBUG_LOCATION, 2);
+  xds::data::orca::v3::OrcaLoadReport backend_metrics;
+  auto& named_metrics = (*backend_metrics.mutable_named_metrics());
+  named_metrics["foo"] = 1.0;
+  named_metrics["bar"] = 2.0;
+  CheckRpcSendOk(DEBUG_LOCATION, 2,
+                 RpcOptions().set_backend_metrics(backend_metrics));
   num_rpcs += 2;
   // Check client stats.
   load_report = balancer_->lrs_service()->WaitForLoadReport();
@@ -1508,6 +1721,14 @@ TEST_P(ClientLoadReportingTest, BalancerRestart) {
   EXPECT_EQ(0U, client_stats.total_requests_in_progress());
   EXPECT_EQ(0U, client_stats.total_error_requests());
   EXPECT_EQ(0U, client_stats.total_dropped_requests());
+  EXPECT_THAT(client_stats.locality_stats(),
+              ::testing::ElementsAre(::testing::Pair(
+                  "locality0",
+                  ::testing::Field(
+                      &ClientStats::LocalityStats::load_metrics,
+                      ::testing::UnorderedElementsAre(
+                          ::testing::Pair("foo", LoadMetricEq(2, 2.0)),
+                          ::testing::Pair("bar", LoadMetricEq(2, 4.0)))))));
 }
 
 // Tests load reporting when switching over from one cluster to another.
@@ -1561,7 +1782,10 @@ TEST_P(ClientLoadReportingTest, ChangeClusters) {
                           0UL),
                       ::testing::Field(
                           &ClientStats::LocalityStats::total_issued_requests,
-                          num_rpcs))))),
+                          num_rpcs),
+                      ::testing::Field(
+                          &ClientStats::LocalityStats::load_metrics,
+                          ::testing::IsEmpty()))))),
           ::testing::Property(&ClientStats::total_dropped_requests, 0UL))));
   // Change RDS resource to point to new cluster.
   RouteConfiguration new_route_config = default_route_config_;
@@ -1599,7 +1823,10 @@ TEST_P(ClientLoadReportingTest, ChangeClusters) {
                               0UL),
                           ::testing::Field(&ClientStats::LocalityStats::
                                                total_issued_requests,
-                                           ::testing::Le(num_rpcs)))))),
+                                           ::testing::Le(num_rpcs)),
+                          ::testing::Field(
+                              &ClientStats::LocalityStats::load_metrics,
+                              ::testing::IsEmpty()))))),
               ::testing::Property(&ClientStats::total_dropped_requests, 0UL)),
           ::testing::AllOf(
               ::testing::Property(&ClientStats::cluster_name, kNewClusterName),
@@ -1621,7 +1848,10 @@ TEST_P(ClientLoadReportingTest, ChangeClusters) {
                               0UL),
                           ::testing::Field(&ClientStats::LocalityStats::
                                                total_issued_requests,
-                                           ::testing::Le(num_rpcs)))))),
+                                           ::testing::Le(num_rpcs)),
+                          ::testing::Field(
+                              &ClientStats::LocalityStats::load_metrics,
+                              ::testing::IsEmpty()))))),
               ::testing::Property(&ClientStats::total_dropped_requests, 0UL))));
   size_t total_ok = 0;
   for (const ClientStats& client_stats : load_report) {
@@ -1688,11 +1918,14 @@ int main(int argc, char** argv) {
   ::testing::InitGoogleTest(&argc, argv);
   // Make the backup poller poll very frequently in order to pick up
   // updates from all the subchannels's FDs.
-  GPR_GLOBAL_CONFIG_SET(grpc_client_channel_backup_poll_interval_ms, 1);
+  grpc_core::ConfigVars::Overrides overrides;
+  overrides.client_channel_backup_poll_interval_ms = 1;
+  grpc_core::ConfigVars::SetOverrides(overrides);
 #if TARGET_OS_IPHONE
   // Workaround Apple CFStream bug
   grpc_core::SetEnv("grpc_cfstream", "0");
 #endif
+  grpc_core::RegisterFakeStatsPlugin();
   grpc_init();
   grpc::testing::ConnectionAttemptInjector::Init();
   const auto result = RUN_ALL_TESTS();
