@@ -18,6 +18,8 @@
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
 
+#include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "absl/status/statusor.h"
 #include "absl/strings/str_cat.h"
 
@@ -31,8 +33,8 @@
 #include "src/core/resolver/endpoint_addresses.h"
 #include "src/core/resolver/fake/fake_resolver.h"
 #include "src/proto/grpc/testing/xds/v3/aggregate_cluster.grpc.pb.h"
-#include "test/core/util/resolve_localhost_ip46.h"
-#include "test/core/util/scoped_env_var.h"
+#include "test/core/test_util/resolve_localhost_ip46.h"
+#include "test/core/test_util/scoped_env_var.h"
 #include "test/cpp/end2end/connection_attempt_injector.h"
 #include "test/cpp/end2end/xds/xds_end2end_test_lib.h"
 
@@ -63,9 +65,9 @@ class ClusterTypeTest : public XdsEnd2endTest {
     for (int port : ports) {
       absl::StatusOr<grpc_core::URI> lb_uri =
           grpc_core::URI::Parse(grpc_core::LocalIpUri(port));
-      GPR_ASSERT(lb_uri.ok());
+      CHECK_OK(lb_uri);
       grpc_resolved_address address;
-      GPR_ASSERT(grpc_parse_uri(*lb_uri, &address));
+      CHECK(grpc_parse_uri(*lb_uri, &address));
       addresses.emplace_back(address, grpc_core::ChannelArgs());
     }
     return addresses;
@@ -366,9 +368,9 @@ TEST_P(AggregateClusterTest, FallBackWithConnectivityChurn) {
   // Meanwhile, the channel will also start a second attempt for backend
   // 0, which we have NOT held, so it will complete normally, and the
   // RPC will finish on backend 0.
-  gpr_log(GPR_INFO, "=== WAITING FOR RPC TO FINISH === ");
+  LOG(INFO) << "=== WAITING FOR RPC TO FINISH === ";
   Status status = rpc.GetStatus();
-  gpr_log(GPR_INFO, "=== RPC FINISHED === ");
+  LOG(INFO) << "=== RPC FINISHED === ";
   EXPECT_TRUE(status.ok()) << "code=" << status.error_code()
                            << " message=" << status.error_message();
   EXPECT_EQ(1UL, backends_[0]->backend_service()->request_count());
