@@ -23,12 +23,12 @@
 #include <thread>
 #include <vector>
 
-#include <gmock/gmock.h>
-#include <gtest/gtest.h>
-
+#include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "absl/types/optional.h"
 
 #include <grpc/support/log.h>
+#include <grpcpp/support/status.h>
 
 #include "src/core/lib/address_utils/parse_address.h"
 #include "src/core/lib/gprpp/crash.h"
@@ -41,7 +41,7 @@
 #include "src/proto/grpc/testing/xds/v3/listener.grpc.pb.h"
 #include "src/proto/grpc/testing/xds/v3/lrs.grpc.pb.h"
 #include "src/proto/grpc/testing/xds/v3/route.grpc.pb.h"
-#include "test/core/util/test_config.h"
+#include "test/core/test_util/test_config.h"
 #include "test/cpp/end2end/counted_service.h"
 
 namespace grpc {
@@ -62,6 +62,9 @@ class AdsServiceImpl
           ::envoy::service::discovery::v3::AggregatedDiscoveryService::Service>,
       public std::enable_shared_from_this<AdsServiceImpl> {
  public:
+  using DiscoveryRequest = ::envoy::service::discovery::v3::DiscoveryRequest;
+  using DiscoveryResponse = ::envoy::service::discovery::v3::DiscoveryResponse;
+
   // State for a given xDS resource type.
   struct ResponseState {
     enum State {
@@ -72,7 +75,15 @@ class AdsServiceImpl
     std::string error_message;
   };
 
-  AdsServiceImpl() {}
+  explicit AdsServiceImpl(
+      std::function<void(const DiscoveryRequest& request)> check_first_request =
+          nullptr,
+      std::function<void(absl::StatusCode)> check_nack_status_code = nullptr,
+      absl::string_view debug_label = "")
+      : check_first_request_(std::move(check_first_request)),
+        check_nack_status_code_(std::move(check_nack_status_code)),
+        debug_label_(absl::StrFormat(
+            "%p%s%s", this, debug_label.empty() ? "" : ":", debug_label)) {}
 
   void set_wrap_resources(bool wrap_resources) {
     grpc_core::MutexLock lock(&ads_mu_);
@@ -119,12 +130,12 @@ class AdsServiceImpl
     resource_types_to_ignore_.emplace(type_url);
   }
 
-  // Sets the minimum version that the server will accept for a given
-  // resource type.  Will cause a gmock expectation failure if we see a
-  // lower version.
-  void SetResourceMinVersion(const std::string& type_url, int version) {
+  // Sets a callback to be invoked on request messages with respoonse_nonce
+  // set.  The callback is passed the resource type and version.
+  void SetCheckVersionCallback(
+      std::function<void(absl::string_view, int)> check_version_callack) {
     grpc_core::MutexLock lock(&ads_mu_);
-    resource_type_min_versions_[type_url] = version;
+    check_version_callack_ = std::move(check_version_callack);
   }
 
   // Get the list of response state for each resource type.
@@ -169,6 +180,11 @@ class AdsServiceImpl
     forced_ads_failure_ = std::move(status);
   }
 
+  void ClearADSFailure() {
+    grpc_core::MutexLock lock(&ads_mu_);
+    forced_ads_failure_ = absl::nullopt;
+  }
+
  private:
   // A queue of resource type/name pairs that have changed since the client
   // subscribed to them.
@@ -214,21 +230,20 @@ class AdsServiceImpl
 
   using ResourceMap = std::map<std::string /* type_url */, ResourceTypeState>;
 
-  using DiscoveryRequest = ::envoy::service::discovery::v3::DiscoveryRequest;
-  using DiscoveryResponse = ::envoy::service::discovery::v3::DiscoveryResponse;
   using Stream = ServerReaderWriter<DiscoveryResponse, DiscoveryRequest>;
 
   Status StreamAggregatedResources(ServerContext* context,
                                    Stream* stream) override {
-    gpr_log(GPR_INFO, "ADS[%p]: StreamAggregatedResources starts", this);
+    LOG(INFO) << "ADS[" << debug_label_
+              << "]: StreamAggregatedResources starts";
     {
       grpc_core::MutexLock lock(&ads_mu_);
       if (forced_ads_failure_.has_value()) {
-        gpr_log(GPR_INFO,
-                "ADS[%p]: StreamAggregatedResources forcing early failure "
-                "with status code: %d, message: %s",
-                this, forced_ads_failure_.value().error_code(),
-                forced_ads_failure_.value().error_message().c_str());
+        LOG(INFO) << "ADS[" << debug_label_
+                  << "]: StreamAggregatedResources forcing early failure "
+                     "with status code: "
+                  << forced_ads_failure_.value().error_code() << ", message: "
+                  << forced_ads_failure_.value().error_message();
         return forced_ads_failure_.value();
       }
     }
@@ -269,9 +284,9 @@ class AdsServiceImpl
           DiscoveryRequest request = std::move(requests.front());
           requests.pop_front();
           did_work = true;
-          gpr_log(GPR_INFO,
-                  "ADS[%p]: Received request for type %s with content %s", this,
-                  request.type_url().c_str(), request.DebugString().c_str());
+          LOG(INFO) << "ADS[" << debug_label_ << "]: Received request for type "
+                    << request.type_url() << " with content "
+                    << request.DebugString();
           SentState& sent_state = sent_state_map[request.type_url()];
           // Process request.
           ProcessRequest(request, &update_queue, &subscription_map, &sent_state,
@@ -279,8 +294,8 @@ class AdsServiceImpl
         }
       }
       if (response.has_value()) {
-        gpr_log(GPR_INFO, "ADS[%p]: Sending response: %s", this,
-                response->DebugString().c_str());
+        LOG(INFO) << "ADS[" << debug_label_
+                  << "]: Sending response: " << response->DebugString();
         stream->Write(response.value());
       }
       response.reset();
@@ -300,8 +315,8 @@ class AdsServiceImpl
         }
       }
       if (response.has_value()) {
-        gpr_log(GPR_INFO, "ADS[%p]: Sending update response: %s", this,
-                response->DebugString().c_str());
+        LOG(INFO) << "ADS[" << debug_label_
+                  << "]: Sending update response: " << response->DebugString();
         stream->Write(response.value());
       }
       {
@@ -335,7 +350,7 @@ class AdsServiceImpl
         }
       }
     }
-    gpr_log(GPR_INFO, "ADS[%p]: StreamAggregatedResources done", this);
+    LOG(INFO) << "ADS[" << debug_label_ << "]: StreamAggregatedResources done";
     RemoveClient(context->peer());
     return Status::OK;
   }
@@ -352,30 +367,34 @@ class AdsServiceImpl
     if (request.response_nonce().empty()) {
       int client_resource_type_version = 0;
       if (!request.version_info().empty()) {
-        GPR_ASSERT(absl::SimpleAtoi(request.version_info(),
-                                    &client_resource_type_version));
+        CHECK(absl::SimpleAtoi(request.version_info(),
+                               &client_resource_type_version));
+      }
+      if (check_version_callack_ != nullptr) {
+        check_version_callack_(request.type_url(),
+                               client_resource_type_version);
       }
-      EXPECT_GE(client_resource_type_version,
-                resource_type_min_versions_[request.type_url()])
-          << "resource_type: " << request.type_url();
     } else {
       int client_nonce;
-      GPR_ASSERT(absl::SimpleAtoi(request.response_nonce(), &client_nonce));
+      CHECK(absl::SimpleAtoi(request.response_nonce(), &client_nonce));
       // Check for ACK or NACK.
       ResponseState response_state;
       if (!request.has_error_detail()) {
         response_state.state = ResponseState::ACKED;
-        gpr_log(GPR_INFO, "ADS[%p]: client ACKed resource_type=%s version=%s",
-                this, request.type_url().c_str(),
-                request.version_info().c_str());
+        LOG(INFO) << "ADS[" << debug_label_
+                  << "]: client ACKed resource_type=" << request.type_url()
+                  << " version=" << request.version_info();
       } else {
         response_state.state = ResponseState::NACKED;
-        EXPECT_EQ(request.error_detail().code(), GRPC_STATUS_INVALID_ARGUMENT);
+        if (check_nack_status_code_ != nullptr) {
+          check_nack_status_code_(
+              static_cast<absl::StatusCode>(request.error_detail().code()));
+        }
         response_state.error_message = request.error_detail().message();
-        gpr_log(GPR_INFO,
-                "ADS[%p]: client NACKed resource_type=%s version=%s: %s", this,
-                request.type_url().c_str(), request.version_info().c_str(),
-                response_state.error_message.c_str());
+        LOG(INFO) << "ADS[" << debug_label_
+                  << "]: client NACKed resource_type=" << request.type_url()
+                  << " version=" << request.version_info() << ": "
+                  << response_state.error_message;
       }
       resource_type_response_state_[request.type_url()].emplace_back(
           std::move(response_state));
@@ -405,8 +424,9 @@ class AdsServiceImpl
                          &resource_state, update_queue) ||
           ClientNeedsResourceUpdate(resource_type_state, resource_state,
                                     sent_state->resource_type_version)) {
-        gpr_log(GPR_INFO, "ADS[%p]: Sending update for type=%s name=%s", this,
-                request.type_url().c_str(), resource_name.c_str());
+        LOG(INFO) << "ADS[" << debug_label_
+                  << "]: Sending update for type=" << request.type_url()
+                  << " name=" << resource_name;
         resources_added_to_response.emplace(resource_name);
         if (!response->has_value()) response->emplace();
         if (resource_state.resource.has_value()) {
@@ -419,9 +439,9 @@ class AdsServiceImpl
           }
         }
       } else {
-        gpr_log(GPR_INFO,
-                "ADS[%p]: client does not need update for type=%s name=%s",
-                this, request.type_url().c_str(), resource_name.c_str());
+        LOG(INFO) << "ADS[" << debug_label_
+                  << "]: client does not need update for type="
+                  << request.type_url() << " name=" << resource_name;
       }
     }
     // Process unsubscriptions for any resource no longer
@@ -444,8 +464,9 @@ class AdsServiceImpl
                      SubscriptionMap* subscription_map, SentState* sent_state,
                      absl::optional<DiscoveryResponse>* response)
       ABSL_EXCLUSIVE_LOCKS_REQUIRED(ads_mu_) {
-    gpr_log(GPR_INFO, "ADS[%p]: Received update for type=%s name=%s", this,
-            resource_type.c_str(), resource_name.c_str());
+    LOG(INFO) << "ADS[" << debug_label_
+              << "]: Received update for type=" << resource_type
+              << " name=" << resource_name;
     auto& subscription_name_map = (*subscription_map)[resource_type];
     auto& resource_type_state = resource_map_[resource_type];
     auto& resource_name_map = resource_type_state.resource_name_map;
@@ -454,8 +475,9 @@ class AdsServiceImpl
       ResourceState& resource_state = resource_name_map[resource_name];
       if (ClientNeedsResourceUpdate(resource_type_state, resource_state,
                                     sent_state->resource_type_version)) {
-        gpr_log(GPR_INFO, "ADS[%p]: Sending update for type=%s name=%s", this,
-                resource_type.c_str(), resource_name.c_str());
+        LOG(INFO) << "ADS[" << debug_label_
+                  << "]: Sending update for type=" << resource_type
+                  << " name=" << resource_name;
         response->emplace();
         if (resource_state.resource.has_value()) {
           auto* resource = (*response)->add_resources();
@@ -476,11 +498,9 @@ class AdsServiceImpl
     bool seen_first_request = false;
     while (stream->Read(&request)) {
       if (!seen_first_request) {
-        EXPECT_TRUE(request.has_node());
-        EXPECT_THAT(request.node().client_features(),
-                    ::testing::UnorderedElementsAre(
-                        "envoy.lb.does_not_support_overprovisioning",
-                        "xds.config.resource-in-sotw"));
+        if (check_first_request_ != nullptr) {
+          check_first_request_(request);
+        }
         seen_first_request = true;
       }
       {
@@ -488,7 +508,7 @@ class AdsServiceImpl
         requests->emplace_back(std::move(request));
       }
     }
-    gpr_log(GPR_INFO, "ADS[%p]: Null read, stream closed", this);
+    LOG(INFO) << "ADS[" << debug_label_ << "]: Null read, stream closed";
     grpc_core::MutexLock lock(&ads_mu_);
     *stream_closed = true;
   }
@@ -557,6 +577,10 @@ class AdsServiceImpl
     clients_.erase(client);
   }
 
+  std::function<void(const DiscoveryRequest& request)> check_first_request_;
+  std::function<void(absl::StatusCode)> check_nack_status_code_;
+  std::string debug_label_;
+
   grpc_core::CondVar ads_cond_;
   grpc_core::Mutex ads_mu_;
   bool ads_done_ ABSL_GUARDED_BY(ads_mu_) = false;
@@ -564,7 +588,7 @@ class AdsServiceImpl
       resource_type_response_state_ ABSL_GUARDED_BY(ads_mu_);
   std::set<std::string /*resource_type*/> resource_types_to_ignore_
       ABSL_GUARDED_BY(ads_mu_);
-  std::map<std::string /*resource_type*/, int> resource_type_min_versions_
+  std::function<void(absl::string_view, int)> check_version_callack_
       ABSL_GUARDED_BY(ads_mu_);
   // An instance data member containing the current state of all resources.
   // Note that an entry will exist whenever either of the following is true:
@@ -585,6 +609,9 @@ class LrsServiceImpl
           ::envoy::service::load_stats::v3::LoadReportingService::Service>,
       public std::enable_shared_from_this<LrsServiceImpl> {
  public:
+  using LoadStatsRequest = ::envoy::service::load_stats::v3::LoadStatsRequest;
+  using LoadStatsResponse = ::envoy::service::load_stats::v3::LoadStatsResponse;
+
   // Stats reported by client.
   class ClientStats {
    public:
@@ -686,10 +713,18 @@ class LrsServiceImpl
   };
 
   LrsServiceImpl(int client_load_reporting_interval_seconds,
-                 std::set<std::string> cluster_names)
+                 std::set<std::string> cluster_names,
+                 std::function<void()> stream_started_callback = nullptr,
+                 std::function<void(const LoadStatsRequest& request)>
+                     check_first_request = nullptr,
+                 absl::string_view debug_label = "")
       : client_load_reporting_interval_seconds_(
             client_load_reporting_interval_seconds),
-        cluster_names_(std::move(cluster_names)) {}
+        cluster_names_(std::move(cluster_names)),
+        stream_started_callback_(std::move(stream_started_callback)),
+        check_first_request_(std::move(check_first_request)),
+        debug_label_(absl::StrFormat(
+            "%p%s%s", this, debug_label.empty() ? "" : ":", debug_label)) {}
 
   // Must be called before the LRS call is started.
   void set_send_all_clusters(bool send_all_clusters) {
@@ -710,13 +745,11 @@ class LrsServiceImpl
       absl::Duration timeout = absl::InfiniteDuration());
 
  private:
-  using LoadStatsRequest = ::envoy::service::load_stats::v3::LoadStatsRequest;
-  using LoadStatsResponse = ::envoy::service::load_stats::v3::LoadStatsResponse;
   using Stream = ServerReaderWriter<LoadStatsResponse, LoadStatsRequest>;
 
   Status StreamLoadStats(ServerContext* /*context*/, Stream* stream) override {
-    gpr_log(GPR_INFO, "LRS[%p]: StreamLoadStats starts", this);
-    EXPECT_GT(client_load_reporting_interval_seconds_, 0);
+    LOG(INFO) << "LRS[" << debug_label_ << "]: StreamLoadStats starts";
+    if (stream_started_callback_ != nullptr) stream_started_callback_();
     // Take a reference of the LrsServiceImpl object, reference will go
     // out of scope after this method exits.
     std::shared_ptr<LrsServiceImpl> lrs_service_impl = shared_from_this();
@@ -724,9 +757,7 @@ class LrsServiceImpl
     LoadStatsRequest request;
     if (stream->Read(&request)) {
       IncreaseRequestCount();
-      // Verify client features.
-      EXPECT_THAT(request.node().client_features(),
-                  ::testing::Contains("envoy.lrs.supports_send_all_clusters"));
+      if (check_first_request_ != nullptr) check_first_request_(request);
       // Send initial response.
       LoadStatsResponse response;
       if (send_all_clusters_) {
@@ -744,8 +775,9 @@ class LrsServiceImpl
       // Wait for report.
       request.Clear();
       while (stream->Read(&request)) {
-        gpr_log(GPR_INFO, "LRS[%p]: received client load report message: %s",
-                this, request.DebugString().c_str());
+        LOG(INFO) << "LRS[" << debug_label_
+                  << "]: received client load report message: "
+                  << request.DebugString();
         std::vector<ClientStats> stats;
         for (const auto& cluster_stats : request.cluster_stats()) {
           stats.emplace_back(cluster_stats);
@@ -762,13 +794,16 @@ class LrsServiceImpl
         lrs_cv_.Wait(&lrs_mu_);
       }
     }
-    gpr_log(GPR_INFO, "LRS[%p]: StreamLoadStats done", this);
+    LOG(INFO) << "LRS[" << debug_label_ << "]: StreamLoadStats done";
     return Status::OK;
   }
 
   const int client_load_reporting_interval_seconds_;
   bool send_all_clusters_ = false;
   std::set<std::string> cluster_names_;
+  std::function<void()> stream_started_callback_;
+  std::function<void(const LoadStatsRequest& request)> check_first_request_;
+  std::string debug_label_;
 
   grpc_core::CondVar lrs_cv_;
   grpc_core::Mutex lrs_mu_;
