@@ -22,6 +22,7 @@
 #include <chrono>
 #include <thread>  // NOLINT
 
+#include "absl/log/log.h"
 #include "absl/strings/escaping.h"
 #include "absl/strings/str_cat.h"
 #include "absl/time/time.h"
@@ -30,6 +31,7 @@
 #include "gtest/gtest.h"
 
 #include <grpc++/grpc++.h>
+#include <grpcpp/opencensus.h>
 #include <grpcpp/support/status.h>
 
 #include "src/core/ext/filters/logging/logging_filter.h"
@@ -37,8 +39,8 @@
 #include "src/cpp/ext/gcp/observability_logging_sink.h"
 #include "src/proto/grpc/testing/echo.grpc.pb.h"
 #include "src/proto/grpc/testing/echo_messages.pb.h"
-#include "test/core/util/port.h"
-#include "test/core/util/test_config.h"
+#include "test/core/test_util/port.h"
+#include "test/core/test_util/test_config.h"
 #include "test/cpp/end2end/test_service_impl.h"
 
 namespace grpc {
@@ -57,10 +59,10 @@ class TestLoggingSink : public grpc_core::LoggingSink {
     grpc::internal::EntryToJsonStructProto(entry, &json);
     std::string output;
     ::google::protobuf::TextFormat::PrintToString(json, &output);
-    gpr_log(GPR_INFO, "%s", output.c_str());
-    gpr_log(GPR_INFO, "trace_id: %s", entry.trace_id.c_str());
-    gpr_log(GPR_INFO, "span_id: %s", entry.span_id.c_str());
-    gpr_log(GPR_INFO, "is_sampled: %d", entry.is_sampled);
+    LOG(INFO) << output;
+    LOG(INFO) << "trace_id: " << entry.trace_id;
+    LOG(INFO) << "span_id: " << entry.span_id;
+    LOG(INFO) << "is_sampled: " << entry.is_sampled;
     grpc_core::MutexLock lock(&mu_);
     entries_.push_back(std::move(entry));
     cv_.SignalAll();
