@@ -22,8 +22,8 @@
 #include "google/protobuf/text_format.h"
 #include "gtest/gtest.h"
 
-#include "src/core/lib/json/json_reader.h"
-#include "test/core/util/test_config.h"
+#include "src/core/util/json/json_reader.h"
+#include "test/core/test_util/test_config.h"
 
 namespace grpc {
 namespace internal {
