@@ -25,11 +25,12 @@
 
 #include <benchmark/benchmark.h>
 
+#include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "absl/random/random.h"
 
 #include <grpc/slice.h>
 #include <grpc/support/alloc.h>
-#include <grpc/support/log.h>
 
 #include "src/core/ext/transport/chttp2/transport/hpack_encoder.h"
 #include "src/core/ext/transport/chttp2/transport/hpack_parser.h"
@@ -40,7 +41,7 @@
 #include "src/core/lib/slice/slice_string_helpers.h"
 #include "src/core/lib/transport/metadata_batch.h"
 #include "src/core/lib/transport/timeout_encoding.h"
-#include "test/core/util/test_config.h"
+#include "test/core/test_util/test_config.h"
 #include "test/cpp/microbenchmarks/helpers.h"
 #include "test/cpp/util/test_config.h"
 
@@ -74,8 +75,7 @@ static void BM_HpackEncoderEncodeDeadline(benchmark::State& state) {
       grpc_core::MemoryAllocator(grpc_core::ResourceQuota::Default()
                                      ->memory_quota()
                                      ->CreateMemoryAllocator("test"));
-  auto arena = grpc_core::MakeScopedArena(1024, &memory_allocator);
-  grpc_metadata_batch b(arena.get());
+  grpc_metadata_batch b;
   b.Set(grpc_core::GrpcTimeoutMetadata(),
         saved_now + grpc_core::Duration::Seconds(30));
 
@@ -106,12 +106,7 @@ static void BM_HpackEncoderEncodeHeader(benchmark::State& state) {
   grpc_core::ExecCtx exec_ctx;
   static bool logged_representative_output = false;
 
-  grpc_core::MemoryAllocator memory_allocator =
-      grpc_core::MemoryAllocator(grpc_core::ResourceQuota::Default()
-                                     ->memory_quota()
-                                     ->CreateMemoryAllocator("test"));
-  auto arena = grpc_core::MakeScopedArena(1024, &memory_allocator);
-  grpc_metadata_batch b(arena.get());
+  grpc_metadata_batch b;
   Fixture::Prepare(&b);
 
   grpc_core::HPackCompressor c;
@@ -134,7 +129,7 @@ static void BM_HpackEncoderEncodeHeader(benchmark::State& state) {
       logged_representative_output = true;
       for (size_t i = 0; i < outbuf.count; i++) {
         char* s = grpc_dump_slice(outbuf.slices[i], GPR_DUMP_HEX);
-        gpr_log(GPR_DEBUG, "%" PRIdPTR ": %s", i, s);
+        VLOG(2) << i << ": " << s;
         gpr_free(s);
       }
     }
@@ -330,7 +325,9 @@ BENCHMARK_TEMPLATE(BM_HpackEncoderEncodeHeader,
 static void BM_HpackParserInitDestroy(benchmark::State& state) {
   grpc_core::ExecCtx exec_ctx;
   for (auto _ : state) {
-    { grpc_core::HPackParser(); }
+    {
+      grpc_core::HPackParser();
+    }
     grpc_core::ExecCtx::Get()->Flush();
   }
 }
@@ -342,14 +339,8 @@ static void BM_HpackParserParseHeader(benchmark::State& state) {
   std::vector<grpc_slice> benchmark_slices = Fixture::GetBenchmarkSlices();
   grpc_core::ExecCtx exec_ctx;
   grpc_core::HPackParser p;
-  const int kArenaSize = 4096 * 4096;
-  grpc_core::MemoryAllocator memory_allocator =
-      grpc_core::MemoryAllocator(grpc_core::ResourceQuota::Default()
-                                     ->memory_quota()
-                                     ->CreateMemoryAllocator("test"));
-  auto* arena = grpc_core::Arena::Create(kArenaSize, &memory_allocator);
   grpc_core::ManualConstructor<grpc_metadata_batch> b;
-  b.Init(arena);
+  b.Init();
   p.BeginFrame(&*b, std::numeric_limits<uint32_t>::max(),
                std::numeric_limits<uint32_t>::max(),
                grpc_core::HPackParser::Boundary::None,
@@ -362,7 +353,7 @@ static void BM_HpackParserParseHeader(benchmark::State& state) {
       auto error =
           p.Parse(slices[i], i == slices.size() - 1, absl::BitGenRef(bitgen),
                   /*call_tracer=*/nullptr);
-      GPR_ASSERT(error.ok());
+      CHECK_OK(error);
     }
   };
   parse_vec(init_slices);
@@ -370,25 +361,11 @@ static void BM_HpackParserParseHeader(benchmark::State& state) {
     b->Clear();
     parse_vec(benchmark_slices);
     grpc_core::ExecCtx::Get()->Flush();
-    // Recreate arena every 4k iterations to avoid oom
-    if (0 == (state.iterations() & 0xfff)) {
-      b.Destroy();
-      arena->Destroy();
-      arena = grpc_core::Arena::Create(kArenaSize, &memory_allocator);
-      b.Init(arena);
-      p.BeginFrame(&*b, std::numeric_limits<uint32_t>::max(),
-                   std::numeric_limits<uint32_t>::max(),
-                   grpc_core::HPackParser::Boundary::None,
-                   grpc_core::HPackParser::Priority::None,
-                   grpc_core::HPackParser::LogInfo{
-                       1, grpc_core::HPackParser::LogInfo::kHeaders, false});
-    }
   }
   // Clean up
   b.Destroy();
   for (auto slice : init_slices) grpc_slice_unref(slice);
   for (auto slice : benchmark_slices) grpc_slice_unref(slice);
-  arena->Destroy();
 }
 
 namespace hpack_parser_fixtures {
@@ -402,13 +379,7 @@ class FromEncoderFixture {
  private:
   static std::vector<grpc_slice> Generate(int iteration) {
     grpc_core::ExecCtx exec_ctx;
-
-    grpc_core::MemoryAllocator memory_allocator =
-        grpc_core::MemoryAllocator(grpc_core::ResourceQuota::Default()
-                                       ->memory_quota()
-                                       ->CreateMemoryAllocator("test"));
-    auto arena = grpc_core::MakeScopedArena(1024, &memory_allocator);
-    grpc_metadata_batch b(arena.get());
+    grpc_metadata_batch b;
     EncoderFixture::Prepare(&b);
 
     grpc_core::HPackCompressor c;
@@ -441,8 +412,8 @@ class FromEncoderFixture {
       i++;
     }
     // Remove the HTTP header.
-    GPR_ASSERT(!out.empty());
-    GPR_ASSERT(GRPC_SLICE_LENGTH(out[0]) > 9);
+    CHECK(!out.empty());
+    CHECK_GT(GRPC_SLICE_LENGTH(out[0]), 9);
     out[0] = grpc_slice_sub_no_ref(out[0], 9, GRPC_SLICE_LENGTH(out[0]));
     return out;
   }
