@@ -110,7 +110,7 @@ tail${i}:
 % endfor
   }
   SeqState(const SeqState& other) noexcept : state(other.state), whence(other.whence) {
-    GPR_ASSERT(state == State::kState0);
+    CHECK(state == State::kState0);
     Construct(&${"prior."*(n-1-i)}current_promise,
             other.${"prior."*(n-1-i)}current_promise);
 % for i in range(0,n-1):
@@ -142,17 +142,19 @@ tail${i}:
     switch (state) {
 % for i in range(0,n-1):
       case State::kState${i}: {
-        if (grpc_trace_promise_primitives.enabled()) {
-          gpr_log(whence.file(), whence.line(), GPR_LOG_SEVERITY_DEBUG, "seq[%p]: begin poll step ${i+1}/${n}", this);
+        if (GRPC_TRACE_FLAG_ENABLED(promise_primitives)) {
+          VLOG(2).AtLocation(whence.file(), whence.line())
+                << "seq[" << this << "]: begin poll step ${i+1}/${n}";
         }
         auto result = ${"prior."*(n-1-i)}current_promise();
         PromiseResult${i}* p = result.value_if_ready();
-        if (grpc_trace_promise_primitives.enabled()) {
-          gpr_log(whence.file(), whence.line(), GPR_LOG_SEVERITY_DEBUG, "seq[%p]: poll step ${i+1}/${n} gets %s", this, 
-                  p != nullptr
+        if (GRPC_TRACE_FLAG_ENABLED(promise_primitives)) {
+          VLOG(2).AtLocation(whence.file(), whence.line())
+                << "seq[" << this << "]: poll step ${i+1}/${n} gets "
+                << (p != nullptr
                     ? (PromiseResultTraits${i}::IsOk(*p)
-                      ? "ready" 
-                      : absl::StrCat("early-error:", PromiseResultTraits${i}::ErrorString(*p)).c_str()) 
+                      ? "ready"
+                      : absl::StrCat("early-error:", PromiseResultTraits${i}::ErrorString(*p)).c_str())
                     : "pending");
         }
         if (p == nullptr) return Pending{};
@@ -169,12 +171,15 @@ tail${i}:
 % endfor
       default:
       case State::kState${n-1}: {
-        if (grpc_trace_promise_primitives.enabled()) {
-          gpr_log(whence.file(), whence.line(), GPR_LOG_SEVERITY_DEBUG, "seq[%p]: begin poll step ${n}/${n}", this);
+        if (GRPC_TRACE_FLAG_ENABLED(promise_primitives)) {
+          VLOG(2).AtLocation(whence.file(), whence.line())
+                << "seq[" << this << "]: begin poll step ${n}/${n}";
         }
         auto result = current_promise();
-        if (grpc_trace_promise_primitives.enabled()) {
-          gpr_log(whence.file(), whence.line(), GPR_LOG_SEVERITY_DEBUG, "seq[%p]: poll step ${n}/${n} gets %s", this, result.ready()? "ready" : "pending");
+        if (GRPC_TRACE_FLAG_ENABLED(promise_primitives)) {
+          VLOG(2).AtLocation(whence.file(), whence.line())
+                << "seq[" << this << "]: poll step ${n}/${n} gets "
+                << (result.ready()? "ready" : "pending");
         }
         auto* p = result.value_if_ready();
         if (p == nullptr) return Pending{};
@@ -189,7 +194,7 @@ front_matter = """
 #ifndef GRPC_SRC_CORE_LIB_PROMISE_DETAIL_SEQ_STATE_H
 #define GRPC_SRC_CORE_LIB_PROMISE_DETAIL_SEQ_STATE_H
 
-// This file is generated by tools/codegen/core/gen_seq.py
+// This file is generated by tools/codegen/core/gen_join.py
 
 #include <grpc/support/port_platform.h>
 
@@ -197,17 +202,17 @@ front_matter = """
 
 #include <utility>
 
+#include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "absl/base/attributes.h"
 #include "absl/strings/str_cat.h"
 
-#include <grpc/support/log.h>
-
+#include "src/core/lib/debug/trace.h"
 #include "src/core/lib/gprpp/construct_destruct.h"
 #include "src/core/lib/gprpp/debug_location.h"
 #include "src/core/lib/promise/detail/promise_factory.h"
 #include "src/core/lib/promise/detail/promise_like.h"
 #include "src/core/lib/promise/poll.h"
-#include "src/core/lib/promise/trace.h"
 
 // A sequence under some traits for some set of callables P, Fs.
 // P should be a promise-like object that yields a value.
@@ -215,8 +220,8 @@ front_matter = """
 // previous step and yield a promise. Note that most of the machinery in
 // PromiseFactory exists to make it possible for those promise-factory-like
 // objects to be anything that's convenient.
-// Traits defines how we move from one step to the next. Traits sets up the 
-// wrapping and escape handling for the sequence. 
+// Traits defines how we move from one step to the next. Traits sets up the
+// wrapping and escape handling for the sequence.
 // Promises return wrapped values that the trait can inspect and unwrap before
 // passing them to the next element of the sequence. The trait can
 // also interpret a wrapped value as an escape value, which terminates
