@@ -44,8 +44,9 @@ EXTERNAL_DEPS = {
     "absl/algorithm/container.h": "absl/algorithm:container",
     "absl/base/attributes.h": "absl/base:core_headers",
     "absl/base/call_once.h": "absl/base",
+    "absl/base/config.h": "absl/base:config",
     # TODO(ctiller) remove this
-    "absl/base/internal/endian.h": "absl/base",
+    "absl/base/internal/endian.h": "absl/base:endian",
     "absl/base/thread_annotations.h": "absl/base:core_headers",
     "absl/container/flat_hash_map.h": "absl/container:flat_hash_map",
     "absl/container/flat_hash_set.h": "absl/container:flat_hash_set",
@@ -63,6 +64,8 @@ EXTERNAL_DEPS = {
     "absl/functional/bind_front.h": "absl/functional:bind_front",
     "absl/functional/function_ref.h": "absl/functional:function_ref",
     "absl/hash/hash.h": "absl/hash",
+    "absl/log/check.h": "absl/log:check",
+    "absl/log/log.h": "absl/log",
     "absl/memory/memory.h": "absl/memory",
     "absl/meta/type_traits.h": "absl/meta:type_traits",
     "absl/numeric/int128.h": "absl/numeric:int128",
@@ -95,7 +98,7 @@ EXTERNAL_DEPS = {
     "absl/types/variant.h": "absl/types:variant",
     "absl/utility/utility.h": "absl/utility",
     "address_sorting/address_sorting.h": "address_sorting",
-    "google/cloud/opentelemetry/resource_detector.h": "google_cloud_cpp:experimental-opentelemetry",
+    "google/cloud/opentelemetry/resource_detector.h": "google_cloud_cpp:opentelemetry",
     "opentelemetry/common/attribute_value.h": "otel/api",
     "opentelemetry/common/key_value_iterable.h": "otel/api",
     "opentelemetry/nostd/function_ref.h": "otel/api",
@@ -160,16 +163,15 @@ EXTERNAL_DEPS = {
     "openssl/x509.h": "libcrypto",
     "openssl/x509v3.h": "libcrypto",
     "re2/re2.h": "re2",
-    "upb/arena.h": "upb_lib",
-    "upb/base/string_view.h": "upb_lib",
-    "upb/collections/map.h": "upb_collections_lib",
-    "upb/def.h": "upb_lib",
-    "upb/json_encode.h": "upb_json_lib",
-    "upb/mem/arena.h": "upb_lib",
-    "upb/text_encode.h": "upb_textformat_lib",
-    "upb/def.hpp": "upb_reflection",
-    "upb/upb.h": "upb_lib",
-    "upb/upb.hpp": "upb_lib",
+    "upb/base/status.hpp": "upb_base_lib",
+    "upb/base/string_view.h": "upb_base_lib",
+    "upb/message/map.h": "upb_message_lib",
+    "upb/reflection/def.h": "upb_reflection",
+    "upb/json/encode.h": "upb_json_lib",
+    "upb/mem/arena.h": "upb_mem_lib",
+    "upb/mem/arena.hpp": "upb_mem_lib",
+    "upb/text/encode.h": "upb_textformat_lib",
+    "upb/reflection/def.hpp": "upb_reflection",
     "xxhash.h": "xxhash",
     "zlib.h": "madler_zlib",
 }
@@ -260,40 +262,44 @@ def grpc_cc_library(
     global num_opted_out_cc_libraries
     global parsing_path
     assert parsing_path is not None
-    name = "//%s:%s" % (parsing_path, name)
-    num_cc_libraries += 1
-    if select_deps or "nofixdeps" in tags:
-        if args.whats_left and not select_deps and "nofixdeps" not in tags:
-            num_opted_out_cc_libraries += 1
-            print("Not opted in: {}".format(name))
-        no_update.add(name)
-    scores[name] = len(public_hdrs + hdrs)
-    # avoid_dep is the internal way of saying prefer something else
-    # we add grpc_avoid_dep to allow internal grpc-only stuff to avoid each
-    # other, whilst not biasing dependent projects
-    if "avoid_dep" in tags or "grpc_avoid_dep" in tags:
-        avoidness[name] += 10
-    if proto:
-        proto_hdr = "%s%s" % (
-            (parsing_path + "/" if parsing_path else ""),
-            proto.replace(".proto", ".pb.h"),
-        )
-        skip_headers[name].add(proto_hdr)
+    try:
+        name = "//%s:%s" % (parsing_path, name)
+        num_cc_libraries += 1
+        if select_deps or "nofixdeps" in tags:
+            if args.whats_left and not select_deps and "nofixdeps" not in tags:
+                num_opted_out_cc_libraries += 1
+                print("Not opted in: {}".format(name))
+            no_update.add(name)
+        scores[name] = len(public_hdrs + hdrs)
+        # avoid_dep is the internal way of saying prefer something else
+        # we add grpc_avoid_dep to allow internal grpc-only stuff to avoid each
+        # other, whilst not biasing dependent projects
+        if "avoid_dep" in tags or "grpc_avoid_dep" in tags:
+            avoidness[name] += 10
+        if proto:
+            proto_hdr = "%s%s" % (
+                (parsing_path + "/" if parsing_path else ""),
+                proto.replace(".proto", ".pb.h"),
+            )
+            skip_headers[name].add(proto_hdr)
 
-    for hdr in hdrs + public_hdrs:
-        vendors[_get_filename(hdr, parsing_path)].append(name)
-    inc = set()
-    original_deps[name] = frozenset(deps)
-    original_external_deps[name] = frozenset(external_deps)
-    for src in hdrs + public_hdrs + srcs:
-        for line in open(_get_filename(src, parsing_path)):
-            m = re.search(r"^#include <(.*)>", line)
-            if m:
-                inc.add(m.group(1))
-            m = re.search(r'^#include "(.*)"', line)
-            if m:
-                inc.add(m.group(1))
-    consumes[name] = list(inc)
+        for hdr in hdrs + public_hdrs:
+            vendors[_get_filename(hdr, parsing_path)].append(name)
+        inc = set()
+        original_deps[name] = frozenset(deps)
+        original_external_deps[name] = frozenset(external_deps)
+        for src in hdrs + public_hdrs + srcs:
+            for line in open(_get_filename(src, parsing_path)):
+                m = re.search(r"^#include <(.*)>", line)
+                if m:
+                    inc.add(m.group(1))
+                m = re.search(r'^#include "(.*)"', line)
+                if m:
+                    inc.add(m.group(1))
+        consumes[name] = list(inc)
+    except:
+        print("Error while parsing ", name)
+        raise
 
 
 def grpc_proto_library(name, srcs, **kwargs):
@@ -359,9 +365,7 @@ SCORERS = {
 }
 
 parser = argparse.ArgumentParser(description="Fix build dependencies")
-parser.add_argument(
-    "targets", nargs="*", default=[], help="targets to fix (empty => all)"
-)
+parser.add_argument("targets", nargs="+", help="targets to fix")
 parser.add_argument(
     "--score",
     type=str,
@@ -395,17 +399,22 @@ for dirname in [
     "src/cpp/ext/csm",
     "src/cpp/ext/otel",
     "test/core/backoff",
+    "test/core/call",
+    "test/core/call/yodel",
+    "test/core/client_channel",
     "test/core/experiments",
     "test/core/uri",
-    "test/core/util",
+    "test/core/test_util",
     "test/core/end2end",
     "test/core/event_engine",
     "test/core/filters",
     "test/core/promise",
     "test/core/resource_quota",
     "test/core/transport/chaotic_good",
+    "test/core/transport/test_suite",
     "fuzztest",
     "fuzztest/core/channel",
+    "fuzztest/core/transport/chttp2",
 ]:
     parsing_path = dirname
     exec(
@@ -423,12 +432,16 @@ for dirname in [
             "grpc_cc_library": grpc_cc_library,
             "grpc_cc_test": grpc_cc_library,
             "grpc_core_end2end_test": lambda **kwargs: None,
+            "grpc_filegroup": lambda **kwargs: None,
+            "grpc_transport_test": lambda **kwargs: None,
+            "grpc_yodel_test": lambda **kwargs: None,
+            "grpc_yodel_simple_test": lambda **kwargs: None,
             "grpc_fuzzer": grpc_cc_library,
             "grpc_fuzz_test": grpc_cc_library,
             "grpc_proto_fuzzer": grpc_cc_library,
             "grpc_proto_library": grpc_proto_library,
             "select": lambda d: d["//conditions:default"],
-            "glob": lambda files: None,
+            "glob": lambda files, **kwargs: None,
             "grpc_end2end_tests": lambda: None,
             "grpc_upb_proto_library": lambda name, **kwargs: None,
             "grpc_upb_proto_reflection_library": lambda name, **kwargs: None,
@@ -437,6 +450,7 @@ for dirname in [
             "grpc_package": lambda **kwargs: None,
             "filegroup": lambda name, **kwargs: None,
             "sh_library": lambda name, **kwargs: None,
+            "platform": lambda name, **kwargs: None,
         },
         {},
     )
@@ -541,9 +555,11 @@ def make_library(library):
     # once EventEngine lands we can clean this up
     deps = Choices(
         library,
-        {"//:grpc_base": ["//:grpc", "//:grpc_unsecure"]}
-        if library.startswith("//test/")
-        else {},
+        (
+            {"//:grpc_base": ["//:grpc", "//:grpc_unsecure"]}
+            if library.startswith("//test/")
+            else {}
+        ),
     )
     external_deps = Choices(None, {})
     for hdr in hdrs:
@@ -655,14 +671,28 @@ def make_library(library):
     return (library, error, deps, external_deps)
 
 
+def matches_target(library, target):
+    if not target.startswith("//"):
+        if "/" in target:
+            target = "//" + target
+        else:
+            target = "//:" + target
+    if target == "..." or target == "//...":
+        return True
+    if target.endswith("/..."):
+        return library.startswith(target[:-4])
+    return library == target
+
+
 def main() -> None:
     update_libraries = []
     for library in sorted(consumes.keys()):
         if library in no_update:
             continue
-        if args.targets and library not in args.targets:
-            continue
-        update_libraries.append(library)
+        for target in args.targets:
+            if matches_target(library, target):
+                update_libraries.append(library)
+                break
     with multiprocessing.Pool(processes=multiprocessing.cpu_count()) as p:
         updated_libraries = p.map(make_library, update_libraries, 1)
 
