@@ -39,15 +39,53 @@ echo "2) some change that was just merged in the submodule head has caused the f
 
 echo ""
 echo "submodule '${SUBMODULE_NAME}' is at commit: $(cd third_party/${SUBMODULE_NAME}; git rev-parse --verify HEAD)"
+echo ""
 
-tools/buildgen/generate_projects.sh
+# Update bazel for generate_projects
+case "$SUBMODULE_NAME" in
+  abseil-cpp)
+    BAZEL_DEP_NAME="com_google_absl"
+    ;;
+  boringssl)
+    BAZEL_DEP_NAME="boringssl"
+    ;;
+  protobuf)
+    BAZEL_DEP_NAME="com_google_protobuf"
+    ;;
+esac
+if [ -z "$BAZEL_DEP_NAME" ]
+then
+   echo "No bazel dependency is specified so skipping bazel reconfiguration."
+else
+   BAZEL_DEP_PATH="$(pwd)/third_party/${SUBMODULE_NAME}"
+   echo "bazel override_repository is set for ${BAZEL_DEP_NAME} to ${BAZEL_DEP_PATH}"
+   echo "build --override_repository=${BAZEL_DEP_NAME}=${BAZEL_DEP_PATH}" >> "tools/bazel.rc"
+   echo "query --override_repository=${BAZEL_DEP_NAME}=${BAZEL_DEP_PATH}" >> "tools/bazel.rc"
+fi
+echo ""
+
+if [ "${SUBMODULE_NAME}" == "abseil-cpp" ]
+then
+  src/abseil-cpp/preprocessed_builds.yaml.gen.py
+fi
 
 if [ "${SUBMODULE_NAME}" == "protobuf" ]
 then
-  tools/distrib/python/make_grpcio_tools.py
+  # update upb
+  rm -rf third_party/upb/upb
+  cp -r third_party/protobuf/upb third_party/upb
+  # generate upb gen source codes
+  export CC=gcc
+  tools/codegen/core/gen_upb_api.sh
+  # update utf8_range
+  rm -rf third_party/utf8_range
+  cp -r third_party/protobuf/third_party/utf8_range third_party/utf8_range/
 fi
 
-# commit so that changes are passed to Docker
-git -c user.name='foo' -c user.email='foo@google.com' commit -a -m 'Update submodule' --allow-empty
+tools/buildgen/generate_projects.sh
 
-tools/run_tests/run_tests_matrix.py -f linux --inner_jobs 4 -j 4 --internal_ci --build_only
+# commit so that changes are passed to Docker
+git add -A
+git -c user.name='foo' -c user.email='foo@google.com' commit -m 'Update submodule' --allow-empty
+
+tools/run_tests/run_tests_matrix.py -f linux --exclude c sanity basictests_arm64 openssl dbg --inner_jobs 16 -j 2 --internal_ci --build_only
